@@ -14,11 +14,13 @@ import re
 from pyparsing import Word, QuotedString, Literal, Group, Empty, StringEnd, ParseException
 from pyparsing import alphas, alphanums
 
+from annalist.util                  import extract_entity_id
+
 from annalist.models.recordtypedata import RecordTypeData
 from annalist.models.entitytypeinfo import EntityTypeInfo, get_built_in_type_ids
 
 #   -------------------------------------------------------------------
-#   Auxilliary fiunctions
+#   Auxilliary functions
 #   -------------------------------------------------------------------
 
 def order_entity_key(entity):
@@ -43,7 +45,7 @@ def order_entity_key(entity):
 
 class EntityFinder(object):
     """
-    Logic for enumerting entities matching a supplied type, selector and/or search string.
+    Logic for enumerating entities matching a supplied type, selector and/or search string.
     """
 
     def __init__(self, coll, selector=None):
@@ -53,7 +55,8 @@ class EntityFinder(object):
         super(EntityFinder, self).__init__()
         self._coll     = coll
         self._site     = coll.get_site()
-        self._selector = EntitySelector(selector)
+        self._selector = EntitySelector(selector, FieldComparison(coll))
+        # self._subtypes = None
         return
 
     def get_collection_type_ids(self):
@@ -68,41 +71,102 @@ class EntityFinder(object):
             yield t
         return
 
-    def get_type_entities(self, type_id, user_permissions, scope):
+    def get_collection_subtypes(self, type_id):
+        """
+        Returns a iterator of `entitytypeinfo` objects for all subtypes
+        of the supplied type in the current collection, including the 
+        identified type itself.
+        """
+        supertypeinfo = EntityTypeInfo(self._site, self._coll, type_id)
+        supertypeuri  = supertypeinfo.get_type_uri()
+        return self.get_collection_uri_subtypes(supertypeuri)
+
+    def get_collection_uri_subtypes(self, type_uri):
+        """
+        Returns a iterator of `entitytypeinfo` objects for all subtypes
+        of the supplied type in the current collection, including the 
+        identified type itself.
+        """
+        if type_uri is not None:
+            for tid in self.get_collection_type_ids():
+                tinfo = EntityTypeInfo(self._site, self._coll, tid)
+                if tinfo and (type_uri in tinfo.get_all_type_uris()):
+                    # log.info(
+                    #     "supertype %s, yield %s: %s"%
+                    #     (type_uri, tinfo.get_type_id(), tinfo.get_type_uri())
+                    #     )
+                    yield tinfo
+        return 
+
+    def get_type_entities(self, type_id, user_permissions, altscope):
         """
         Iterate over entities from collection matching the supplied type.
 
-        'scope' is used to determine the extend of data top be included in the listing:
-        a value of 'all' means that site-wide entyioties are icnluded in the listing.
+        'altscope' is used to determine the extent of data to be included in the listing:
+        a value of 'all' means that site-wide entyities are icnluded in the listing.
         Otherwise only collection entities are included.        
         """
+        #@@
+        # log.info("get_type_entities: type_id %s"%type_id)
+        #@@
         entitytypeinfo = EntityTypeInfo(self._site, self._coll, type_id)
-        include_sitedata = (scope == "all")
-        for e in entitytypeinfo.enum_entities(user_permissions, usealtparent=include_sitedata):
-            yield e
-        return
-
-    def get_all_types_entities(self, types, user_permissions, scope):
-        """
-        Iterate mover all entities of all types from a supplied type iterator
-        """
-        assert user_permissions is not None
-        for t in types:
-            for e in self.get_type_entities(t, user_permissions, scope):
+        for e in entitytypeinfo.enum_entities_with_inferred_values(
+                user_permissions, altscope=altscope
+                ):
+            if e.get_id() != "_initial_values":
                 yield e
         return
 
-    def get_base_entities(self, type_id=None, user_permissions=None, scope=None):
+    def get_subtype_entities(self, type_id, user_permissions, altscope):
+        """
+        Iterate over entities from collection that are of the indicated type
+        or any of its subtypes.
+
+        'altscope' is used to determine the extent of data to be included in the listing:
+        a value of 'all' means that site-wide entities are included in the listing.
+        Otherwise only collection entities are included.        
+        """
+        # log.info("get_subtype_entities: type_id %s"%type_id)
+        for entitytypeinfo in self.get_collection_subtypes(type_id):
+            for e in entitytypeinfo.enum_entities_with_inferred_values(
+                    user_permissions, altscope=altscope
+                    ):
+                # log.info(
+                #     "get_subtype_entities type_id %s, yield %s/%s"%
+                #     (type_id, e.get_type_id(), e.get_id())
+                #     )
+                if e.get_id() != "_initial_values":
+                    yield e
+        return
+
+    def get_all_types_entities(self, types, user_permissions, altscope):
+        """
+        Iterate mover all entities of all type ids from a supplied type iterator
+        """
+        assert user_permissions is not None
+        for t in types:
+            for e in self.get_type_entities(t, user_permissions, altscope):
+                yield e
+        return
+
+    def get_base_entities(self, type_id=None, user_permissions=None, altscope=None):
         """
         Iterate over base entities from collection, matching the supplied type id if supplied.
 
         If a type_id is supplied, site data values are included.
         """
         if type_id:
-            return self.get_type_entities(type_id, user_permissions, scope)
+            #@@
+            # log.info("get_base_entities: type_id %s"%type_id)
+            #@@
+            return self.get_subtype_entities(type_id, user_permissions, altscope)
+            # return self.get_type_entities(type_id, user_permissions, scope)
         else:
+            #@@
+            # log.info("get_base_entities: all types")
+            #@@
             return self.get_all_types_entities(
-                self.get_collection_type_ids(), user_permissions, scope
+                self.get_collection_type_ids(), user_permissions, altscope
                 )
         return
 
@@ -173,13 +237,13 @@ class EntitySelector(object):
     """
     This class implements a selector filter.  It is initialized with a selector
     expression, and may be invoked as a filter applied to an entity generator,
-    or as a predicater applied to a single entity.
+    or as a predicate applied to a single entity.
 
     >>> e  = { 'p:a': '1', 'p:b': '2', 'p:c': '3', '@type': ["http://example.com/type", "foo:bar"] }
     >>> c  = { 'view': { 'v:a': '1', 'v:b': ['2', '3'] } }
     >>> f1 = "'1' == [p:a]"
     >>> f2 = "[p:a]=='2'"
-    >>> f3 = ''
+    >>> f3 = ""
     >>> f4 = "'http://example.com/type' in [@type]"
     >>> f5 = "'foo:bar' in [@type]"
     >>> f6 = "'bar:foo' in [@type]"
@@ -214,9 +278,10 @@ class EntitySelector(object):
     >>> EntitySelector(f12).select_entity(e, c)
     False
     """
-    def __init__(self, selector):
+    def __init__(self, selector, fieldcomp=None):
+        self._fieldcomp = fieldcomp
         # Returns None if no filter is applied, otherwise a predcicate function
-        self._selector = self.compile_selector_filter(selector)
+        self._selector  = self.compile_selector_filter(selector)
         return
 
     def filter(self, entities, context=None):
@@ -261,6 +326,8 @@ class EntitySelector(object):
             <val1> == <val2>            values are same
             <val1> in <val2>            second value is list containing 1st value, 
                                         or values are same, or val1 is None.
+            <val1> <name> <val2>        invoke comparison method from supplied 
+                                        FieldComparison object
 
             <val1> and <val2> may be:
 
@@ -278,6 +345,8 @@ class EntitySelector(object):
            gen-delims    = ":" / "/" / "?" / "#" / "[" / "]" / "@"
            sub-delims    = "!" / "$" / "&" / "'" / "(" / ")"
                          / "*" / "+" / "," / ";" / "="
+
+        Parser uses pyparsing combinators (cf. http://pyparsing.wikispaces.com).
         """
         def get_value(val_list):
             if len(val_list) == 1:
@@ -296,7 +365,7 @@ class EntitySelector(object):
                      | Group( QuotedString("'", "\\") )
                      | Group( p_id )
                      )
-        p_comp     = ( Literal("==") | Literal("in") )
+        p_comp     = ( Literal("==") | Literal("in") | p_name )
         p_selector = ( p_val + p_comp + p_val + StringEnd() )
         try:
             resultlist = p_selector.parseString(selector).asList()
@@ -309,25 +378,15 @@ class EntitySelector(object):
             resultdict['val2'] = get_value(resultlist[2])
         return resultdict
 
-    @classmethod  #@@TODO: @staticmethod, no cls?
-    def compile_selector_filter(cls, selector):
+    def compile_selector_filter(self, selector):
         """
         Return filter for for testing entities matching a supplied selector.
 
         Returns None if no selection is performed; i.e. all possible entities are selected.
 
-        Selector formats:
-            ALL (or blank)              match any entity
-            <val1> == <val2>            values are same
-            <val1> in <val2>            second value is list containing 1st value, 
-                                        or values are same, or val1 is None.
+        Selector formats: see `parse_selector` above.
 
-            <val1> and <val2> may be:
-
-            [<field-id>]                refers to field in entity under test
-            <name>[<field-id>]          refers to field of context value, or None if the
-                                        indicated context value or field is not defined.
-            "<string>"                  literal string value.  Quotes within are escaped.
+        This function returns a filter function compiled from the supplied selector.
         """
         def get_entity(field_id):
             "Get field from entity tested by filter"
@@ -338,7 +397,6 @@ class EntitySelector(object):
         def get_context(name, field_id):
             "Get field from named value in current display context"
             def get_context_f(e, c):
-                # Raises error if context value not supplied
                 if name in c and c[name]:
                     return c[name].get(field_id, None)
                 return None
@@ -376,9 +434,14 @@ class EntitySelector(object):
                 return v1 == v2
             return match_in_f
         #
+        def match_subtype(v1f, v2f):
+            def match_subtype_f(e, c):
+                return self._fieldcomp.subtype(v1f(e, c), v2f(e, c))
+            return match_subtype_f
+        #
         if selector in {None, "", "ALL"}:
             return None
-        sel = cls.parse_selector(selector)
+        sel = self.parse_selector(selector)
         if not sel:
             raise ValueError("Unrecognized selector syntax (%s)"%selector)
         v1f = get_val_f(sel['val1'])
@@ -387,9 +450,53 @@ class EntitySelector(object):
             return match_eq(v1f, v2f)
         if sel['comp'] == "in":
             return match_in(v1f, v2f)
+        if sel['comp'] == "subtype":
+            return match_subtype(v1f, v2f)
         # Drop through: raise error
         raise ValueError("Unrecognized entity selector (%s)"%selector)
 
+#   -------------------------------------------------------------------
+#   FieldComparison
+#   -------------------------------------------------------------------
+
+class FieldComparison(object):
+    """
+    Logic for comparing fields using additional context information not available
+    directly to 'EntitySelector'
+    """
+
+    def __init__(self, coll):
+        super(FieldComparison, self).__init__()
+        self._coll     = coll
+        self._site     = coll.get_site()
+        return
+
+    def get_uri_type_info(self, type_uri):
+        """
+        Return typeinfo corresponding to the supplied type URI
+        """
+        t     = self._coll.get_uri_type(type_uri)
+        return t and EntityTypeInfo(self._site, self._coll, t.get_id())
+
+    def subtype(self, type1_uri, type2_uri):
+        """
+        Returns True if the first type is a subtype of the second type, where both
+        types are supplied as type URIs.  Returns True if both URIs are the same.
+
+        If type1_uri is not specified, assume no restriction.
+
+        If type2_uri is not specified, assume it does not satisfy the restriction.
+        """
+        # log.info("FieldComparison.subtype(%s, %s)"%(type1_uri, type2_uri))
+        if not type2_uri or (type1_uri == type2_uri):
+            return True
+        if not type1_uri:
+            return False
+        type1_info = self.get_uri_type_info(type1_uri)
+        type1_supertype_uris = (type1_info and type1_info.get_all_type_uris()) or []
+        # log.info("FieldComparison.subtype: type1_uris (supertypes) %r"%(type1_uris,))
+        return type2_uri in type1_supertype_uris
+
 if __name__ == "__main__":
     import doctest
     doctest.testmod()
