@@ -17,7 +17,7 @@ from django.http                import QueryDict
 from django.utils.http          import urlquote, urlunquote
 from django.core.urlresolvers   import resolve, reverse
 
-from annalist.util              import valid_id
+from annalist.util              import valid_id, extract_entity_id
 from annalist.identifiers       import RDF, RDFS, ANNAL
 from annalist                   import layout
 from annalist                   import message
@@ -69,6 +69,16 @@ def entity_url(coll_id="testcoll", type_id="testtype", entity_id="entity_id"):
         entity_id = "___"
     return collection_entity_view_url(coll_id=coll_id, type_id=type_id, entity_id=entity_id)
 
+def entity_resource_url(
+    coll_id="testcoll", type_id="testtype", entity_id="entity_id", 
+    resource_ref=layout.ENTITY_DATA_FILE
+    ):
+    """
+    URI for entity data; also view using default entity view
+    """
+    base = entity_url(coll_id, type_id, entity_id)
+    return urlparse.urljoin(base, resource_ref)
+
 def entitydata_edit_url(action=None, coll_id="testcoll", type_id=None, entity_id=None, view_id="Default_view"):
     viewname = ( 
         'AnnalistEntityNewView'             if action == "new" else
@@ -139,22 +149,31 @@ def entitydata_value_keys(entity_uri=False):
     keys = (
         [ '@type'
         , 'annal:id', 'annal:type_id'
-        , 'annal:type', 'annal:url'
+        , 'annal:type'
+        , 'annal:url'
         , 'rdfs:label', 'rdfs:comment'
         ])
     if entity_uri:
         keys.add('annal:uri')
     return keys
 
+def entitydata_load_keys(entity_uri=False):
+    return (
+        recordview_value_keys(entity_uri=entity_uri) | 
+        {"@id", '@type', '@context'}
+        )
+
 def entitydata_create_values(
         entity_id, update="Entity", coll_id="testcoll", type_id="testtype", 
-        entity_uri=None, typeuri=None, hosturi=TestHostUri):
+        entity_uri=None, type_uri=None, hosturi=TestHostUri,
+        extra_fields=None):
     """
     Data used when creating entity test data
     """
-    if typeuri is None:
-        typeuri = entity_url(coll_id, "_type", type_id)
-    types   = [entitydata_type(type_id), typeuri]
+    if type_uri is not None:
+        types = [type_uri, type_uri+"/super1", type_uri+"/super2", entitydata_type(type_id)]
+    else:
+        types = [entity_url(coll_id, "_type", type_id), entitydata_type(type_id)]
     # log.info('entitydata_create_values: types %r'%(types,)) 
     d = (
         { '@type':          types
@@ -164,6 +183,8 @@ def entitydata_create_values(
         })
     if entity_uri:
         d['annal:uri'] = entity_uri
+    if extra_fields:
+        d.update(extra_fields)
     return d
 
 def entitydata_values_add_field(data, property_uri, dup_index, value):
@@ -183,15 +204,18 @@ def entitydata_values(
         entity_id, update="Entity", 
         coll_id="testcoll", type_id="testtype", 
         entity_uri=None,
-        typeuri=None, hosturi=TestHostUri):
-    typeuri = entity_url(coll_id, "_type", type_id)
+        type_uri=None, hosturi=TestHostUri
+        ):
+    # type_uri = entity_url(coll_id, "_type", type_id)
     dataurl = entity_url(coll_id, type_id, entity_id)
     d = entitydata_create_values(
         entity_id, update=update, coll_id=coll_id, type_id=type_id, 
-        entity_uri=entity_uri, typeuri=typeuri, hosturi=hosturi
+        entity_uri=entity_uri, type_uri=type_uri, hosturi=hosturi
         ).copy() #@@ copy needed here?
     d.update(
         { '@id':            './'
+        # , '@base':          "../.."
+        , '@context':       ["../../coll_context.jsonld"]
         , 'annal:id':       entity_id
         , 'annal:type_id':  type_id
         , 'annal:url':      dataurl
@@ -214,7 +238,8 @@ def entitydata_context_data(
             , 'field_name':         'entity_id'
             , 'field_render_type':  'EntityId'
             , 'field_placement':    get_placement_classes('small:0,12;medium:0,6')
-            , 'field_value_type':   'annal:Slug'
+            , 'field_value_mode':   'Value_direct'
+            , 'field_target_type':  "annal:Slug"
             # , 'field_value':      (Supplied separately)
             , 'options':            []
             }
@@ -222,8 +247,9 @@ def entitydata_context_data(
             , 'field_id':           'Entity_type'
             , 'field_name':         'entity_type'
             , 'field_render_type':  'EntityTypeId'
+            , 'field_target_type':  "annal:Slug"
             , 'field_placement':    get_placement_classes('small:0,12;medium:6,6')
-            , 'field_value_type':   'annal:Slug'
+            , 'field_value_mode':   'Value_direct'
             # , 'field_value':      (Supplied separately)
             , 'options':            []
             }
@@ -232,16 +258,18 @@ def entitydata_context_data(
             , 'field_name':         'Entity_label'
             , 'field_render_type':  'Text'
             , 'field_placement':    get_placement_classes('small:0,12')
-            , 'field_value_type':   'annal:Text'
+            , 'field_value_mode':   'Value_direct'
+            , 'field_target_type':  "annal:Text"
             , 'field_value':        '%s data ... (testcoll/testtype)'%(update)
             , 'options':            []
             }
           , { 'field_label':        'Comment'
             , 'field_id':           'Entity_comment'
             , 'field_name':         'Entity_comment'
-            , 'field_render_type':  'Textarea'
+            , 'field_render_type':  'Markdown'
             , 'field_placement':    get_placement_classes('small:0,12')
-            , 'field_value_type':   'annal:Longtext'
+            , 'field_value_mode':   'Value_direct'
+            , 'field_target_type':  "annal:Richtext"
             , 'field_value':        '%s description ... (testcoll/testtype)'%(update)
             , 'options':            []
             }
@@ -250,7 +278,7 @@ def entitydata_context_data(
         })
     if entity_id:
         context_dict['fields'][0]['field_value'] = entity_id
-        context_dict['fields'][1]['field_value'] = type_id if valid_id(type_id) else None
+        context_dict['fields'][1]['field_value'] = "_type/"+type_id if valid_id(type_id) else None
         context_dict['fields'][2]['field_value'] = '%s testcoll/testtype/%s'%(update,entity_id)
         context_dict['fields'][3]['field_value'] = '%s coll testcoll, type testtype, entity %s'%(update,entity_id)
         context_dict['orig_id']     = entity_id
@@ -262,11 +290,12 @@ def entitydata_context_data(
 
 def entitydata_context_add_field(
     context_dict, field_id, dup_index, field_value,
-        field_name= 'Entity_comment',
+        field_name='Entity_comment',
         field_label='Comment',
-        field_render_type= 'Textarea',
+        field_render_type='Markdown',
+        field_value_mode='Value_direct',
+        field_value_type='annal:Richtext',
         field_placement='small:0,12',
-        field_value_type= 'annal:Longtext',
         field_options=[]
         ):
     """
@@ -287,8 +316,9 @@ def entitydata_context_add_field(
         , 'field_name':         field_name+suffix
         , 'field_label':        field_label
         , 'field_render_type':  field_render_type
+        , 'field_value_mode':   field_value_mode
+        , 'field_target_type':  field_value_type
         , 'field_placement':    get_placement_classes(field_placement)
-        , 'field_value_type':   field_value_type
         , 'options':            field_options
         })
     return context_dict
@@ -308,7 +338,7 @@ def entitydata_form_data(
         })
     if entity_id:
         form_data_dict['entity_id']         = entity_id
-        form_data_dict['entity_type']       = type_id
+        form_data_dict['entity_type']       = "_type/"+type_id
         form_data_dict['Entity_label']      = '%s %s/%s/%s'%(update, coll_id, type_id, entity_id)
         form_data_dict['Entity_comment']    = '%s coll %s, type %s, entity %s'%(update, coll_id, type_id, entity_id)
         form_data_dict['orig_id']           = entity_id
@@ -346,7 +376,7 @@ def entitydata_form_add_field(form_data_dict, field_name, dup_index, value):
 
 def entitydata_delete_form_data(entity_id=None, type_id="Default_type", list_id="Default_list"):
     return (
-        { 'list_choice':        list_id
+        { 'list_choice':        "_list/"+list_id
         , 'continuation_url':   ""
         , 'search_for':         ""
         , 'entity_select':      ["%s/%s"%(type_id, entity_id)]
@@ -386,7 +416,8 @@ def entitydata_default_view_context_data(
             , 'field_name':         'entity_id'
             , 'field_placement':    get_placement_classes('small:0,12;medium:0,6')
             , 'field_id':           'Entity_id'
-            , 'field_value_type':   'annal:Slug'
+            , 'field_value_mode':   'Value_direct'
+            , 'field_target_type':  'annal:Slug'
             # , 'field_value':      (Supplied separately)
             , 'options':            []
             }
@@ -394,7 +425,8 @@ def entitydata_default_view_context_data(
             , 'field_name':         'entity_type'
             , 'field_placement':    get_placement_classes('small:0,12;medium:6,6')
             , 'field_id':           'Entity_type'
-            , 'field_value_type':   'annal:Text'
+            , 'field_value_mode':   'Value_direct'
+            , 'field_target_type':  'annal:Text'
             # , 'field_value':      (Supplied separately)
             , 'options':            []
             }
@@ -402,7 +434,8 @@ def entitydata_default_view_context_data(
             , 'field_name':         'Entity_label'
             , 'field_placement':    get_placement_classes('small:0,12')
             , 'field_id':           'Entity_label'
-            , 'field_value_type':   'annal:Text'
+            , 'field_value_mode':   'Value_direct'
+            , 'field_target_type':  'annal:Text'
             , 'field_value':        '%s data ... (testcoll/testtype)'%(update)
             , 'options':            []
             }
@@ -410,7 +443,8 @@ def entitydata_default_view_context_data(
             , 'field_name':         'Entity_comment'
             , 'field_placement':    get_placement_classes('small:0,12')
             , 'field_id':           'Entity_comment'
-            , 'field_value_type':   'annal:Longtext'
+            , 'field_value_mode':   'Value_direct'
+            , 'field_target_type':  'annal:Richtext'
             , 'field_value':        '%s description ... (testcoll/testtype)'%(update)
             , 'options':            []
             }
@@ -434,19 +468,22 @@ def entitydata_default_view_form_data(
         coll_id="testcoll", 
         type_id="testtype", orig_type=None,
         entity_id=None, orig_id=None, 
-        action=None, cancel=None, close=None, edit=None, copy=None, update="Entity",
-        add_view_field=None, use_view=None, 
-        new_view=None, new_field=None, new_type=None, new_enum=None):
+        action=None, cancel=None, close=None, view=None, edit=None, copy=None, 
+        update="Entity",
+        add_view_field=None, use_view=None,
+        new_view=None, new_field=None, new_type=None, 
+        new_enum=None, do_import=None
+        ):
     # log.info("entitydata_default_view_form_data: entity_id %s"%(entity_id))
     form_data_dict = (
-        { 'Entity_label':         '%s data ... (%s/%s)'%(update, coll_id, type_id)
-        , 'Entity_comment':       '%s description ... (%s/%s)'%(update, coll_id, type_id)
+        { 'Entity_label':       '%s data ... (%s/%s)'%(update, coll_id, type_id)
+        , 'Entity_comment':     '%s description ... (%s/%s)'%(update, coll_id, type_id)
         , 'orig_id':            'orig_entity_id'
         , 'continuation_url':   entitydata_list_type_url(coll_id, orig_type or type_id)
         })
     if entity_id:
         form_data_dict['entity_id']       = entity_id
-        form_data_dict['entity_type']     = type_id
+        form_data_dict['entity_type']     = "_type/"+type_id
         form_data_dict['Entity_label']    = '%s %s/%s/%s'%(update, coll_id, type_id, entity_id)
         form_data_dict['Entity_comment']  = '%s coll %s, type %s, entity %s'%(update, coll_id, type_id, entity_id)
         form_data_dict['orig_id']         = entity_id
@@ -461,6 +498,8 @@ def entitydata_default_view_form_data(
         form_data_dict['cancel']          = "Cancel"
     elif close:
         form_data_dict['close']           = "Close"
+    elif view:
+        form_data_dict['view']            = "View"
     elif edit:
         form_data_dict['edit']            = "Edit"
     elif copy:
@@ -478,6 +517,8 @@ def entitydata_default_view_form_data(
         form_data_dict['new_type']        = new_type
     elif new_enum:
         form_data_dict[new_enum]          = new_enum
+    elif do_import:
+        form_data_dict[do_import]         = do_import
     else:
         form_data_dict['save']            = 'Save'
     return form_data_dict
@@ -504,7 +545,8 @@ def entitydata_recordtype_view_context_data(
             , 'field_name':         'entity_id'
             , 'field_placement':    get_placement_classes('small:0,12;medium:0,6')
             , 'field_id':           'Type_id'
-            , 'field_value_type':   'annal:Slug'
+            , 'field_value_mode':   'Value_direct'
+            , 'field_target_type':  'annal:Slug'
             # , 'field_value':      (Supplied separately)
             , 'options':            []
             }
@@ -512,7 +554,8 @@ def entitydata_recordtype_view_context_data(
             , 'field_name':         'Type_label'
             , 'field_placement':    get_placement_classes('small:0,12')
             , 'field_id':           'Type_label'
-            , 'field_value_type':   'annal:Text'
+            , 'field_value_mode':   'Value_direct'
+            , 'field_target_type':  'annal:Text'
             , 'field_value':        '%s data ... (testcoll/testtype)'%(update)
             , 'options':            []
             }
@@ -520,7 +563,8 @@ def entitydata_recordtype_view_context_data(
             , 'field_name':         'Type_comment'
             , 'field_placement':    get_placement_classes('small:0,12')
             , 'field_id':           'Type_comment'
-            , 'field_value_type':   'annal:Longtext'
+            , 'field_value_mode':   'Value_direct'
+            , 'field_target_type':  'annal:Richtext'
             , 'field_value':        '%s description ... (testcoll/testtype)'%(update)
             , 'options':            []
             }
@@ -528,7 +572,8 @@ def entitydata_recordtype_view_context_data(
             , 'field_name':         'Type_uri'
             , 'field_placement':    get_placement_classes('small:0,12')
             , 'field_id':           'Type_uri'
-            , 'field_value_type':   'annal:Identifier'
+            , 'field_value_mode':   'Value_direct'
+            , 'field_target_type':  'annal:Identifier'
             , 'field_value':        ""
             , 'options':            []
             }
@@ -570,10 +615,10 @@ def entitydata_recordtype_view_form_data(
         form_data_dict['Type_uri']        = "" # type_url
         form_data_dict['orig_id']         = entity_id
     if type_id:
-        form_data_dict['entity_type']     = type_id
-        form_data_dict['orig_type']       = type_id
+        form_data_dict['entity_type']     = "_type/"+type_id
+        form_data_dict['orig_type']       = extract_entity_id(type_id)
     if type_uri:
-        form_data_dict['Type_uri']        = type_uri        
+        form_data_dict['Type_uri']        = type_uri
     if orig_id:
         form_data_dict['orig_id']         = orig_id
     if orig_type:
@@ -603,7 +648,7 @@ def entitylist_form_data(action, search="", list_id="Default_list", entities=Non
     list_form_data = (
         { 'search_for':         ""
         , 'search':             "Find"
-        , 'list_choice':        "Default_list"
+        , 'list_choice':        "_list/Default_list"
         , 'list_view':          "View"
         , 'entity_select':      ["{{entity.entity_id}}"]
         , 'new':                "New"
@@ -630,7 +675,7 @@ def entitylist_form_data(action, search="", list_id="Default_list", entities=Non
         continuation_url = "" # collection_edit_url("testcoll")
     form_data = (
         { 'search_for':         search
-        , 'list_choice':        list_id
+        , 'list_choice':        "_list/"+list_id
         , 'continuation_url':   continuation_url
         })
     if entities is not None:
@@ -705,8 +750,8 @@ def layout_classes(width=12):
         class_dict = (
             { 'label_classes':          "view-label small-12 medium-6 columns"
             , 'input_classes':          "view-value small-12 medium-6 columns"
-            , 'col_head_classes':       "view-label col-head small-4 columns"
-            , 'col_item_classes':       "view-value col-???? small-4 columns"
+            , 'col_head_classes':       "view-label col-head small-12 medium-4 columns"
+            , 'col_item_classes':       "view-value col-???? small-12 medium-4 columns"
             , 'button_wide_classes':    "small-4 columns"
             , 'button_left_classes':    "form-buttons small-12 columns"
             , 'button_right_classes':   "form-buttons small-12 columns text-right"
@@ -715,8 +760,8 @@ def layout_classes(width=12):
         class_dict = (
             { 'label_classes':          "view-label small-12 medium-4 columns"
             , 'input_classes':          "view-value small-12 medium-8 columns"
-            , 'col_head_classes':       "view-label col-head small-6 columns"
-            , 'col_item_classes':       "view-value col-???? small-6 columns"
+            , 'col_head_classes':       "view-label col-head small-12 medium-6 columns"
+            , 'col_item_classes':       "small-12 medium-6 columns"
             , 'button_wide_classes':    "small-6 columns"
             , 'button_left_classes':    "form-buttons small-12 columns"
             , 'button_right_classes':   "form-buttons small-12 columns text-right"
