@@ -36,12 +36,10 @@ from annalist.models.annalistuser           import AnnalistUser
 from annalist.views.annalistuserdelete      import AnnalistUserDeleteConfirmedView
 from annalist.views.fields.render_tokenset  import get_field_tokenset_renderer
 
-from AnnalistTestCase                       import AnnalistTestCase
-from tests                                  import init_annalist_test_site
-from tests                                  import (
-    TestHost, TestHostUri, TestBasePath, TestBaseUri, TestBaseDir
-    )
-from entity_testutils                       import (
+from AnnalistTestCase       import AnnalistTestCase
+from tests                  import TestHost, TestHostUri, TestBasePath, TestBaseUri, TestBaseDir
+from init_tests             import init_annalist_test_site, init_annalist_test_coll, resetSitedata
+from entity_testutils       import (
     site_dir, collection_dir,
     site_view_url, collection_edit_url, 
     collection_entity_view_url,
@@ -49,7 +47,7 @@ from entity_testutils                       import (
     create_user_permissions,
     create_test_user
     )
-from entity_testuserdata                    import (
+from entity_testuserdata    import (
     annalistuser_dir,
     annalistuser_site_url, annalistuser_coll_url, annalistuser_url, annalistuser_edit_url,
     annalistuser_value_keys, annalistuser_load_keys,
@@ -57,7 +55,7 @@ from entity_testuserdata                    import (
     annalistuser_view_form_data,
     annalistuser_delete_confirm_form_data
     )
-from entity_testentitydata                  import (
+from entity_testentitydata  import (
     entity_url, entitydata_edit_url, entitydata_list_type_url,
     default_fields, default_label, default_comment, error_label,
     layout_classes
@@ -65,7 +63,7 @@ from entity_testentitydata                  import (
 
 #   -----------------------------------------------------------------------------
 #
-#   AnnalistUser tests
+#   AnnalistUser model tests
 #
 #   -----------------------------------------------------------------------------
 
@@ -82,6 +80,7 @@ class AnnalistUserTest(AnnalistTestCase):
         return
 
     def tearDown(self):
+        resetSitedata()
         return
 
     def test_AnnalistUserTest(self):
@@ -89,11 +88,11 @@ class AnnalistUserTest(AnnalistTestCase):
         return
 
     def test_annalistuser_init(self):
-        usr = AnnalistUser(self.testcoll, "testuser", self.testsite)
+        usr = AnnalistUser(self.testcoll, "testuser")
         url = annalistuser_coll_url(self.testsite, coll_id="testcoll", user_id="testuser")
         self.assertEqual(usr._entitytype,   ANNAL.CURIE.User)
         self.assertEqual(usr._entityfile,   layout.USER_META_FILE)
-        self.assertEqual(usr._entityref,    layout.META_USER_REF)
+        self.assertEqual(usr._entityref,    layout.META_USER_REF%{'id': "testuser"})
         self.assertEqual(usr._entityid,     "testuser")
         self.assertEqual(usr._entityurl,    url)
         self.assertEqual(usr._entitydir,    annalistuser_dir(user_id="testuser"))
@@ -101,7 +100,7 @@ class AnnalistUserTest(AnnalistTestCase):
         return
 
     def test_annalistuser1_data(self):
-        usr = AnnalistUser(self.testcoll, "user1", self.testsite)
+        usr = AnnalistUser(self.testcoll, "user1")
         self.assertEqual(usr.get_id(), "user1")
         self.assertEqual(usr.get_type_id(), "_user")
         self.assertIn("/c/testcoll/_annalist_collection/users/user1/", usr.get_url())
@@ -114,7 +113,7 @@ class AnnalistUserTest(AnnalistTestCase):
         return
 
     def test_annalistuser2_data(self):
-        usr = AnnalistUser(self.testcoll, "user2", self.testsite)
+        usr = AnnalistUser(self.testcoll, "user2")
         self.assertEqual(usr.get_id(), "user2")
         self.assertEqual(usr.get_type_id(), "_user")
         self.assertIn("/c/testcoll/_annalist_collection/users/user2/", usr.get_url())
@@ -130,14 +129,21 @@ class AnnalistUserTest(AnnalistTestCase):
         usr = AnnalistUser.create(self.testcoll, "user1", annalistuser_create_values(user_id="user1"))
         uld = AnnalistUser.load(self.testcoll, "user1").get_values()
         ued = annalistuser_read_values(user_id="user1")
-        self.assertKeysMatch(uld, ued)
-        self.assertDictionaryMatch(uld, ued)
+        self.assertKeysMatch(ued, uld)
+        self.assertDictionaryMatch(ued, uld)
         return
 
     def test_annalistuser_default_data(self):
-        usr = AnnalistUser.load(self.testcoll, "_unknown_user_perms", altparent=self.testsite)
+        usr = AnnalistUser.load(self.testcoll, "_unknown_user_perms", altscope="all")
         self.assertEqual(usr.get_id(), "_unknown_user_perms")
-        self.assertIn("/c/testcoll/_annalist_collection/users/_unknown_user_perms", usr.get_url())
+        self.assertIn(
+            "/c/_annalist_site/_annalist_collection/users/_unknown_user_perms/", 
+            usr.get_url()
+            )
+        self.assertIn(
+            "/c/testcoll/d/_user/_unknown_user_perms", 
+            usr.get_view_url()
+            )
         self.assertEqual(usr.get_type_id(), "_user")
         uld = usr.get_values()
         self.assertEqual(set(uld.keys()), set(annalistuser_load_keys()))
@@ -159,11 +165,7 @@ class AnnalistUserTest(AnnalistTestCase):
 
 class AnnalistUserEditViewTest(AnnalistTestCase):
     """
-    Tests for record user edit view
-
-    This view is generated entirely by generic view code, so opnly the form rendering test is included
-    here to cross-check the form definmition.  Other logic is already tested by the generic form
-    handling tests.
+    Tests Annalist user edit view
     """
 
     def setUp(self):
@@ -183,6 +185,7 @@ class AnnalistUserEditViewTest(AnnalistTestCase):
         return
 
     def tearDown(self):
+        resetSitedata()
         return
 
     #   -----------------------------------------------------------------------------
@@ -236,7 +239,7 @@ class AnnalistUserEditViewTest(AnnalistTestCase):
             , 'field_label':        "Permissions"
             , 'field_placeholder':  "(user permissions)"
             , 'field_name':         "User_permissions"
-            , 'field_value':        ["VIEW", "CREATE", "UPDATE", "DELETE", "CONFIG", "ADMIN"]
+            , 'field_edit_value':   ["VIEW", "CREATE", "UPDATE", "DELETE", "CONFIG", "ADMIN"]
             })
         context  = Context({'field': field})
         rendered = get_field_tokenset_renderer().label_edit().render(context)
@@ -266,7 +269,7 @@ class AnnalistUserEditViewTest(AnnalistTestCase):
             , 'field_label':            "Permissions"
             , 'field_placeholder':      "(user permissions)"
             , 'field_name':             "User_permissions"
-            , 'field_value':            ["VIEW", "CREATE", "UPDATE", "DELETE", "CONFIG", "ADMIN"]
+            , 'field_edit_value':       ["VIEW", "CREATE", "UPDATE", "DELETE", "CONFIG", "ADMIN"]
             , 'field_render_object':    get_field_tokenset_renderer().label_edit()
             })
         template = Template("{% include field.field_render_object %}")
@@ -381,11 +384,12 @@ class AnnalistUserEditViewTest(AnnalistTestCase):
               <div class="row">
                 <div class="%(button_left_classes)s">
                   <input type="submit" name="save"      value="Save" />
+                  <input type="submit" name="view"      value="View" />
                   <input type="submit" name="cancel"    value="Cancel" />
                 </div>
               </div>
             </div>
-            """%field_vals(width=6)
+            """%field_vals(width=4)
         # log.info(r.content)
         self.assertContains(r, formrow1,  html=True)
         self.assertContains(r, formrow2,  html=True)
@@ -475,7 +479,7 @@ class AnnalistUserEditViewTest(AnnalistTestCase):
         self.assertEqual(r.reason_phrase, "FOUND")
         self.assertEqual(r.content,       "")
         self.assertEqual(r['location'], self.continuation_url)
-        # Check that new record type exists
+        # Check that new user exists
         self.assertTrue(AnnalistUser.exists(self.testcoll, "copyuser"))
         self._check_annalist_user_values("copyuser", ["VIEW", "CREATE", "UPDATE", "DELETE"])
         return
@@ -488,7 +492,7 @@ class AnnalistUserEditViewTest(AnnalistTestCase):
 
 class ConfirmAnnalistUserDeleteTests(AnnalistTestCase):
     """
-    Tests for record type deletion on response to confirmation form
+    Tests for Annalist user deletion on response to confirmation form
     """
 
     def setUp(self):
@@ -503,6 +507,7 @@ class ConfirmAnnalistUserDeleteTests(AnnalistTestCase):
         return
 
     def tearDown(self):
+        resetSitedata()
         return
 
     def test_DeleteConfirmedViewTest(self):
