@@ -30,16 +30,16 @@ from annalist.models.recordtype     import RecordType
 from annalist.models.recordtypedata import RecordTypeData
 from annalist.models.entitydata     import EntityData
 
-from tests                          import TestHost, TestHostUri, TestBasePath, TestBaseUri, TestBaseDir
-from tests                          import init_annalist_test_site
-from AnnalistTestCase               import AnnalistTestCase
-from entity_testutils               import (
+from AnnalistTestCase       import AnnalistTestCase
+from tests                  import TestHost, TestHostUri, TestBasePath, TestBaseUri, TestBaseDir
+from init_tests             import init_annalist_test_site, init_annalist_test_coll, resetSitedata
+from entity_testutils       import (
     create_user_permissions,
     collection_new_form_data, collection_remove_form_data,
     context_list_entities,
     context_list_item_fields, context_list_item_field_value
     )
-from entity_testentitydata          import (
+from entity_testentitydata  import (
     # recorddata_dir,  entitydata_dir,
     entity_url, entitydata_edit_url, entitydata_delete_confirm_url,
     entitydata_list_type_url, entitydata_list_all_url,
@@ -48,7 +48,7 @@ from entity_testentitydata          import (
     entitydata_delete_form_data,
     entitydata_delete_confirm_form_data
     )
-from entity_testuserdata            import (
+from entity_testuserdata    import (
     annalistuser_dir,
     annalistuser_site_url, annalistuser_coll_url, annalistuser_url, annalistuser_edit_url,
     annalistuser_value_keys, annalistuser_load_keys,
@@ -57,14 +57,16 @@ from entity_testuserdata            import (
     annalistuser_delete_form_data,
     annalistuser_delete_confirm_form_data
     )
-from entity_testtypedata                import (
+from entity_testcolldata    import (
+    collectiondata_view_form_data
+    )
+from entity_testtypedata    import (
     recordtype_create_values, 
     recordtype_entity_view_form_data,
     recordtype_delete_form_data,
     recordtype_delete_confirm_form_data
     )
 
-
 #   -----------------------------------------------------------------------------
 #
 #   Authorization tests
@@ -78,8 +80,12 @@ class AuthorizationTest(AnnalistTestCase):
 
     def setUp(self):
         init_annalist_test_site()
-        self.testsite    = Site(TestBaseUri, TestBaseDir)
-        self.testcoll    = Collection(self.testsite, "testcoll")
+        init_annalist_test_coll()
+        self.testsite = Site(TestBaseUri, TestBaseDir)
+        self.testcoll = Collection(self.testsite, "testcoll")
+        self.coll1    = Collection(self.testsite, "coll1")
+        self.coll2    = Collection(self.testsite, "coll2")
+        self.coll3    = Collection(self.testsite, "coll3")
         # populate site with different classes of users: admin, config, create, update, delete, view
         self.user_admin  = AnnalistUser.create(self.testcoll, "user_admin", 
             annalistuser_create_values(
@@ -129,53 +135,71 @@ class AuthorizationTest(AnnalistTestCase):
                 user_permissions=["VIEW"]
                 )
             )
+        self.user_delete_coll_coll1 = AnnalistUser.create(self.coll1, "user_delete_coll", 
+            annalistuser_create_values(
+                coll_id="coll1", user_id="user_delete_coll",
+                user_name="Admin User",
+                user_uri="mailto:user_delete_coll@%s"%TestHost, 
+                user_permissions=["VIEW", "DELETE_COLLECTION"]
+                )
+            )
+        self.user_delete_coll_coll3 = AnnalistUser.create(self.coll3, "user_delete_coll", 
+            annalistuser_create_values(
+                coll_id="coll3", user_id="user_delete_coll",
+                user_name="Admin User",
+                user_uri="mailto:user_delete_coll@%s"%TestHost, 
+                user_permissions=["VIEW", "DELETE_COLLECTION"]
+                )
+            )
         self.user_site_admin  = AnnalistUser.create(
-            self.testsite, 
+            self.testsite.site_data_collection(), 
             "user_site_admin", 
             annalistuser_create_values(
                 coll_id="testcoll", user_id="user_site_admin",
                 user_name="Site_admin User",
                 user_uri="mailto:user_site_admin@%s"%TestHost, 
                 user_permissions=["VIEW", "CREATE", "UPDATE", "DELETE", "CONFIG", "ADMIN"]
-                ),
-            use_altpath=True
+                )
             )
         self.user_site_create_coll  = AnnalistUser.create(
-            self.testsite, 
+            self.testsite.site_data_collection(), 
             "user_site_create_coll", 
             annalistuser_create_values(
                 coll_id="testcoll", user_id="user_site_create_coll",
                 user_name="Site_create User",
                 user_uri="mailto:user_site_create_coll@%s"%TestHost, 
                 user_permissions=["VIEW", "CREATE_COLLECTION"]
-                ),
-            use_altpath=True
+                )
             )
         self.user_site_delete_coll  = AnnalistUser.create(
-            self.testsite, 
+            self.testsite.site_data_collection(), 
             "user_site_delete_coll", 
             annalistuser_create_values(
                 coll_id="testcoll", user_id="user_site_delete_coll_coll",
                 user_name="Site_delete User",
                 user_uri="mailto:user_site_delete_coll@%s"%TestHost, 
                 user_permissions=["VIEW", "CREATE_COLLECTION", "DELETE_COLLECTION"]
-                ),
-            use_altpath=True
+                )
             )
         self.user_site_view  = AnnalistUser.create(
-            self.testsite, 
+            self.testsite.site_data_collection(), 
             "user_site_view", 
             annalistuser_create_values(
                 coll_id="testcoll", user_id="user_site_view",
                 user_name="Site_view User",
                 user_uri="mailto:user_site_view@%s"%TestHost, 
                 user_permissions=["VIEW"]
-                ),
-            use_altpath=True
+                )
             )
         return
 
     def tearDown(self):
+        resetSitedata(scope="collections")
+        return
+
+    @classmethod
+    def tearDownClass(cls):
+        resetSitedata()
         return
 
     # Utility functions
@@ -195,23 +219,72 @@ class AuthorizationTest(AnnalistTestCase):
         r = self.client.get(u)
         return r
 
-    # Create collection: requires site-level CREATE permission
+    # ----- Collection access -----
 
     def create_collection(self):
+        # Create collection
         u = reverse("AnnalistSiteView")
         f = collection_new_form_data("testnew")
         r = self.client.post(u, f)
         return r
 
-    # Remove collection: requires site-level DELETE permission
-
     def remove_collection(self):
+        # Remove collections
         u = reverse("AnnalistSiteView")
         f = collection_remove_form_data(["coll1", "coll3"])
         r = self.client.post(u, f)
         return r
 
-    # User access - requires ADMIN permission
+    def view_coll_get(self):
+        u = entitydata_edit_url(action="view", 
+            coll_id="_annalist_site", type_id="_coll", entity_id="testcoll", 
+            view_id="Collection_view"
+            )
+        r = self.client.get(u)
+        return r
+
+    def view_coll_post_edit(self):
+        u = entitydata_edit_url(action="view", 
+            coll_id="_annalist_site", type_id="_coll", entity_id="testcoll", 
+            view_id="Collection_view"
+            )
+        f = collectiondata_view_form_data(action="view",
+            coll_id="testcoll",
+            edit="edit"
+            )
+        r = self.client.post(u, f)
+        return r
+
+    def edit_coll_get(self):
+        u = entitydata_edit_url(action="edit", 
+            coll_id="_annalist_site", type_id="_coll", entity_id="testcoll", 
+            view_id="Collection_view"
+            )
+        r = self.client.get(u)
+        return r
+
+    def edit_coll_post_edit(self):
+        u = entitydata_edit_url(action="edit", 
+            coll_id="_annalist_site", type_id="_coll", entity_id="testcoll", 
+            view_id="Collection_view"
+            )
+        f = collectiondata_view_form_data(action="edit",
+            coll_id="testcoll",
+            edit="edit"
+            )
+        r = self.client.post(u, f)
+        return r
+
+    # Remove site data: should always be refused
+
+    def remove_site_data(self):
+        u = reverse("AnnalistSiteView")
+        f = collection_remove_form_data(["_annalist_site"])
+        with SuppressLogging(logging.WARNING):
+            r = self.client.post(u, f)
+        return r
+
+    # ----- User access - requires ADMIN permission -----
 
     def create_user(self, user_id):
         # Create placeholder for testing
@@ -595,6 +668,11 @@ class AuthorizationTest(AnnalistTestCase):
         self.assertEqual(self.view_site_home_page().status_code,    200)
         self.assertEqual(self.create_collection().status_code,      403)
         self.assertEqual(self.remove_collection().status_code,      403)
+        self.assertEqual(self.view_coll_get().status_code,          200)
+        self.assertEqual(self.view_coll_post_edit().status_code,    403)
+        self.assertEqual(self.edit_coll_get().status_code,          403)
+        self.assertEqual(self.edit_coll_post_edit().status_code,    403)
+        self.assertEqual(self.remove_site_data().status_code,       403)
         #
         self.assertEqual(self.list_users().status_code,             200)
         self.assertEqual(self.view_user_get().status_code,          200)
@@ -633,6 +711,11 @@ class AuthorizationTest(AnnalistTestCase):
         self.assertEqual(self.view_site_home_page().status_code,    200)
         self.assertEqual(self.create_collection().status_code,      403)
         self.assertEqual(self.remove_collection().status_code,      403)
+        self.assertEqual(self.view_coll_get().status_code,          200)
+        self.assertEqual(self.view_coll_post_edit().status_code,    403)
+        self.assertEqual(self.edit_coll_get().status_code,          403)
+        self.assertEqual(self.edit_coll_post_edit().status_code,    403)
+        self.assertEqual(self.remove_site_data().status_code,       403)
         #
         self.assertEqual(self.list_users().status_code,             403)
         self.assertEqual(self.view_user_get().status_code,          403)
@@ -671,6 +754,11 @@ class AuthorizationTest(AnnalistTestCase):
         self.assertEqual(self.view_site_home_page().status_code,    200)
         self.assertEqual(self.create_collection().status_code,      403)
         self.assertEqual(self.remove_collection().status_code,      403)
+        self.assertEqual(self.view_coll_get().status_code,          200)
+        self.assertEqual(self.view_coll_post_edit().status_code,    403)
+        self.assertEqual(self.edit_coll_get().status_code,          403)
+        self.assertEqual(self.edit_coll_post_edit().status_code,    403)
+        self.assertEqual(self.remove_site_data().status_code,       403)
         #
         self.assertEqual(self.list_users().status_code,             403)
         self.assertEqual(self.view_user_get().status_code,          403)
@@ -709,6 +797,11 @@ class AuthorizationTest(AnnalistTestCase):
         self.assertEqual(self.view_site_home_page().status_code,    200)
         self.assertEqual(self.create_collection().status_code,      403)
         self.assertEqual(self.remove_collection().status_code,      403)
+        self.assertEqual(self.view_coll_get().status_code,          200)
+        self.assertEqual(self.view_coll_post_edit().status_code,    403)
+        self.assertEqual(self.edit_coll_get().status_code,          403)
+        self.assertEqual(self.edit_coll_post_edit().status_code,    403)
+        self.assertEqual(self.remove_site_data().status_code,       403)
         #
         self.assertEqual(self.list_users().status_code,             403)
         self.assertEqual(self.view_user_get().status_code,          403)
@@ -747,6 +840,11 @@ class AuthorizationTest(AnnalistTestCase):
         self.assertEqual(self.view_site_home_page().status_code,    200)
         self.assertEqual(self.create_collection().status_code,      403)
         self.assertEqual(self.remove_collection().status_code,      403)
+        self.assertEqual(self.view_coll_get().status_code,          200)
+        self.assertEqual(self.view_coll_post_edit().status_code,    403)
+        self.assertEqual(self.edit_coll_get().status_code,          403)
+        self.assertEqual(self.edit_coll_post_edit().status_code,    403)
+        self.assertEqual(self.remove_site_data().status_code,       403)
         #
         self.assertEqual(self.list_users().status_code,             403)
         self.assertEqual(self.view_user_get().status_code,          403)
@@ -785,6 +883,11 @@ class AuthorizationTest(AnnalistTestCase):
         self.assertEqual(self.view_site_home_page().status_code,    200)
         self.assertEqual(self.create_collection().status_code,      403)
         self.assertEqual(self.remove_collection().status_code,      403)
+        self.assertEqual(self.view_coll_get().status_code,          200)
+        self.assertEqual(self.view_coll_post_edit().status_code,    403)
+        self.assertEqual(self.edit_coll_get().status_code,          403)
+        self.assertEqual(self.edit_coll_post_edit().status_code,    403)
+        self.assertEqual(self.remove_site_data().status_code,       403)
         #
         self.assertEqual(self.list_users().status_code,             403)
         self.assertEqual(self.view_user_get().status_code,          403)
@@ -817,12 +920,58 @@ class AuthorizationTest(AnnalistTestCase):
         self.assertEqual(self.delete_data_confirmed().status_code,  403)
         return
 
+    def test_delete_coll_user(self):
+        self.login_user("user_delete_coll")
+        # try each function, test result
+        self.assertEqual(self.create_collection().status_code,      403)
+        self.assertEqual(self.remove_collection().status_code,      200)
+        self.assertEqual(self.view_coll_get().status_code,          200)
+        self.assertEqual(self.view_coll_post_edit().status_code,    403)
+        self.assertEqual(self.edit_coll_get().status_code,          403)
+        self.assertEqual(self.edit_coll_post_edit().status_code,    403)
+        self.assertEqual(self.remove_site_data().status_code,       403)
+        #
+        self.assertEqual(self.list_users().status_code,             403)
+        self.assertEqual(self.view_user_get().status_code,          403)
+        self.assertEqual(self.view_user_edit().status_code,         403)
+        self.assertEqual(self.new_user().status_code,               403)
+        self.assertEqual(self.copy_user().status_code,              403)
+        self.assertEqual(self.edit_user().status_code,              403)
+        self.assertEqual(self.delete_user().status_code,            403)
+        self.assertEqual(self.delete_user_confirmed().status_code,  403)
+        #
+        self.assertEqual(self.list_types().status_code,             200)
+        self.assertEqual(self.view_type_get().status_code,          200)
+        self.assertEqual(self.view_type_edit().status_code,         403)
+        self.assertEqual(self.view_type_copy().status_code,         403)
+        self.assertEqual(self.new_type().status_code,               403)
+        self.assertEqual(self.copy_type().status_code,              403)
+        self.assertEqual(self.edit_type().status_code,              403)
+        self.assertEqual(self.delete_type().status_code,            403)
+        self.assertEqual(self.delete_type_confirmed().status_code,  403)
+        #
+        self.assertEqual(self.list_data().status_code,              200)
+        self.assertEqual(self.view_data_get().status_code,          200)
+        self.assertEqual(self.view_data_edit().status_code,         403)
+        self.assertEqual(self.view_data_copy().status_code,         403)
+        self.assertEqual(self.new_data().status_code,               403)
+        self.assertEqual(self.copy_data().status_code,              403)
+        self.assertEqual(self.edit_data().status_code,              403)
+        self.assertEqual(self.delete_data().status_code,            403)
+        self.assertEqual(self.delete_data_confirmed().status_code,  403)
+        return
+
     def test_default_user(self):
         self.login_user("other_user")
         # try each function, test result
         self.assertEqual(self.view_site_home_page().status_code,    200)
         self.assertEqual(self.create_collection().status_code,      403)
         self.assertEqual(self.remove_collection().status_code,      403)
+        self.assertEqual(self.view_coll_get().status_code,          200)
+        self.assertEqual(self.view_coll_post_edit().status_code,    403)
+        self.assertEqual(self.edit_coll_get().status_code,          403)
+        self.assertEqual(self.edit_coll_post_edit().status_code,    403)
+        self.assertEqual(self.remove_site_data().status_code,       403)
         #
         self.assertEqual(self.list_users().status_code,             403)
         self.assertEqual(self.view_user_get().status_code,          403)
@@ -860,6 +1009,11 @@ class AuthorizationTest(AnnalistTestCase):
         self.assertEqual(self.view_site_home_page().status_code,    200)
         self.assertEqual(self.create_collection().status_code,      401)
         self.assertEqual(self.remove_collection().status_code,      401)
+        self.assertEqual(self.view_coll_get().status_code,          200)
+        self.assertEqual(self.view_coll_post_edit().status_code,    401)
+        self.assertEqual(self.edit_coll_get().status_code,          401)
+        self.assertEqual(self.edit_coll_post_edit().status_code,    401)
+        self.assertEqual(self.remove_site_data().status_code,       401)
         #
         self.assertEqual(self.list_users().status_code,             401)
         self.assertEqual(self.view_user_get().status_code,          401)
@@ -899,6 +1053,11 @@ class AuthorizationTest(AnnalistTestCase):
         # try each function, test result
         self.assertEqual(self.create_collection().status_code,      302)
         self.assertEqual(self.remove_collection().status_code,      200)
+        self.assertEqual(self.view_coll_get().status_code,          200)
+        self.assertEqual(self.view_coll_post_edit().status_code,    302)
+        self.assertEqual(self.edit_coll_get().status_code,          200)
+        self.assertEqual(self.edit_coll_post_edit().status_code,    302)
+        self.assertEqual(self.remove_site_data().status_code,       403)
         #
         self.assertEqual(self.list_users().status_code,             200)
         self.assertEqual(self.view_user_get().status_code,          200)
@@ -936,6 +1095,11 @@ class AuthorizationTest(AnnalistTestCase):
         # try each function, test result
         self.assertEqual(self.create_collection().status_code,      302)
         self.assertEqual(self.remove_collection().status_code,      403)
+        self.assertEqual(self.view_coll_get().status_code,          200)
+        self.assertEqual(self.view_coll_post_edit().status_code,    403)
+        self.assertEqual(self.edit_coll_get().status_code,          403)
+        self.assertEqual(self.edit_coll_post_edit().status_code,    403)
+        self.assertEqual(self.remove_site_data().status_code,       403)
         #
         self.assertEqual(self.list_users().status_code,             403)
         self.assertEqual(self.view_user_get().status_code,          403)
@@ -973,6 +1137,11 @@ class AuthorizationTest(AnnalistTestCase):
         # try each function, test result
         self.assertEqual(self.create_collection().status_code,      302)
         self.assertEqual(self.remove_collection().status_code,      200)
+        self.assertEqual(self.view_coll_get().status_code,          200)
+        self.assertEqual(self.view_coll_post_edit().status_code,    403)
+        self.assertEqual(self.edit_coll_get().status_code,          403)
+        self.assertEqual(self.edit_coll_post_edit().status_code,    403)
+        self.assertEqual(self.remove_site_data().status_code,       403)
         #
         self.assertEqual(self.list_users().status_code,             403)
         self.assertEqual(self.view_user_get().status_code,          403)
@@ -1009,6 +1178,11 @@ class AuthorizationTest(AnnalistTestCase):
         # try each function, test result
         self.assertEqual(self.create_collection().status_code,      403)
         self.assertEqual(self.remove_collection().status_code,      403)
+        self.assertEqual(self.view_coll_get().status_code,          200)
+        self.assertEqual(self.view_coll_post_edit().status_code,    403)
+        self.assertEqual(self.edit_coll_get().status_code,          403)
+        self.assertEqual(self.edit_coll_post_edit().status_code,    403)
+        self.assertEqual(self.remove_site_data().status_code,       403)
         #
         self.assertEqual(self.list_users().status_code,             403)
         self.assertEqual(self.view_user_get().status_code,          403)
@@ -1054,7 +1228,7 @@ class AuthorizationTest(AnnalistTestCase):
         entities = context_list_entities(r.context)
         self.assertEqual(len(entities), 2)
         entity_fields = (
-            [ ('testtype',    '_type',    'RecordType testtype/testtype')
+            [ ('testtype',    '_type',    'RecordType testcoll/testtype')
             , ('entity1',     'testtype', 'Entity testcoll/testtype/entity1')
             ])
         for eid in range(2):
