@@ -20,9 +20,9 @@ from django.test.client         import Client
 from annalist.identifiers       import ANNAL
 from annalist.models.entity     import EntityRoot, Entity
 
-from tests                      import TestBaseDir, TestBaseUri, TestHost, TestBasePath
-from tests                      import dict_to_str, init_annalist_test_site
-from AnnalistTestCase           import AnnalistTestCase
+from AnnalistTestCase       import AnnalistTestCase
+from tests                  import TestHost, TestHostUri, TestBasePath, TestBaseUri, TestBaseDir
+from init_tests             import init_annalist_test_site, init_annalist_test_coll, resetSitedata
 
 #   -----------------------------------------------------------------------------
 #
@@ -36,6 +36,8 @@ class TestEntityRootType(EntityRoot):
     _entitypath = None
     _entityfile = ".sub/manifest.jsonld"
     _entityref  = "../"
+    _contextref = "../../coll_context.jsonld"
+
 
 class EntityRootTest(TestCase):
     """
@@ -49,30 +51,36 @@ class EntityRootTest(TestCase):
     def tearDown(self):
         return
 
+    @classmethod
+    def tearDownClass(cls):
+        resetSitedata()
+        return
+
     def test_EntityRootTest(self):
         self.assertEqual(EntityRootTest.__name__, "EntityRootTest", "Check class name")
         return
 
     def test_entityroot_init(self):
-        e = EntityRoot(TestBaseUri, TestBaseDir)
+        e = EntityRoot(TestBaseUri, TestBaseUri, TestBaseDir, TestBaseDir)
         self.assertEqual(e._entitytype,     ANNAL.CURIE.EntityRoot)
         self.assertEqual(e._entityfile,     None)
         self.assertEqual(e._entityref,      None)
         self.assertEqual(e._entityid,       None)
         self.assertEqual(e._entityurl,      TestBaseUri+"/")
+        self.assertEqual(e._entityviewurl,  TestBaseUri+"/")
         self.assertEqual(e._entitydir,      TestBaseDir+"/")
         self.assertEqual(e._values,         None)
         return
 
     def test_entityroot_id(self):
-        e = EntityRoot(TestBaseUri, TestBaseDir)
+        e = EntityRoot(TestBaseUri, TestBaseUri, TestBaseDir, TestBaseDir)
         self.assertEqual(e.get_id(),        None)
         e.set_id("testId")
         self.assertEqual(e.get_id(),        "testId")
         return
 
     def test_entityroot_type_id(self):
-        e = EntityRoot(TestBaseUri, TestBaseDir)
+        e = EntityRoot(TestBaseUri, TestBaseUri, TestBaseDir, TestBaseDir)
         self.assertEqual(e.get_type_id(),   None)
         return
 
@@ -105,7 +113,7 @@ class EntityRootTest(TestCase):
             , 'type':           'annal:EntityRoot'
             , 'rdfs:label':     'Name collection coll2'
             })
-        e = EntityRoot(TestBaseUri, TestBaseDir)
+        e = EntityRoot(TestBaseUri, TestBaseUri, TestBaseDir, TestBaseDir)
         self.assertEqual(e.get_values(),    None)
         e.set_values(test_values1)
         self.assertEqual(e.get_values(),    test_values1_returned)
@@ -136,7 +144,7 @@ class EntityRootTest(TestCase):
             , 'type':           'annal:EntityRoot'
             , 'title':          'new title'
             })
-        e = EntityRoot(TestBaseUri, TestBaseDir)
+        e = EntityRoot(TestBaseUri, TestBaseUri, TestBaseDir, TestBaseDir)
         self.assertEqual(e.get_values(),    None)
         e.set_values(test_values1)
         self.assertEqual(set(e.get_values().keys()), set(test_values1_returned.keys()))
@@ -152,7 +160,7 @@ class EntityRootTest(TestCase):
         return
 
     def test_entityroot_subclass(self):
-        e = TestEntityRootType(TestBaseUri, TestBaseDir)
+        e = TestEntityRootType(TestBaseUri, TestBaseUri, TestBaseDir, TestBaseDir)
         self.assertEqual(e._entitytype,     "test:EntityRootType")
         self.assertEqual(e._entityfile,     ".sub/manifest.jsonld")
         self.assertEqual(e._entityref,      "../")
@@ -172,6 +180,8 @@ class EntityRootTest(TestCase):
         test_values_returned = (
             { '@id':            '../'
             , '@type':          ['test:EntityRootType']
+            # , '@base':          "../.."
+            , '@context':       ["../../coll_context.jsonld"]
             , 'annal:id':       'testId'
             , 'annal:type_id':  None
             , 'annal:type':     'test:EntityRootType'
@@ -179,7 +189,7 @@ class EntityRootTest(TestCase):
             , 'title':          'Name collection coll1'
             , 'type':           'annal:EntityRoot'
             })
-        e = TestEntityRootType(TestBaseUri, TestBaseDir)
+        e = TestEntityRootType(TestBaseUri, TestBaseUri, TestBaseDir, TestBaseDir)
         e.set_id("testId")
         self.assertEqual(e._entitytype,     "test:EntityRootType")
         self.assertEqual(e._entityfile,     ".sub/manifest.jsonld")
@@ -190,7 +200,7 @@ class EntityRootTest(TestCase):
         self.assertEqual(e._values,         None)
         e.set_values(test_values)
         e._save()
-        e2 = TestEntityRootType(TestBaseUri, TestBaseDir)
+        e2 = TestEntityRootType(TestBaseUri, TestBaseUri, TestBaseDir, TestBaseDir)
         e2.set_id("testId")
         v2 = e2._load_values()
         self.assertEqual(set(v2.keys()), set(test_values_returned.keys()))
@@ -203,16 +213,7 @@ class EntityRootTest(TestCase):
             , 'title':  'Name collection coll1'
             # , 'uri':    '/annalist/coll1'
             })
-        test_values_returned = (
-            { '@id': '../'
-            , 'annal:id': 'testId'
-            , 'annal:type': 'test:EntityRootType'
-            , 'id': 'testId'
-            , 'title': 'Name collection coll1'
-            , 'type': 'annal:EntityRoot'
-            # , 'uri': '/annalist/coll1'
-            })
-        e = TestEntityRootType(TestBaseUri, TestBaseDir)
+        e = TestEntityRootType(TestBaseUri, TestBaseUri, TestBaseDir, TestBaseDir)
         e.set_id("testId")
         e.set_values(test_values)
         self.assertFalse(e._exists())
@@ -220,6 +221,49 @@ class EntityRootTest(TestCase):
         self.assertTrue(e._exists())
         return
 
+    def test_entityroot_enum_fields(self):
+        test_values = (
+            { "@type": ['test:EntityRootType']
+            , "a": "a"
+            , "b": "b"
+            , "c": 2
+            , "d":
+              [ { "d1a": "d1a"
+                , "d1b": "d1b"
+                }
+              , { "d2a": "d2a"
+                , "d2b": "d2b"
+                }
+              ]
+            , "e": { "ex": "ex", "ey": "ey"}
+            })
+        e = TestEntityRootType(TestBaseUri, TestBaseUri, TestBaseDir, TestBaseDir)
+        e.set_id("testId")
+        e.set_values(test_values)
+        test_fields_returned = (
+            [ (['@type'],           ['test:EntityRootType'])
+            , (['annal:id'],        'testId')
+            , (['annal:url'],       TestBasePath+'/')
+            , (['annal:type_id'],   None)
+            , (['annal:type'],      'test:EntityRootType')
+            , (["a"],               "a")
+            , (["b"],               "b")
+            , (["c"],               2)
+            , (["d",0,"d1a"],       "d1a")
+            , (["d",0,"d1b"],       "d1b")
+            , (["d",1,"d2a"],       "d2a")
+            , (["d",1,"d2b"],       "d2b")
+            , (["e"],               { "ex": "ex", "ey": "ey" })
+            ])
+        actual_fields_returned = list(e.enum_fields())
+        for f in test_fields_returned:
+            self.assertIn(f, actual_fields_returned)
+        self.assertEqual(len(actual_fields_returned), len(test_fields_returned))
+        # Also get get_field...
+        for p, v in test_fields_returned:
+            self.assertEqual(e.get_field(p), v)
+        return
+
 
 #   -----------------------------------------------------------------------------
 #
@@ -234,6 +278,7 @@ class TestEntityType(Entity):
     _entityview = "%(id)s/"
     _entityfile = ".sub/manifest.jsonld"
     _entityref  = "../"
+    _contextref = "../../coll_context.jsonld"
 
 class TestEntityTypeSub(Entity):
 
@@ -242,6 +287,7 @@ class TestEntityTypeSub(Entity):
     _entityview = "sub/%(id)s/"
     _entityfile = ".sub/manifest.jsonld"
     _entityref  = "../"
+    _contextref = "../../coll_context.jsonld"
 
 class EntityTest(AnnalistTestCase):
     """
@@ -255,6 +301,11 @@ class EntityTest(AnnalistTestCase):
     def tearDown(self):
         return
 
+    @classmethod
+    def tearDownClass(cls):
+        resetSitedata()
+        return
+
     #   ----------------------------------------
     #
     #   Helpers
@@ -297,8 +348,10 @@ class EntityTest(AnnalistTestCase):
             entity_parent_path=entity_parent_path
             )
         vals.update(
-            { '@id':    '../'
-            , '@type':  [entity_type]
+            { '@id':        '../'   # Local testing only: see `TestEntityType`
+            , '@type':      [entity_type]
+            # , '@base':      "../.."
+            , '@context':   ["../../coll_context.jsonld"]
             })
         return vals
 
@@ -313,19 +366,20 @@ class EntityTest(AnnalistTestCase):
         return
 
     def test_entity_init(self):
-        r = EntityRoot(TestBaseUri, TestBaseDir)
+        r = EntityRoot(TestBaseUri, TestBaseUri, TestBaseDir, TestBaseDir)
         e = Entity(r, "testid")
         self.assertEqual(e._entitytype,     ANNAL.CURIE.Entity)
         self.assertEqual(e._entityfile,     None)
         self.assertEqual(e._entityref,      None)
         self.assertEqual(e._entityid,       "testid")
         self.assertEqual(e._entityurl,      TestBaseUri+"/testid/")
+        self.assertEqual(e._entityviewurl,  TestBaseUri+"/testid/")
         self.assertEqual(e._entitydir,      TestBaseDir+"/testid/")
         self.assertEqual(e._values,         None)
         return
 
     def test_entity_type_id(self):
-        r = EntityRoot(TestBaseUri, TestBaseDir)
+        r = EntityRoot(TestBaseUri, TestBaseUri, TestBaseDir, TestBaseDir)
         self.assertEqual(r.get_type_id(),   None)
         e1 = Entity(r, "testid1")
         self.assertEqual(e1.get_type_id(),  None)
@@ -334,7 +388,7 @@ class EntityTest(AnnalistTestCase):
         return
 
     def test_entity_subclass(self):
-        r = EntityRoot(TestBaseUri, TestBaseDir)
+        r = EntityRoot(TestBaseUri, TestBaseUri, TestBaseDir, TestBaseDir)
         e = TestEntityType(r, "testid")
         self.assertEqual(e._entitytype,     "test:EntityType")
         self.assertEqual(e._entityfile,     ".sub/manifest.jsonld")
@@ -347,7 +401,7 @@ class EntityTest(AnnalistTestCase):
         return
 
     def test_entity_dict(self):
-        r = EntityRoot(TestBaseUri, TestBaseDir)
+        r = EntityRoot(TestBaseUri, TestBaseUri, TestBaseDir, TestBaseDir)
         e = TestEntityType(r, "testid")
         self.assertEqual(e.get("foo",  "bar"),    "bar")
         self.assertEqual(e.get("type", "notype"), "notype")
@@ -355,7 +409,9 @@ class EntityTest(AnnalistTestCase):
         self.assertEqual(e.get("foo",  "bar"),    "bar")
         self.assertEqual(e.get("type", "notype"), "annal:EntityRoot")
         expect_keys = set(
-            ['annal:id', 'annal:type_id', 'annal:type', 'annal:url'
+            ['annal:id', 'annal:type_id'
+            , 'annal:type'
+            , 'annal:url'
             , 'type'
             ])
         self.assertEqual(set(e.keys()), expect_keys)
@@ -371,7 +427,7 @@ class EntityTest(AnnalistTestCase):
 
     def test_entity_path(self):
         test_values = ({})
-        r = EntityRoot(TestBaseUri, TestBaseDir)
+        r = EntityRoot(TestBaseUri, TestBaseUri, TestBaseDir, TestBaseDir)
         e = TestEntityType(r, "testid")
         e.set_values(test_values)
         e._save()
@@ -384,7 +440,7 @@ class EntityTest(AnnalistTestCase):
         test_values_returned = self.values_returned(
             entity_id='testid', entity_type='test:EntityType', entity_title='Name entity test'
             )
-        r = EntityRoot(TestBaseUri, TestBaseDir)
+        r = EntityRoot(TestBaseUri, TestBaseUri, TestBaseDir, TestBaseDir)
         self.assertFalse(TestEntityType.exists(r, "testid"))
         e = TestEntityType.create(r, "testid", test_values)
         self.assertTrue(TestEntityType.exists(r, "testid"))
@@ -398,7 +454,7 @@ class EntityTest(AnnalistTestCase):
         test_values_returned = self.values_reloaded(
             entity_id='testid2', entity_type='test:EntityType', entity_title='Name entity test2'
             )
-        r = EntityRoot(TestBaseUri, TestBaseDir)
+        r = EntityRoot(TestBaseUri, TestBaseUri, TestBaseDir, TestBaseDir)
         e = TestEntityType.create(r, "testid2", test_values)
         e2 = TestEntityType.load(r, "testid2")
         v = e2.get_values()
@@ -417,7 +473,7 @@ class EntityTest(AnnalistTestCase):
             entity_id='testid2', entity_type='test:EntityType', entity_title='Name entity test2',
             entity_parent_path="/testbase"
             )
-        r  = EntityRoot(TestBaseUri, TestBaseDir)
+        r  = EntityRoot(TestBaseUri, TestBaseUri, TestBaseDir, TestBaseDir)
         b  = TestEntityType.create(r, "testbase", {})
         e1 = TestEntityType.create(b, "testid1", test_values1)
         e2 = TestEntityType.create(b, "testid2", test_values2)
@@ -438,7 +494,7 @@ class EntityTest(AnnalistTestCase):
 
     def test_entity_create_remove(self):
         test_values = self.values_created(entity_type='test:EntityType', entity_title='Name entity test')
-        r = EntityRoot(TestBaseUri, TestBaseDir)
+        r = EntityRoot(TestBaseUri, TestBaseUri, TestBaseDir, TestBaseDir)
         self.assertFalse(TestEntityType.exists(r, "testid3"))
         e = TestEntityType.create(r, "testid3", test_values)
         self.assertTrue(TestEntityType.exists(r, "testid3"))
@@ -451,7 +507,7 @@ class EntityTest(AnnalistTestCase):
 
     def test_entity_sub_path(self):
         test_values = self.values_created(entity_type='annal:EntityRoot', entity_title='Name collection coll1')
-        r = EntityRoot(TestBaseUri, TestBaseDir)
+        r = EntityRoot(TestBaseUri, TestBaseUri, TestBaseDir, TestBaseDir)
         e = TestEntityTypeSub(r, "testid")
         self.assertEqual(e._entitydir, TestBaseDir+"/sub/testid/")
         e.set_values(test_values)
@@ -466,7 +522,7 @@ class EntityTest(AnnalistTestCase):
             entity_id='testid', entity_type='test:EntityTypeSub', entity_title='Name entity test',
             entity_parent_path="/sub"
             )
-        r = EntityRoot(TestBaseUri, TestBaseDir)
+        r = EntityRoot(TestBaseUri, TestBaseUri, TestBaseDir, TestBaseDir)
         self.assertFalse(TestEntityTypeSub.exists(r, "testid"))
         e = TestEntityTypeSub.create(r, "testid", test_values)
         self.assertTrue(TestEntityTypeSub.exists(r, "testid"))
@@ -484,7 +540,7 @@ class EntityTest(AnnalistTestCase):
             entity_parent_path="/sub"
             )
 
-        r = EntityRoot(TestBaseUri, TestBaseDir)
+        r = EntityRoot(TestBaseUri, TestBaseUri, TestBaseDir, TestBaseDir)
         e = TestEntityTypeSub.create(r, "testid2", test_values)
         e2 = TestEntityTypeSub.load(r, "testid2")
         v = e2.get_values()
@@ -494,7 +550,7 @@ class EntityTest(AnnalistTestCase):
 
     def test_entity_sub_create_remove(self):
         test_values = self.values_created(entity_type='test:EntityTypeSub', entity_title='Name entity test')
-        r = EntityRoot(TestBaseUri, TestBaseDir)
+        r = EntityRoot(TestBaseUri, TestBaseUri, TestBaseDir, TestBaseDir)
         self.assertFalse(TestEntityTypeSub.exists(r, "testid3"))
         e = TestEntityTypeSub.create(r, "testid3", test_values)
         self.assertTrue(TestEntityTypeSub.exists(r, "testid3"))
@@ -504,7 +560,7 @@ class EntityTest(AnnalistTestCase):
 
     def test_entity_allocate_id(self):
         test_values = self.values_created(entity_type='test:EntityType', entity_title='Name entity test')
-        r = EntityRoot(TestBaseUri, TestBaseDir)
+        r = EntityRoot(TestBaseUri, TestBaseUri, TestBaseDir, TestBaseDir)
         eid = TestEntityType.allocate_new_id(r)
         self.assertEqual(eid, "00000001")
         self.assertFalse(TestEntityType.exists(r, eid))
