@@ -27,24 +27,26 @@ from annalist.models.recordtype     import RecordType
 from annalist.models.recordtypedata import RecordTypeData
 from annalist.models.entitydata     import EntityData
 
-from annalist.views.defaultedit     import EntityDefaultEditView
+from annalist.views.defaultedit             import EntityDefaultEditView
+from annalist.views.form_utils.fieldchoice  import FieldChoice
 
-from tests                          import TestHost, TestHostUri, TestBasePath, TestBaseUri, TestBaseDir
-from tests                          import init_annalist_test_site
-from AnnalistTestCase               import AnnalistTestCase
-from entity_testutils               import (
+from AnnalistTestCase       import AnnalistTestCase
+from tests                  import TestHost, TestHostUri, TestBasePath, TestBaseUri, TestBaseDir
+from init_tests             import init_annalist_test_site, init_annalist_test_coll, resetSitedata
+from entity_testutils       import (
     site_dir, collection_dir, 
     collection_edit_url,
     collection_create_values,
     site_title,
-    render_select_options,
+    render_select_options, render_choice_options,
     create_test_user
     )
-from entity_testtypedata            import (
+from entity_testtypedata    import (
+    recordtype_url,
     recordtype_edit_url,
     recordtype_create_values,
     )
-from entity_testentitydata          import (
+from entity_testentitydata  import (
     recorddata_dir,  entitydata_dir,
     entity_url, entitydata_edit_url, 
     entitydata_list_type_url,
@@ -54,8 +56,8 @@ from entity_testentitydata          import (
     default_fields, default_label, default_comment, error_label,
     layout_classes
     )
-from entity_testsitedata            import (
-    get_site_types, get_site_types_sorted,
+from entity_testsitedata    import (
+    get_site_types, get_site_types_sorted, get_site_types_linked,
     get_site_lists, get_site_lists_sorted,
     get_site_list_types, get_site_list_types_sorted,
     get_site_views, get_site_views_sorted,
@@ -77,12 +79,17 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
 
     def setUp(self):
         init_annalist_test_site()
+        init_annalist_test_coll()
         self.testsite = Site(TestBaseUri, TestBaseDir)
         self.testcoll = Collection.create(self.testsite, "testcoll", collection_create_values("testcoll"))
-        self.testtype = RecordType.create(self.testcoll, "testtype", recordtype_create_values("testtype"))
+        self.testtype = RecordType.create(self.testcoll, "testtype", recordtype_create_values("testcoll", "testtype"))
         self.testdata = RecordTypeData.create(self.testcoll, "testtype", {})
-        self.type_ids   = get_site_types_sorted() + ['testtype']
-        self.no_options = ['(no options)']
+        self.type_ids = get_site_types_linked("testcoll")
+        self.type_ids.append(FieldChoice("_type/testtype", 
+                label="RecordType testcoll/testtype",
+                link=recordtype_url("testcoll", "testtype")
+            ))
+        self.no_options = [ FieldChoice('', label="(no options)") ]
         # Login and permissions
         create_test_user(self.testcoll, "testuser", "testpassword")
         self.client = Client(HTTP_HOST=TestHost)
@@ -91,6 +98,12 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
         return
 
     def tearDown(self):
+        resetSitedata(scope="collections")
+        return
+
+    @classmethod
+    def tearDownClass(cls):
+        resetSitedata()
         return
 
     #   -----------------------------------------------------------------------------
@@ -105,7 +118,8 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
         return e    
 
     def _check_entity_data_values(self, 
-            entity_id, type_id="testtype", update="Entity", parent=None, 
+            entity_id, type_id="testtype", type_uri=None,
+            update="Entity", parent=None, 
             update_dict=None):
         "Helper function checks content of form-updated record type entry with supplied entity_id"
         recorddata = RecordTypeData.load(self.testcoll, type_id)
@@ -113,7 +127,7 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
         e = EntityData.load(recorddata, entity_id)
         self.assertEqual(e.get_id(), entity_id)
         self.assertEqual(e.get_url(""), TestHostUri + entity_url("testcoll", type_id, entity_id))
-        v = entitydata_values(entity_id, type_id=type_id, update=update)
+        v = entitydata_values(entity_id, type_id=type_id, type_uri=type_uri, update=update)
         if update_dict:
             v.update(update_dict)
             for k in update_dict:
@@ -159,10 +173,10 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
                   </div>
                   <div class="%(input_classes)s">
                   """+
-                    render_select_options(
-                      "entity_type", "Type",
-                      get_site_types_sorted()+["testtype"],
-                      "testtype")+
+                    render_choice_options(
+                        "entity_type",
+                        self.type_ids,
+                        "_type/testtype")+
                   """
                   </div>
                 </div>
@@ -217,7 +231,6 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
         self.assertEqual(r.context['type_id'],          "testtype")
         self.assertEqual(r.context['entity_id'],        "00000001")
         self.assertEqual(r.context['orig_id'],          "00000001")
-        self.assertEqual(r.context['entity_url'],       view_url)
         self.assertEqual(r.context['action'],           "new")
         self.assertEqual(r.context['continuation_url'], "/xyzzy/")
         # Fields
@@ -234,7 +247,8 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
         self.assertEqual(r.context['fields'][0]['field_placeholder'],   "(entity id)")
         self.assertEqual(r.context['fields'][0]['field_property_uri'],  "annal:id")
         self.assertEqual(r.context['fields'][0]['field_render_type'],   "EntityId")
-        self.assertEqual(r.context['fields'][0]['field_value_type'],    "annal:Slug")
+        self.assertEqual(r.context['fields'][0]['field_value_mode'],    "Value_direct")
+        self.assertEqual(r.context['fields'][0]['field_target_type'],   "annal:Slug")
         self.assertEqual(r.context['fields'][0].field_value,            "00000001")
         self.assertEqual(r.context['fields'][0]['field_value'],         "00000001")
         self.assertEqual(r.context['fields'][0]['entity_type_id'],      "testtype")
@@ -250,7 +264,8 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
         self.assertEqual(r.context['fields'][1]['field_placeholder'],   "(type id)")
         self.assertEqual(r.context['fields'][1]['field_property_uri'],  "annal:type_id")
         self.assertEqual(r.context['fields'][1]['field_render_type'],   "EntityTypeId")
-        self.assertEqual(r.context['fields'][1]['field_value_type'],    "annal:Slug")
+        self.assertEqual(r.context['fields'][1]['field_value_mode'],    "Value_direct")
+        self.assertEqual(r.context['fields'][1]['field_target_type'],   "annal:Slug")
         self.assertEqual(r.context['fields'][1].field_value,            "testtype")
         self.assertEqual(r.context['fields'][1]['field_value'],         "testtype")
         self.assertEqual(r.context['fields'][1]['entity_type_id'],      "testtype")
@@ -267,7 +282,8 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
         self.assertEqual(r.context['fields'][2]['field_placeholder'],   "(label)")
         self.assertEqual(r.context['fields'][2]['field_property_uri'],  "rdfs:label")
         self.assertEqual(r.context['fields'][2]['field_render_type'],   "Text")
-        self.assertEqual(r.context['fields'][2]['field_value_type'],    "annal:Text")
+        self.assertEqual(r.context['fields'][2]['field_value_mode'],    "Value_direct")
+        self.assertEqual(r.context['fields'][2]['field_target_type'],   "annal:Text")
         self.assertEqual(r.context['fields'][2]['field_value'],         field_label_value)
         self.assertEqual(r.context['fields'][2]['entity_type_id'],      "testtype")
         self.assertEqual(r.context['fields'][2]['options'],             self.no_options)
@@ -282,8 +298,9 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
         self.assertEqual(r.context['fields'][3]['field_help'],          field_comment_help)
         self.assertEqual(r.context['fields'][3]['field_placeholder'],   "(description)")
         self.assertEqual(r.context['fields'][3]['field_property_uri'],  "rdfs:comment")
-        self.assertEqual(r.context['fields'][3]['field_render_type'],   "Textarea")
-        self.assertEqual(r.context['fields'][3]['field_value_type'],    "annal:Longtext")
+        self.assertEqual(r.context['fields'][3]['field_render_type'],   "Markdown")
+        self.assertEqual(r.context['fields'][3]['field_value_mode'],    "Value_direct")
+        self.assertEqual(r.context['fields'][3]['field_target_type'],   "annal:Richtext")
         self.assertEqual(r.context['fields'][3]['field_value'],         field_comment_value)
         self.assertEqual(r.context['fields'][3]['entity_type_id'],      "testtype")
         self.assertEqual(r.context['fields'][3]['options'],             self.no_options)
@@ -301,7 +318,6 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
         self.assertEqual(r.context['type_id'],          "testtype")
         self.assertEqual(r.context['entity_id'],        "entity1")
         self.assertEqual(r.context['orig_id'],          "entity1")
-        self.assertEqual(r.context['entity_url'],       view_url)
         self.assertEqual(r.context['action'],           "edit")
         self.assertEqual(r.context['continuation_url'], "/xyzzy/")
         # Fields
@@ -311,16 +327,17 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
             "A short identifier that distinguishes this record from "+
             "all other records of the same type in the same collection."
             )
-        self.assertEqual(r.context['fields'][0]['field_id'], 'Entity_id')
-        self.assertEqual(r.context['fields'][0]['field_name'], 'entity_id')
-        self.assertEqual(r.context['fields'][0]['field_label'], 'Id')
-        self.assertEqual(r.context['fields'][0]['field_help'], field_id_help)
-        self.assertEqual(r.context['fields'][0]['field_placeholder'], "(entity id)")
+        self.assertEqual(r.context['fields'][0]['field_id'],           'Entity_id')
+        self.assertEqual(r.context['fields'][0]['field_name'],         'entity_id')
+        self.assertEqual(r.context['fields'][0]['field_label'],        'Id')
+        self.assertEqual(r.context['fields'][0]['field_help'],         field_id_help)
+        self.assertEqual(r.context['fields'][0]['field_placeholder'],  "(entity id)")
         self.assertEqual(r.context['fields'][0]['field_property_uri'], "annal:id")
         self.assertEqual(r.context['fields'][0]['field_render_type'],  "EntityId")
-        self.assertEqual(r.context['fields'][0]['field_value_type'], "annal:Slug")
-        self.assertEqual(r.context['fields'][0]['field_value'], "entity1")
-        self.assertEqual(r.context['fields'][0]['options'], self.no_options)
+        self.assertEqual(r.context['fields'][0]['field_value_mode'],   "Value_direct")
+        self.assertEqual(r.context['fields'][0]['field_target_type'],  "annal:Slug")
+        self.assertEqual(r.context['fields'][0]['field_value'],        "entity1")
+        self.assertEqual(r.context['fields'][0]['options'],            self.no_options)
         # 2nd field
         field_type_help = (
             "A short identifier that identifies the type of the corresponding entity."
@@ -332,9 +349,10 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
         self.assertEqual(r.context['fields'][1]['field_placeholder'],  "(type id)")
         self.assertEqual(r.context['fields'][1]['field_property_uri'], "annal:type_id")
         self.assertEqual(r.context['fields'][1]['field_render_type'],  "EntityTypeId")
-        self.assertEqual(r.context['fields'][1]['field_value_type'], "annal:Slug")
-        self.assertEqual(r.context['fields'][1]['field_value'], "testtype")
-        self.assertEqual(set(r.context['fields'][1]['options']), set(self.type_ids))
+        self.assertEqual(r.context['fields'][1]['field_value_mode'],   "Value_direct")
+        self.assertEqual(r.context['fields'][1]['field_target_type'],  "annal:Slug")
+        self.assertEqual(r.context['fields'][1]['field_value'],        "testtype")
+        self.assertEqual(set(r.context['fields'][1]['options']),       set(self.type_ids))
         # 3rd field
         field_label_help = (
             "Short string used to describe entity when displayed"
@@ -342,16 +360,17 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
         field_label_value = (
             "Entity testcoll/testtype/entity1"
             )
-        self.assertEqual(r.context['fields'][2]['field_id'], 'Entity_label')
-        self.assertEqual(r.context['fields'][2]['field_name'], 'Entity_label')
-        self.assertEqual(r.context['fields'][2]['field_label'], 'Label')
-        self.assertEqual(r.context['fields'][2]['field_help'], field_label_help)
-        self.assertEqual(r.context['fields'][2]['field_placeholder'], "(label)")
+        self.assertEqual(r.context['fields'][2]['field_id'],           'Entity_label')
+        self.assertEqual(r.context['fields'][2]['field_name'],         'Entity_label')
+        self.assertEqual(r.context['fields'][2]['field_label'],        'Label')
+        self.assertEqual(r.context['fields'][2]['field_help'],         field_label_help)
+        self.assertEqual(r.context['fields'][2]['field_placeholder'],  "(label)")
         self.assertEqual(r.context['fields'][2]['field_property_uri'], "rdfs:label")
-        self.assertEqual(r.context['fields'][2]['field_render_type'], "Text")
-        self.assertEqual(r.context['fields'][2]['field_value_type'], "annal:Text")
-        self.assertEqual(r.context['fields'][2]['field_value'], field_label_value)
-        self.assertEqual(r.context['fields'][2]['options'], self.no_options)
+        self.assertEqual(r.context['fields'][2]['field_render_type'],  "Text")
+        self.assertEqual(r.context['fields'][2]['field_value_mode'],   "Value_direct")
+        self.assertEqual(r.context['fields'][2]['field_target_type'],  "annal:Text")
+        self.assertEqual(r.context['fields'][2]['field_value'],        field_label_value)
+        self.assertEqual(r.context['fields'][2]['options'],            self.no_options)
         # 4th field
         field_comment_help = (
             "Descriptive text about an entity."
@@ -359,16 +378,17 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
         field_comment_value = (
             "Entity coll testcoll, type testtype, entity entity1"
             )
-        self.assertEqual(r.context['fields'][3]['field_id'], 'Entity_comment')
-        self.assertEqual(r.context['fields'][3]['field_name'], 'Entity_comment')
-        self.assertEqual(r.context['fields'][3]['field_label'], 'Comment')
-        self.assertEqual(r.context['fields'][3]['field_help'], field_comment_help)
-        self.assertEqual(r.context['fields'][3]['field_placeholder'], "(description)")
+        self.assertEqual(r.context['fields'][3]['field_id'],           'Entity_comment')
+        self.assertEqual(r.context['fields'][3]['field_name'],         'Entity_comment')
+        self.assertEqual(r.context['fields'][3]['field_label'],        'Comment')
+        self.assertEqual(r.context['fields'][3]['field_help'],         field_comment_help)
+        self.assertEqual(r.context['fields'][3]['field_placeholder'],  "(description)")
         self.assertEqual(r.context['fields'][3]['field_property_uri'], "rdfs:comment")
-        self.assertEqual(r.context['fields'][3]['field_render_type'],  "Textarea")
-        self.assertEqual(r.context['fields'][3]['field_value_type'], "annal:Longtext")
-        self.assertEqual(r.context['fields'][3]['field_value'], field_comment_value)
-        self.assertEqual(r.context['fields'][3]['options'], self.no_options)
+        self.assertEqual(r.context['fields'][3]['field_render_type'],  "Markdown")
+        self.assertEqual(r.context['fields'][3]['field_value_mode'],   "Value_direct")
+        self.assertEqual(r.context['fields'][3]['field_target_type'],  "annal:Richtext")
+        self.assertEqual(r.context['fields'][3]['field_value'],        field_comment_value)
+        self.assertEqual(r.context['fields'][3]['options'],            self.no_options)
         return
 
     def test_get_edit_not_exists(self):
@@ -380,7 +400,7 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
         self.assertContains(r, "<h3>404: Not found</h3>", status_code=404)
         # log.debug(r.content)
         def_label = error_label("testcoll", "testtype", "entitynone")
-        self.assertContains(r, "<p>%s does not exist</p>"%(def_label), status_code=404)
+        self.assertContains(r, "<p>Entity %s does not exist</p>"%(def_label), status_code=404)
         return
 
     #   -----------------------------------------------------------------------------
@@ -455,9 +475,11 @@ class EntityDefaultEditViewTest(AnnalistTestCase):
         self.assertEqual(r.content,       "")
         self.assertEqual(r['location'], TestHostUri + entitydata_list_type_url("testcoll", "Default_type"))
         # Check new entity data created
-        self._check_entity_data_values("newentity", type_id="Default_type", update_dict=
-            { '@type':         ['annal:Default_type', 'annal:EntityData']
-            })
+        self._check_entity_data_values(
+            "newentity", type_id="Default_type", type_uri="annal:Default_type",
+            update_dict=
+                { '@type': ['annal:Default_type', 'annal:EntityData'] }
+            )
         return
 
     def test_new_entity_new_type(self):
