@@ -19,21 +19,23 @@ from django.contrib.auth.models     import User
 from django.test                    import TestCase # cf. https://docs.djangoproject.com/en/dev/topics/testing/tools/#assertions
 from django.test.client             import Client
 
-from annalist.identifiers           import RDF, RDFS, ANNAL
 from annalist                       import layout
+from annalist.identifiers           import RDF, RDFS, ANNAL
+
 from annalist.models.site           import Site
 from annalist.models.collection     import Collection
 from annalist.models.recordtype     import RecordType
 from annalist.models.recordtypedata import RecordTypeData
 from annalist.models.entitydata     import EntityData
 
-from annalist.views.uri_builder     import uri_params, uri_with_params
-from annalist.views.defaultlist     import EntityDefaultListView
+from annalist.views.uri_builder             import uri_params, uri_with_params
+from annalist.views.defaultlist             import EntityDefaultListView
+from annalist.views.form_utils.fieldchoice  import FieldChoice
 
-from tests                          import TestHost, TestHostUri, TestBasePath, TestBaseUri, TestBaseDir
-from tests                          import init_annalist_test_site
-from AnnalistTestCase               import AnnalistTestCase
-from entity_testutils               import (
+from AnnalistTestCase       import AnnalistTestCase
+from tests                  import TestHost, TestHostUri, TestBasePath, TestBaseUri, TestBaseDir
+from init_tests             import init_annalist_test_site, init_annalist_test_coll, resetSitedata
+from entity_testutils       import (
     site_dir, collection_dir, 
     collection_edit_url,
     continuation_url_param,
@@ -45,12 +47,12 @@ from entity_testutils               import (
     context_list_head_fields,
     context_list_item_fields, context_list_item_field_value
     )
-from entity_testtypedata            import (
+from entity_testtypedata    import (
     recordtype_dir, 
     recordtype_url,
     recordtype_create_values
     )
-from entity_testentitydata          import (
+from entity_testentitydata  import (
     recorddata_dir,  entitydata_dir,
     entity_url, entitydata_edit_url, entitydata_delete_confirm_url,
     entitydata_list_type_url, entitydata_list_all_url,
@@ -58,15 +60,16 @@ from entity_testentitydata          import (
     entitydata_context_data, entitydata_form_data, entitydata_delete_confirm_form_data,
     entitylist_form_data
     )
-from entity_testsitedata            import (
-    get_site_types, get_site_types_sorted,
-    get_site_lists, get_site_lists_sorted,
+from entity_testsitedata    import (
+    get_site_types, get_site_types_sorted, get_site_types_linked,
+    get_site_lists, get_site_lists_sorted, get_site_lists_linked,
+    get_site_views, get_site_views_sorted, get_site_views_linked,
     get_site_list_types, get_site_list_types_sorted,
-    get_site_views, get_site_views_sorted,
     get_site_field_groups, get_site_field_groups_sorted, 
     get_site_fields, get_site_fields_sorted, 
     get_site_field_types, get_site_field_types_sorted, 
     )
+from entity_testlistdata    import recordlist_url
 
 
 #   -----------------------------------------------------------------------------
@@ -78,7 +81,7 @@ from entity_testsitedata            import (
 
 class EntityDefaultListViewTest(AnnalistTestCase):
     """
-    Tests for record type edit views
+    Tests for default list views
     """
 
     def setUp(self):
@@ -95,7 +98,12 @@ class EntityDefaultListViewTest(AnnalistTestCase):
         e4 = EntityData.create(self.testdata2, "entity4", 
             entitydata_create_values("entity4", type_id="testtype2")
             )
-        self.initial_list_ids = get_site_lists()
+        self.type_ids = get_site_types_linked("testcoll")
+        self.type_ids.append(FieldChoice("testtype", 
+                label="RecordType testcoll/testtype",
+                link=recordtype_url("testcoll", "testtype")
+            ))
+        self.list_ids = get_site_lists_linked("testcoll")
         # Login and permissions
         create_test_user(self.testcoll, "testuser", "testpassword")
         self.client = Client(HTTP_HOST=TestHost)
@@ -104,6 +112,12 @@ class EntityDefaultListViewTest(AnnalistTestCase):
         return
 
     def tearDown(self):
+        # resetSitedata(scope="collections")
+        return
+
+    @classmethod
+    def tearDownClass(cls):
+        resetSitedata()
         return
 
     #   -----------------------------------------------------------------------------
@@ -134,16 +148,7 @@ class EntityDefaultListViewTest(AnnalistTestCase):
         self.assertContains(r, "<h3>List entities with type information</h3>", html=True)
         self.assertMatch(r.content, r'<input.type="hidden".name="continuation_url".+value="/xyzzy/"/>')
         cont = uri_params({"continuation_url": u})
-        # rowdata = """
-        #     <tr class="select-row">
-        #         <td class="small-2 columns"><a href="%(base)s/c/testcoll/d/testtype/entity1/%(cont)s">entity1</a></td>
-        #         <td class="small-2 columns"><a href="%(base)s/c/testcoll/d/_type/testtype/%(cont)s">testtype</a></td>
-        #         <td class="small-8 columns">Entity testcoll/testtype/entity1</td>
-        #         <td class="select-row">
-        #             <input type="checkbox" name="entity_select" value="testtype/entity1" />
-        #         </td>
-        #     </tr>
-        #     """%({'base': TestBasePath, 'cont': cont})
+        cont = ""
         rowdata = """
             <div class="tbody row select-row">
               <div class="small-1 columns">
@@ -156,7 +161,7 @@ class EntityDefaultListViewTest(AnnalistTestCase):
                     <a href="%(base)s/c/testcoll/d/testtype/entity1/%(cont)s">entity1</a>
                   </div>
                   <div class="view-value small-2 columns">
-                    <a href="/testsite/c/testcoll/d/_type/testtype/%(cont)s">testtype</a>
+                    <a href="/testsite/c/testcoll/d/_type/testtype/%(cont)s">RecordType testcoll/testtype</a>
                   </div>
                   <div class="view-value small-7 columns">
                     <span>Entity testcoll/testtype/entity1</span>
@@ -171,47 +176,50 @@ class EntityDefaultListViewTest(AnnalistTestCase):
         # self.assertEqual(r.context['title'],            site_title())
         self.assertEqual(r.context['title'],            "Collection testcoll")
         self.assertEqual(r.context['coll_id'],          "testcoll")
-        self.assertEqual(r.context['type_id'],          None)
+        self.assertEqual(r.context['type_id'],          "Default_type")
         list_choices = r.context['list_choices']
-        self.assertEqual(set(list_choices.options),    set(self.initial_list_ids))
+        self.assertEqual(set(list_choices.options),    set(self.list_ids))
         self.assertEqual(list_choices['field_value'],   "Default_list_all")
         self.assertEqual(r.context['continuation_url'], "/xyzzy/")
 
         # Unbound field descriptions
         self.assertEqual(len(r.context['fields']), 3)
         #  1st field
-        self.assertEqual(r.context['fields'][0]['field_id'], 'Entity_id')
-        self.assertEqual(r.context['fields'][0]['field_name'], 'entity_id')
-        self.assertEqual(r.context['fields'][0]['field_label'], 'Id')
-        self.assertEqual(r.context['fields'][0]['field_placeholder'], "(entity id)")
+        self.assertEqual(r.context['fields'][0]['field_id'],           'Entity_id')
+        self.assertEqual(r.context['fields'][0]['field_name'],         'entity_id')
+        self.assertEqual(r.context['fields'][0]['field_label'],        'Id')
+        self.assertEqual(r.context['fields'][0]['field_placeholder'],  "(entity id)")
         self.assertEqual(r.context['fields'][0]['field_property_uri'], "annal:id")
-        self.assertEqual(r.context['fields'][0]['field_render_type'], "EntityId")
+        self.assertEqual(r.context['fields'][0]['field_render_type'],  "EntityId")
+        self.assertEqual(r.context['fields'][0]['field_value_mode'],   "Value_direct")
+        self.assertEqual(r.context['fields'][0]['field_target_type'],  "annal:Slug")
         self.assertEqual(r.context['fields'][0]['field_placement'].field, "small-3 columns")
-        self.assertEqual(r.context['fields'][0]['field_value_type'], "annal:Slug")
-        self.assertEqual(r.context['fields'][0]['field_value'], "")
-        self.assertEqual(r.context['fields'][0]['entity_type_id'], "")
+        self.assertEqual(r.context['fields'][0]['field_value'],        "")
+        self.assertEqual(r.context['fields'][0]['entity_type_id'],     "")
         #  2nd field
-        self.assertEqual(r.context['fields'][1]['field_id'], 'Entity_type')
-        self.assertEqual(r.context['fields'][1]['field_name'], 'entity_type')
-        self.assertEqual(r.context['fields'][1]['field_label'], 'Type')
-        self.assertEqual(r.context['fields'][1]['field_placeholder'], "(type id)")
+        self.assertEqual(r.context['fields'][1]['field_id'],           'Entity_type')
+        self.assertEqual(r.context['fields'][1]['field_name'],         'entity_type')
+        self.assertEqual(r.context['fields'][1]['field_label'],        'Type')
+        self.assertEqual(r.context['fields'][1]['field_placeholder'],  "(type id)")
         self.assertEqual(r.context['fields'][1]['field_property_uri'], "annal:type_id")
-        self.assertEqual(r.context['fields'][1]['field_render_type'], "EntityTypeId")
+        self.assertEqual(r.context['fields'][1]['field_render_type'],  "EntityTypeId")
+        self.assertEqual(r.context['fields'][1]['field_value_mode'],   "Value_direct")
+        self.assertEqual(r.context['fields'][1]['field_target_type'],  "annal:Slug")
         self.assertEqual(r.context['fields'][1]['field_placement'].field, "small-2 columns")
-        self.assertEqual(r.context['fields'][1]['field_value_type'], "annal:Slug")
-        self.assertEqual(r.context['fields'][1]['field_value'], "")
-        self.assertEqual(r.context['fields'][1]['entity_type_id'], "")
+        self.assertEqual(r.context['fields'][1]['field_value'],        "")
+        self.assertEqual(r.context['fields'][1]['entity_type_id'],     "")
         # 3rd field
-        self.assertEqual(r.context['fields'][2]['field_id'], 'Entity_label')
-        self.assertEqual(r.context['fields'][2]['field_name'], 'Entity_label')
-        self.assertEqual(r.context['fields'][2]['field_label'], 'Label')
-        self.assertEqual(r.context['fields'][2]['field_placeholder'], "(label)")
+        self.assertEqual(r.context['fields'][2]['field_id'],           'Entity_label')
+        self.assertEqual(r.context['fields'][2]['field_name'],         'Entity_label')
+        self.assertEqual(r.context['fields'][2]['field_label'],        'Label')
+        self.assertEqual(r.context['fields'][2]['field_placeholder'],  "(label)")
         self.assertEqual(r.context['fields'][2]['field_property_uri'], "rdfs:label")
-        self.assertEqual(r.context['fields'][2]['field_render_type'], "Text")
+        self.assertEqual(r.context['fields'][2]['field_render_type'],  "Text")
+        self.assertEqual(r.context['fields'][2]['field_value_mode'],   "Value_direct")
+        self.assertEqual(r.context['fields'][2]['field_target_type'],  "annal:Text")
         self.assertEqual(r.context['fields'][2]['field_placement'].field, "small-7 columns")
-        self.assertEqual(r.context['fields'][2]['field_value_type'], "annal:Text")
-        self.assertEqual(r.context['fields'][2]['field_value'], "")
-        self.assertEqual(r.context['fields'][2]['entity_type_id'], "")
+        self.assertEqual(r.context['fields'][2]['field_value'],        "")
+        self.assertEqual(r.context['fields'][2]['entity_type_id'],     "")
         # Entities and bound fields
         entities = context_list_entities(r.context)
         self.assertEqual(len(entities), 6)
@@ -234,7 +242,7 @@ class EntityDefaultListViewTest(AnnalistTestCase):
                 for fkey in (
                         'field_id', 'field_name', 'field_label', 
                         'field_property_uri', 'field_render_type',
-                        'field_placement', 'field_value_type'):
+                        'field_placement', 'field_target_type'):
                     self.assertEqual(item_field[fkey], head_field[fkey])
                 # Check row field values
                 fkey = field_keys[fid]
@@ -251,6 +259,7 @@ class EntityDefaultListViewTest(AnnalistTestCase):
         self.assertContains(r, "<title>Collection testcoll</title>")
         self.assertContains(r, "<h3>List entities</h3>", html=True)
         cont = uri_params({"continuation_url": u})
+        cont = ""
         # log.info(r.content)
         rowdata = """
             <div class="tbody row select-row">
@@ -277,31 +286,33 @@ class EntityDefaultListViewTest(AnnalistTestCase):
         self.assertEqual(r.context['coll_id'],          "testcoll")
         self.assertEqual(r.context['type_id'],          "testtype")
         list_choices = r.context['list_choices']
-        self.assertEqual(set(list_choices.options),     set(self.initial_list_ids))
+        self.assertEqual(set(list_choices.options),     set(self.list_ids))
         self.assertEqual(list_choices['field_value'],   "Default_list")
         self.assertEqual(r.context['continuation_url'], "/xyzzy/")
         # Fields
         self.assertEqual(len(r.context['fields']), 2)
         # 1st field
-        self.assertEqual(r.context['fields'][0]['field_id'], 'Entity_id')
-        self.assertEqual(r.context['fields'][0]['field_name'], 'entity_id')
-        self.assertEqual(r.context['fields'][0]['field_label'], 'Id')
-        self.assertEqual(r.context['fields'][0]['field_placeholder'], "(entity id)")
+        self.assertEqual(r.context['fields'][0]['field_id'],           'Entity_id')
+        self.assertEqual(r.context['fields'][0]['field_name'],         'entity_id')
+        self.assertEqual(r.context['fields'][0]['field_label'],        'Id')
+        self.assertEqual(r.context['fields'][0]['field_placeholder'],  "(entity id)")
         self.assertEqual(r.context['fields'][0]['field_property_uri'], "annal:id")
-        self.assertEqual(r.context['fields'][0]['field_render_type'], "EntityId")
+        self.assertEqual(r.context['fields'][0]['field_render_type'],  "EntityId")
+        self.assertEqual(r.context['fields'][0]['field_value_mode'],   "Value_direct")
+        self.assertEqual(r.context['fields'][0]['field_target_type'],  "annal:Slug")
         self.assertEqual(r.context['fields'][0]['field_placement'].field, "small-3 columns")
-        self.assertEqual(r.context['fields'][0]['field_value_type'], "annal:Slug")
-        self.assertEqual(r.context['fields'][0]['field_value'], "")
+        self.assertEqual(r.context['fields'][0]['field_value'],        "")
         # 2nd field
-        self.assertEqual(r.context['fields'][1]['field_id'], 'Entity_label')
-        self.assertEqual(r.context['fields'][1]['field_name'], 'Entity_label')
-        self.assertEqual(r.context['fields'][1]['field_label'], 'Label')
-        self.assertEqual(r.context['fields'][1]['field_placeholder'], "(label)")
+        self.assertEqual(r.context['fields'][1]['field_id'],           'Entity_label')
+        self.assertEqual(r.context['fields'][1]['field_name'],         'Entity_label')
+        self.assertEqual(r.context['fields'][1]['field_label'],        'Label')
+        self.assertEqual(r.context['fields'][1]['field_placeholder'],  "(label)")
         self.assertEqual(r.context['fields'][1]['field_property_uri'], "rdfs:label")
-        self.assertEqual(r.context['fields'][1]['field_render_type'], "Text")
+        self.assertEqual(r.context['fields'][1]['field_render_type'],  "Text")
+        self.assertEqual(r.context['fields'][1]['field_value_mode'],   "Value_direct")
+        self.assertEqual(r.context['fields'][1]['field_target_type'],  "annal:Text")
         self.assertEqual(r.context['fields'][1]['field_placement'].field, "small-9 columns")
-        self.assertEqual(r.context['fields'][1]['field_value_type'], "annal:Text")
-        self.assertEqual(r.context['fields'][1]['field_value'], "")
+        self.assertEqual(r.context['fields'][1]['field_value'],        "")
         # Entities
         entities = context_list_entities(r.context)
         self.assertEqual(len(entities), 3)
@@ -313,7 +324,7 @@ class EntityDefaultListViewTest(AnnalistTestCase):
                 for fkey in (
                         'field_id', 'field_name', 'field_label', 
                         'field_property_uri', 'field_render_type',
-                        'field_placement', 'field_value_type'):
+                        'field_placement', 'field_target_type'):
                     self.assertEqual(item_field[fkey], head_field[fkey])
                 self.assertEqual(item_field['field_value'], field_val[fid]%(eid+1))
         return
