@@ -28,24 +28,26 @@ from django.test.client             import Client
 
 from utils.SuppressLoggingContext   import SuppressLogging
 
-from annalist.identifiers           import RDF, RDFS, ANNAL
 from annalist                       import layout
+from annalist.identifiers           import RDF, RDFS, ANNAL
+
 from annalist.models.site           import Site
 from annalist.models.collection     import Collection
 from annalist.models.recordtype     import RecordType
 from annalist.models.recordtypedata import RecordTypeData
 from annalist.models.entitydata     import EntityData
 
-from annalist.views.uri_builder     import uri_params, uri_with_params
-from annalist.views.entitylist      import EntityGenericListView #, EntityDataDeleteConfirmedView
+from annalist.views.uri_builder             import uri_params, uri_with_params
+from annalist.views.entitylist              import EntityGenericListView
+from annalist.views.form_utils.fieldchoice  import FieldChoice
 
-from tests                          import TestHost, TestHostUri, TestBasePath, TestBaseUri, TestBaseDir
-from tests                          import init_annalist_test_site
-from AnnalistTestCase               import AnnalistTestCase
-from entity_testutils               import (
+from AnnalistTestCase       import AnnalistTestCase
+from tests                  import TestHost, TestHostUri, TestBasePath, TestBaseUri, TestBaseDir
+from init_tests             import init_annalist_test_site, init_annalist_test_coll, resetSitedata
+from entity_testutils       import (
     site_dir, collection_dir,
     site_view_url,
-    collection_edit_url,
+    collection_view_url, collection_edit_url,
     continuation_url_param,
     confirm_delete_params,
     collection_create_values,
@@ -55,12 +57,12 @@ from entity_testutils               import (
     context_list_head_fields, context_list_item_fields,
     context_list_item_field, context_list_item_field_value
     )
-from entity_testtypedata            import (
+from entity_testtypedata    import (
     recordtype_dir, 
     recordtype_url,
     recordtype_create_values, 
     )
-from entity_testentitydata          import (
+from entity_testentitydata  import (
     recorddata_dir,  entitydata_dir,
     entity_url, entitydata_edit_url, entitydata_delete_confirm_url,
     entitydata_list_type_url, entitydata_list_all_url,
@@ -68,15 +70,17 @@ from entity_testentitydata          import (
     entitydata_context_data, entitydata_form_data, entitydata_delete_confirm_form_data,
     entitylist_form_data
     )
-from entity_testsitedata            import (
-    get_site_types, get_site_types_sorted,
-    get_site_lists, get_site_lists_sorted,
+from entity_testsitedata    import (
+    make_field_choices, no_selection,
+    get_site_types, get_site_types_sorted, get_site_types_linked,
+    get_site_lists, get_site_lists_sorted, get_site_lists_linked,
+    get_site_views, get_site_views_sorted, get_site_views_linked,
     get_site_list_types, get_site_list_types_sorted,
-    get_site_views, get_site_views_sorted,
     get_site_field_groups, get_site_field_groups_sorted, 
     get_site_fields, get_site_fields_sorted, 
     get_site_field_types, get_site_field_types_sorted, 
     )
+from entity_testlistdata    import recordlist_url
 
 #   -----------------------------------------------------------------------------
 #
@@ -110,10 +114,16 @@ class EntityGenericListViewTest(AnnalistTestCase):
         e4 = EntityData.create(self.testdata2, "entity4", 
             entitydata_create_values("entity4", type_id="testtype2")
             )
-        self.initial_list_ids = get_site_lists()
+        self.list_ids = get_site_lists_linked("testcoll")
         return
 
     def tearDown(self):
+        # resetSitedata()
+        return
+
+    @classmethod
+    def tearDownClass(cls):
+        resetSitedata()
         return
 
     #   -----------------------------------------------------------------------------
@@ -146,6 +156,7 @@ class EntityGenericListViewTest(AnnalistTestCase):
         self.assertMatch(r.content, r'<input.type="hidden".name="continuation_url".+value="/xyzzy/"/>')
         # log.info(r.content) #@@
         cont = uri_params({"continuation_url": u})
+        cont = ""
         rowdata = """
             <div class="tbody row select-row">
               <div class="small-1 columns">
@@ -158,7 +169,7 @@ class EntityGenericListViewTest(AnnalistTestCase):
                     <a href="%(base)s/c/testcoll/d/testtype/entity1/%(cont)s">entity1</a>
                   </div>
                   <div class="view-value small-2 columns">
-                    <a href="/testsite/c/testcoll/d/_type/testtype/%(cont)s">testtype</a>
+                    <a href="/testsite/c/testcoll/d/_type/testtype/%(cont)s">RecordType testcoll/testtype</a>
                   </div>
                   <div class="view-value small-7 columns">
                     <span>Entity testcoll/testtype/entity1</span>
@@ -174,10 +185,10 @@ class EntityGenericListViewTest(AnnalistTestCase):
         # Test context
         self.assertEqual(r.context['title'],            "Collection testcoll")
         self.assertEqual(r.context['coll_id'],          "testcoll")
-        self.assertEqual(r.context['type_id'],          None)
+        self.assertEqual(r.context['type_id'],          "Default_type")
         self.assertEqual(r.context['continuation_url'], "/xyzzy/")
         list_choices = r.context['list_choices']
-        self.assertEqual(set(list_choices.options),     set(self.initial_list_ids))
+        self.assertEqual(set(list_choices.options),     set(self.list_ids))
         self.assertEqual(list_choices['field_value'],   "Default_list_all")
         # Unbound field descriptions
         head_fields = context_list_head_fields(r.context)
@@ -207,7 +218,7 @@ class EntityGenericListViewTest(AnnalistTestCase):
                 for fkey in (
                         'field_id', 'field_name', 'field_label', 
                         'field_property_uri', 'field_render_type',
-                        'field_placement', 'field_value_type'):
+                        'field_placement', 'field_target_type'):
                     self.assertEqual(item_field[fkey], head_field[fkey])
                 # Check row field values
                 fkey = field_keys[fid]
@@ -229,9 +240,9 @@ class EntityGenericListViewTest(AnnalistTestCase):
         # Test context
         self.assertEqual(r.context['title'],            "Collection testcoll")
         self.assertEqual(r.context['coll_id'],          "testcoll")
-        self.assertEqual(r.context['type_id'],          None)
+        self.assertEqual(r.context['type_id'],          "Default_type")
         list_choices = r.context['list_choices']
-        self.assertEqual(set(list_choices.options),     set(self.initial_list_ids))
+        self.assertEqual(set(list_choices.options),     set(self.list_ids))
         self.assertEqual(list_choices['field_value'],   "Default_list_all")
         # Unbound field descriptions
         head_fields = context_list_head_fields(r.context)
@@ -241,7 +252,7 @@ class EntityGenericListViewTest(AnnalistTestCase):
         self.assertEqual(head_fields[2]['field_id'], 'Entity_label')
         # Entities and bound fields
         entities = context_list_entities(r.context)
-        self.assertEqual(len(entities), 183)    # Will change with site data
+        self.assertEqual(len(entities), 150)    # Will change with site data
         return
 
     def test_get_types_list(self):
@@ -299,8 +310,8 @@ class EntityGenericListViewTest(AnnalistTestCase):
         # Entities
         entities   = context_list_entities(r.context)
         listed_entities = { e['entity_id']: e for e in entities }
-        self.assertIn('_initial_values', listed_entities)
-        type_entities = get_site_types() | {"_initial_values", "testtype", "testtype2"}
+        #@@ self.assertIn('_initial_values', listed_entities)
+        type_entities = get_site_types() | {"testtype", "testtype2"}
         self.assertEqual(set(listed_entities.keys()), type_entities)
         return
 
@@ -316,103 +327,174 @@ class EntityGenericListViewTest(AnnalistTestCase):
         # self.assertContains(r, site_title("<title>%s</title>"))
         # self.assertContains(r, "<h3>List 'Field_list' of entities in collection 'testcoll'</h3>", html=True)
         cont = uri_params({"continuation_url": u})
+        cont = ""
         rowdata1 = """
             <div class="tbody row select-row">
               <div class="small-1 columns">
                 <input type="checkbox" class="select-box right" name="entity_select"
-                       value="_field/Bib_address" />
+                       value="_field/Coll_comment" />
               </div>
               <div class="small-11 columns">
-                <div class="row view-listrow">
-                  <div class="view-value small-3 columns">
-                    <a href="%(base)s/c/testcoll/d/_field/Bib_address/%(cont)s">Bib_address</a>
+                <div class="view-listrow row">
+                  <div class="view-value small-4 medium-3 columns">
+                    <a href="%(base)s/c/testcoll/d/_field/Coll_comment/%(cont)s">Coll_comment</a>
+                  </div>
+                  <div class="view-value small-4 medium-3 columns">
+                    <a href="%(base)s/c/testcoll/d/Enum_render_type/Textarea/%(cont)s">Multiline text</a>
+                  </div>
+                  <div class="view-value small-12 medium-3 columns show-for-medium-up">
+                    <span>annal:Longtext</span>
+                  </div>
+                  <div class="view-value small-4 medium-3 columns">
+                    <span>Collection metadata</span>
+                  </div>
+                </div>
+              </div>
+            </div>
+            """%({'base': TestBasePath, 'cont': cont})
+        rowdata2 = """
+            <div class="tbody row select-row">
+              <div class="small-1 columns">
+                <input type="checkbox" class="select-box right" name="entity_select"
+                       value="_field/Coll_parent" />
+              </div>
+              <div class="small-11 columns">
+                <div class="view-listrow row">
+                  <div class="view-value small-4 medium-3 columns">
+                    <a href="%(base)s/c/testcoll/d/_field/Coll_parent/%(cont)s">Coll_parent</a>
+                  </div>
+                  <div class="view-value small-4 medium-3 columns">
+                    <a href="%(base)s/c/testcoll/d/Enum_render_type/Enum_choice/%(cont)s">Entity choice</a>
+                  </div>
+                  <div class="view-value small-12 medium-3 columns show-for-medium-up">
+                    <span>annal:Slug</span>
+                  </div>
+                  <div class="view-value small-4 medium-3 columns">
+                    <span>Parent</span>
+                  </div>
+                </div>
+              </div>
+            </div>
+            """%({'base': TestBasePath, 'cont': cont})
+        rowdata3 = """
+            <div class="tbody row select-row">
+              <div class="small-1 columns">
+                <input type="checkbox" class="select-box right" name="entity_select"
+                       value="_field/Coll_software_version" />
+              </div>
+              <div class="small-11 columns">
+                <div class="view-listrow row">
+                  <div class="view-value small-4 medium-3 columns">
+                    <a href="%(base)s/c/testcoll/d/_field/Coll_software_version/%(cont)s">Coll_software_version</a>
+                  </div>
+                  <div class="view-value small-4 medium-3 columns">
+                    <a href="%(base)s/c/testcoll/d/Enum_render_type/Text/%(cont)s">Short text</a>
+                  </div>
+                  <div class="view-value small-12 medium-3 columns show-for-medium-up">
+                    <span>annal:Text</span>
+                  </div>
+                  <div class="view-value small-4 medium-3 columns">
+                    <span>S/W version</span>
                   </div>
-                  <div class="view-value small-3 columns"><span>annal:Text</span></div>
-                  <div class="view-value small-6 columns"><span>Address</span></div>
                 </div>
               </div>
             </div>
             """%({'base': TestBasePath, 'cont': cont})
         # log.info(r.content)
         self.assertContains(r, rowdata1, html=True)
+        self.assertContains(r, rowdata2, html=True)
+        self.assertContains(r, rowdata3, html=True)
         # Test context
         self.assertEqual(r.context['title'],            "Collection testcoll")
         self.assertEqual(r.context['coll_id'],          "testcoll")
         self.assertEqual(r.context['type_id'],          "_field")
         self.assertEqual(r.context['continuation_url'], "/xyzzy/")
         list_choices = r.context['list_choices']
-        self.assertEqual(set(list_choices.options),     set(self.initial_list_ids))
+        self.assertEqual(set(list_choices.options),     set(self.list_ids))
         self.assertEqual(list_choices['field_value'],   "Field_list")
         # Fields
         head_fields = context_list_head_fields(r.context)
-        self.assertEqual(len(head_fields), 3)
+        self.assertEqual(len(head_fields), 4)
         # 1st field
-        self.assertEqual(head_fields[0]['field_id'], 'Entity_id')
-        self.assertEqual(head_fields[0]['field_name'], 'entity_id')
-        self.assertEqual(head_fields[0]['field_label'], 'Id')
-        self.assertEqual(head_fields[0]['field_placeholder'], "(entity id)")
+        self.assertEqual(head_fields[0]['field_id'],           'Entity_id')
+        self.assertEqual(head_fields[0]['field_name'],         'entity_id')
+        self.assertEqual(head_fields[0]['field_label'],        'Id')
+        self.assertEqual(head_fields[0]['field_placeholder'],  "(entity id)")
         self.assertEqual(head_fields[0]['field_property_uri'], "annal:id")
-        self.assertEqual(head_fields[0]['field_render_type'], "EntityId")
-        self.assertEqual(head_fields[0]['field_placement'].field, "small-3 columns")
-        self.assertEqual(head_fields[0]['field_value_type'], "annal:Slug")
-        self.assertEqual(head_fields[0]['field_value'], "")
+        self.assertEqual(head_fields[0]['field_render_type'],  "EntityId")
+        self.assertEqual(head_fields[0]['field_value_mode'],   "Value_direct")
+        self.assertEqual(head_fields[0]['field_target_type'],  "annal:Slug")
+        self.assertEqual(head_fields[0]['field_placement'].field, "small-4 medium-3 columns")
+        self.assertEqual(head_fields[0]['field_value'],        "")
         # 2nd field
-        self.assertEqual(head_fields[1]['field_id'], 'Field_type')
-        self.assertEqual(head_fields[1]['field_name'], 'Field_type')
-        self.assertEqual(head_fields[1]['field_label'], 'Field value type')
-        self.assertEqual(head_fields[1]['field_placeholder'], "(field value type)")
-        self.assertEqual(head_fields[1]['field_property_uri'], "annal:field_value_type")
-        self.assertEqual(head_fields[1]['field_render_type'], "Identifier")
-        self.assertEqual(head_fields[1]['field_placement'].field, "small-3 columns")
-        self.assertEqual(head_fields[1]['field_value_type'], "annal:Identifier")
+        self.assertEqual(head_fields[1]['field_id'],           'Field_render')
+        self.assertEqual(head_fields[1]['field_name'],         'Field_render')
+        self.assertEqual(head_fields[1]['field_label'],        'Field render type')
+        self.assertEqual(head_fields[1]['field_placeholder'],  "(field render type)")
+        self.assertEqual(head_fields[1]['field_property_uri'], "annal:field_render_type")
+        self.assertEqual(head_fields[1]['field_render_type'],  "Enum_choice")
+        self.assertEqual(head_fields[1]['field_value_mode'],   "Value_direct")
+        self.assertEqual(head_fields[1]['field_target_type'],  "annal:Slug")
+        self.assertEqual(head_fields[1]['field_placement'].field, "small-4 medium-3 columns")
         # 3rd field
-        self.assertEqual(head_fields[2]['field_id'], 'Entity_label')
-        self.assertEqual(head_fields[2]['field_name'], 'Entity_label')
-        self.assertEqual(head_fields[2]['field_label'], 'Label')
-        self.assertEqual(head_fields[2]['field_placeholder'], "(label)")
-        self.assertEqual(head_fields[2]['field_property_uri'], "rdfs:label")
-        self.assertEqual(head_fields[2]['field_render_type'], "Text")
-        self.assertEqual(head_fields[2]['field_placement'].field, "small-6 columns")
-        self.assertEqual(head_fields[2]['field_value_type'], "annal:Text")
+        self.assertEqual(head_fields[2]['field_id'],           'Field_type')
+        self.assertEqual(head_fields[2]['field_name'],         'Field_type')
+        self.assertEqual(head_fields[2]['field_label'],        'Field value type')
+        self.assertEqual(head_fields[2]['field_placeholder'],  "(field value type)")
+        self.assertEqual(head_fields[2]['field_property_uri'], "annal:field_value_type")
+        self.assertEqual(head_fields[2]['field_render_type'],  "Identifier")
+        self.assertEqual(head_fields[2]['field_value_mode'],   "Value_direct")
+        self.assertEqual(head_fields[2]['field_target_type'],  "annal:Identifier")
+        self.assertEqual(head_fields[2]['field_placement'].field, "small-12 medium-3 columns show-for-medium-up")
+        # 3th field
+        self.assertEqual(head_fields[3]['field_id'],           'Entity_label')
+        self.assertEqual(head_fields[3]['field_name'],         'Entity_label')
+        self.assertEqual(head_fields[3]['field_label'],        'Label')
+        self.assertEqual(head_fields[3]['field_placeholder'],  "(label)")
+        self.assertEqual(head_fields[3]['field_property_uri'], "rdfs:label")
+        self.assertEqual(head_fields[3]['field_render_type'],  "Text")
+        self.assertEqual(head_fields[3]['field_value_mode'],   "Value_direct")
+        self.assertEqual(head_fields[3]['field_target_type'],  "annal:Text")
+        self.assertEqual(head_fields[3]['field_placement'].field, "small-4 medium-3 columns")
         # Entities
         entities = context_list_entities(r.context)
         entity_ids = [ context_list_item_field_value(r.context, e, 0) for e in entities ]
-        self.assertIn('_initial_values', entity_ids)
+        #@@ self.assertIn('_initial_values', entity_ids)
         field_entities = (
-            { ('Entity_id',         "annal:Slug",          "Id")
-            , ('Bib_address',       "annal:Text",          "Address")
-            , ('Bib_authors',       "bib:Authors",         "Author(s)")
-            , ('Bib_booktitle',     "annal:Text",          "Book title")
-            , ('Entity_type',       "annal:Slug",          "Type")
-            , ('Entity_label',      "annal:Text",          "Label")
-            , ('Field_comment',     "annal:Longtext",      "Help")
-            , ('Field_placement',   "annal:Placement",     "Position/size")
-            , ('Field_type',        "annal:Identifier",    "Field value type")
-            , ('Field_render',      "annal:Slug",          "Field render type")
-            , ('Field_default',     "annal:Text",          "Default")
-            , ('Field_typeref',     "annal:Slug",          "Enum type")
-            , ('Field_restrict',    "annal:Text",          "Enum restriction")
-            , ('List_comment',      "annal:Longtext",      "Help")
-            , ('List_default_type', "annal:Type",          "Record type")
-            , ('List_default_view', "annal:View",          "View")
-            , ('Type_label',        "annal:Text",          "Label")
-            , ('Type_comment',      "annal:Longtext",      "Comment")
-            , ('Type_uri',          "annal:Identifier",    "URI")
-            , ('List_choice',       "annal:Slug",          "List view")
-            , ('View_choice',       "annal:Slug",          "Choose view")
-            , ('Group_field_sel',   "annal:Slug",          "Field id")
+            { ('Entity_id',         "EntityId",      "annal:Slug",          "Id")
+            , ( "Coll_comment",     "Textarea",      "annal:Longtext",      "Collection metadata")
+            , ( "Coll_parent",      "Enum_choice",   "annal:Slug",          "Parent")
+            , ( "Coll_software_version", "Text",     "annal:Text",          "S/W version")
+            , ('Entity_type',       "EntityTypeId",  "annal:Slug",          "Type")
+            , ('Entity_label',      "Text",          "annal:Text",          "Label")
+            , ('Field_comment',     "Markdown",      "annal:Richtext",      "Help")
+            , ('Field_placement',   "Placement",     "annal:Placement",     "Position/size")
+            , ('Field_type',        "Identifier",    "annal:Identifier",    "Field value type")
+            , ('Field_render',      "Enum_choice",   "annal:Slug",          "Field render type")
+            , ('Field_default',     "Text",          "annal:Text",          "Default")
+            , ('Field_typeref',     "Enum_optional", "annal:Slug",          "Refer to type")
+            , ('Field_restrict',    "Text",          "annal:Text",          "Value restriction")
+            , ('List_comment',      "Markdown",      "annal:Richtext",      "Help")
+            , ('List_default_type', "Enum_optional", "annal:Type",          "Record type")
+            , ('List_default_view', "Enum_optional", "annal:View",          "View")
+            , ('Type_label',        "Text",          "annal:Text",          "Label")
+            , ('Type_comment',      "Markdown",      "annal:Richtext",      "Comment")
+            , ('Type_uri',          "Identifier",    "annal:Identifier",    "URI")
+            , ('List_choice',       "Enum_choice",   "annal:Slug",          "List view")
+            , ('View_choice',       "View_choice",   "annal:Slug",          "Choose view")
+            , ('Group_field_sel',   "Enum_optional", "annal:Slug",          "Field id")
             })
         for f in field_entities:
             for eid in range(len(entities)):
                 item_fields = context_list_item_fields(r.context, entities[eid])
                 if item_fields[0]['field_value'] == f[0]:
-                    for fid in range(3):
+                    for fid in range(4):
                         item_field = item_fields[fid]
                         head_field = head_fields[fid]
                         for fkey in (
                                 'field_id', 'field_name', 'field_label', 
                                 'field_property_uri', 'field_render_type',
-                                'field_placement', 'field_value_type'):
+                                'field_placement', 'field_target_type'):
                             self.assertEqual(item_field[fkey], head_field[fkey])
                         self.assertEqual(item_field['field_value'], f[fid])
                     break
@@ -430,106 +512,206 @@ class EntityGenericListViewTest(AnnalistTestCase):
         # self.assertContains(r, site_title("<title>%s</title>"))
         # self.assertContains(r, "<h3>List 'Field_list' of entities in collection 'testcoll'</h3>", html=True)
         cont = uri_params({"continuation_url": u})
+        cont = ""
         rowdata1 = """
             <div class="tbody row select-row">
               <div class="small-1 columns">
                 <input type="checkbox" class="select-box right" name="entity_select"
-                       value="_field/Bib_address" />
+                       value="_field/Coll_comment" />
               </div>
               <div class="small-11 columns">
-                <div class="row view-listrow">
-                  <div class="view-value small-3 columns">
-                    <a href="%(base)s/c/testcoll/d/_field/Bib_address/%(cont)s">Bib_address</a>
+                <div class="view-listrow row">
+                  <div class="view-value small-4 medium-3 columns">
+                    <a href="%(base)s/c/testcoll/d/_field/Coll_comment/%(cont)s">Coll_comment</a>
+                  </div>
+                  <div class="view-value small-4 medium-3 columns">
+                    <a href="%(base)s/c/testcoll/d/Enum_render_type/Textarea/%(cont)s">Multiline text</a>
+                  </div>
+                  <div class="view-value small-12 medium-3 columns show-for-medium-up">
+                    <span>annal:Longtext</span>
+                  </div>
+                  <div class="view-value small-4 medium-3 columns">
+                    <span>Collection metadata</span>
+                  </div>
+                </div>
+              </div>
+            </div>
+            """%({'base': TestBasePath, 'cont': cont})
+        rowdata2 = """
+            <div class="tbody row select-row">
+              <div class="small-1 columns">
+                <input type="checkbox" class="select-box right" name="entity_select"
+                       value="_field/Coll_parent" />
+              </div>
+              <div class="small-11 columns">
+                <div class="view-listrow row">
+                  <div class="view-value small-4 medium-3 columns">
+                    <a href="%(base)s/c/testcoll/d/_field/Coll_parent/%(cont)s">Coll_parent</a>
+                  </div>
+                  <div class="view-value small-4 medium-3 columns">
+                    <a href="%(base)s/c/testcoll/d/Enum_render_type/Enum_choice/%(cont)s">Entity choice</a>
+                  </div>
+                  <div class="view-value small-12 medium-3 columns show-for-medium-up">
+                    <span>annal:Slug</span>
+                  </div>
+                  <div class="view-value small-4 medium-3 columns">
+                    <span>Parent</span>
+                  </div>
+                </div>
+              </div>
+            </div>
+            """%({'base': TestBasePath, 'cont': cont})
+        rowdata3 = """
+            <div class="tbody row select-row">
+              <div class="small-1 columns">
+                <input type="checkbox" class="select-box right" name="entity_select"
+                       value="_field/Coll_software_version" />
+              </div>
+              <div class="small-11 columns">
+                <div class="view-listrow row">
+                  <div class="view-value small-4 medium-3 columns">
+                    <a href="%(base)s/c/testcoll/d/_field/Coll_software_version/%(cont)s">Coll_software_version</a>
+                  </div>
+                  <div class="view-value small-4 medium-3 columns">
+                    <a href="%(base)s/c/testcoll/d/Enum_render_type/Text/%(cont)s">Short text</a>
+                  </div>
+                  <div class="view-value small-12 medium-3 columns show-for-medium-up">
+                    <span>annal:Text</span>
+                  </div>
+                  <div class="view-value small-4 medium-3 columns">
+                    <span>S/W version</span>
                   </div>
-                  <div class="view-value small-3 columns"><span>annal:Text</span></div>
-                  <div class="view-value small-6 columns"><span>Address</span></div>
                 </div>
               </div>
             </div>
             """%({'base': TestBasePath, 'cont': cont})
         # log.info("*** r.content: "+r.content)
         self.assertContains(r, rowdata1, html=True)
+        self.assertContains(r, rowdata2, html=True)
+        self.assertContains(r, rowdata3, html=True)
         # Test context
         self.assertEqual(r.context['coll_id'],          "testcoll")
         self.assertEqual(r.context['type_id'],          "_field")
         self.assertEqual(r.context['continuation_url'], "")
         list_choices = r.context['list_choices']
-        self.assertEqual(set(list_choices.options),     set(self.initial_list_ids))
+        self.assertEqual(set(list_choices.options),     set(self.list_ids))
         self.assertEqual(list_choices['field_value'],   "Field_list")
         # Fields
         head_fields = context_list_head_fields(r.context)
-        self.assertEqual(len(head_fields), 3)
+        self.assertEqual(len(head_fields), 4)
         return
 
     def test_get_fields_list_search(self):
         u = entitydata_list_type_url(
             "testcoll", "_field", list_id="Field_list", scope="all"
-            ) + "?search=Bib_&continuation_url=/xyzzy/"
+            ) + "?search=Coll_&continuation_url=/xyzzy/"
         r = self.client.get(u)
         self.assertEqual(r.status_code,   200)
         self.assertEqual(r.reason_phrase, "OK")
         # self.assertContains(r, site_title("<title>%s</title>"))
         # self.assertContains(r, "<h3>List 'Field_list' of entities in collection 'testcoll'</h3>", html=True)
         cont = uri_params({"continuation_url": u})
-        rowdata = """
+        cont = ""
+        rowdata1 = """
             <div class="tbody row select-row">
               <div class="small-1 columns">
                 <input type="checkbox" class="select-box right" name="entity_select"
-                       value="_field/Bib_address" />
+                       value="_field/Coll_comment" />
               </div>
               <div class="small-11 columns">
-                <div class="row view-listrow">
-                  <div class="view-value small-3 columns">
-                    <a href="%(base)s/c/testcoll/d/_field/Bib_address/%(cont)s">Bib_address</a>
+                <div class="view-listrow row">
+                  <div class="view-value small-4 medium-3 columns">
+                    <a href="%(base)s/c/testcoll/d/_field/Coll_comment/%(cont)s">Coll_comment</a>
+                  </div>
+                  <div class="view-value small-4 medium-3 columns">
+                    <a href="%(base)s/c/testcoll/d/Enum_render_type/Textarea/%(cont)s">Multiline text</a>
+                  </div>
+                  <div class="view-value small-12 medium-3 columns show-for-medium-up">
+                    <span>annal:Longtext</span>
+                  </div>
+                  <div class="view-value small-4 medium-3 columns">
+                    <span>Collection metadata</span>
                   </div>
-                  <div class="view-value small-3 columns"><span>annal:Text</span></div>
-                  <div class="view-value small-6 columns"><span>Address</span></div>
                 </div>
               </div>
             </div>
             """%({'base': TestBasePath, 'cont': cont})
+        rowdata2 = """
+            <div class="tbody row select-row">
+              <div class="small-1 columns">
+                <input type="checkbox" class="select-box right" name="entity_select"
+                       value="_field/Coll_parent" />
+              </div>
+              <div class="small-11 columns">
+                <div class="view-listrow row">
+                  <div class="view-value small-4 medium-3 columns">
+                    <a href="%(base)s/c/testcoll/d/_field/Coll_parent/%(cont)s">Coll_parent</a>
+                  </div>
+                  <div class="view-value small-4 medium-3 columns">
+                    <a href="%(base)s/c/testcoll/d/Enum_render_type/Enum_choice/%(cont)s">Entity choice</a>
+                  </div>
+                  <div class="view-value small-12 medium-3 columns show-for-medium-up">
+                    <span>annal:Slug</span>
+                  </div>
+                  <div class="view-value small-4 medium-3 columns">
+                    <span>Parent</span>
+                  </div>
+                </div>
+              </div>
+            </div>
+            """%({'base': TestBasePath, 'cont': cont})
+        rowdata3 = """
+            <div class="tbody row select-row">
+              <div class="small-1 columns">
+                <input type="checkbox" class="select-box right" name="entity_select"
+                       value="_field/Coll_software_version" />
+              </div>
+              <div class="small-11 columns">
+                <div class="view-listrow row">
+                  <div class="view-value small-4 medium-3 columns">
+                    <a href="%(base)s/c/testcoll/d/_field/Coll_software_version/%(cont)s">Coll_software_version</a>
+                  </div>
+                  <div class="view-value small-4 medium-3 columns">
+                    <a href="%(base)s/c/testcoll/d/Enum_render_type/Text/%(cont)s">Short text</a>
+                  </div>
+                  <div class="view-value small-12 medium-3 columns show-for-medium-up">
+                    <span>annal:Text</span>
+                  </div>
+                  <div class="view-value small-4 medium-3 columns">
+                    <span>S/W version</span>
+                  </div>
+                </div>
+              </div>
+            </div>
+            """%({'base': TestBasePath, 'cont': cont})
+
         # log.info(r.content)
         # If this test fails, check ordering of URI parameters
-        self.assertContains(r, rowdata, html=True)
+        self.assertContains(r, rowdata1, html=True)
+        self.assertContains(r, rowdata2, html=True)
+        self.assertContains(r, rowdata3, html=True)
         # Test context
         self.assertEqual(r.context['coll_id'],          "testcoll")
         self.assertEqual(r.context['type_id'],          "_field")
         self.assertEqual(r.context['continuation_url'], "/xyzzy/")
-        self.assertEqual(r.context['search_for'],       "Bib_")
+        self.assertEqual(r.context['search_for'],       "Coll_")
         list_choices = r.context['list_choices']
-        self.assertEqual(set(list_choices.options),     set(self.initial_list_ids))
+        self.assertEqual(set(list_choices.options),     set(self.list_ids))
         self.assertEqual(list_choices['field_value'],   "Field_list")
         # Fields
         head_fields = context_list_head_fields(r.context)
-        self.assertEqual(len(head_fields), 3)
+        self.assertEqual(len(head_fields), 4)
         self.assertEqual(head_fields[0]['field_id'], 'Entity_id')
-        self.assertEqual(head_fields[1]['field_id'], 'Field_type')
-        self.assertEqual(head_fields[2]['field_id'], 'Entity_label')
+        self.assertEqual(head_fields[1]['field_id'], 'Field_render')
+        self.assertEqual(head_fields[2]['field_id'], 'Field_type')
+        self.assertEqual(head_fields[3]['field_id'], 'Entity_label')
         # Entities
         entities = context_list_entities(r.context)
-        self.assertEqual(len(entities), 36)
+        self.assertEqual(len(entities), 3)
         field_entities = (
-            { ('Bib_address',       "annal:Text",          "Address")
-            , ('Bib_authors',       "bib:Authors",         "Author(s)")
-            , ('Bib_booktitle',     "annal:Text",          "Book title")
-            , ('Bib_chapter',       "annal:Text",          "Chapter")
-            , ('Bib_edition',       "annal:Text",          "Edition")
-            , ('Bib_editors',       "bib:Editors",         "Editor(s)")
-            , ('Bib_eprint',        "annal:Text",          "Bib_eprint")
-            , ('Bib_howpublished',  "annal:Text",          "How published")
-            , ('Bib_institution',   "annal:Text",          "Institution")
-            , ('Bib_journal',       "bib:Journal",         "Journal")
-            , ('Bib_month',         "annal:Text",          "Month")
-            , ('Bib_number',        "annal:Text",          "Number")
-            , ('Bib_organization',  "annal:Text",          "Organization")
-            , ('Bib_pages',         "annal:Text",          "Pages")
-            , ('Bib_publisher',     "annal:Text",          "Publisher")
-            , ('Bib_school',        "annal:Text",          "School")
-            , ('Bib_title',         "annal:Text",          "Title")
-            , ('Bib_type',          "annal:Slug",          "Type")
-            , ('Bib_url',           "annal:Text",          "URL")
-            , ('Bib_volume',        "annal:Text",          "Volume")
-            , ('Bib_year',          "annal:Text",          "Year")
+            { ( "Coll_comment",          "Textarea",    "annal:Longtext", "Collection metadata" )
+            , ( "Coll_parent",           "Enum_choice", "annal:Slug",    "Parent"               )
+            , ( "Coll_software_version", "Text",        "annal:Text",    "S/W version"          )
             })
         for f in field_entities:
             for eid in range(len(entities)):
@@ -541,7 +723,7 @@ class EntityGenericListViewTest(AnnalistTestCase):
                         for fkey in (
                                 'field_id', 'field_name', 'field_label', 
                                 'field_property_uri', 'field_render_type',
-                                'field_placement', 'field_value_type'):
+                                'field_placement', 'field_target_type'):
                             self.assertEqual(item_field[fkey], head_field[fkey])
                         self.assertEqual(item_field['field_value'], f[fid])
                     break
@@ -558,7 +740,7 @@ class EntityGenericListViewTest(AnnalistTestCase):
         self.assertEqual(r.context['coll_id'],          "testcoll")
         self.assertEqual(r.context['type_id'],          "_field")
         list_choices = r.context['list_choices']
-        self.assertEqual(set(list_choices.options),     set(self.initial_list_ids))
+        self.assertEqual(set(list_choices.options),     set(self.list_ids))
         self.assertEqual(list_choices['field_value'],   "Field_list")
         return
 
@@ -609,7 +791,8 @@ class EntityGenericListViewTest(AnnalistTestCase):
 
     def test_post_new_all_entity(self):
         # Also tests continuation_url parameter handling
-        s = site_view_url()
+        #@@ s = site_view_url()
+        s = collection_view_url(coll_id="testcoll")
         f = entitylist_form_data("new", list_id="Field_list", continuation_url=s)
         u = entitydata_list_all_url("testcoll", list_id="Field_list")
         r = self.client.post(u, f)
@@ -875,7 +1058,7 @@ class EntityGenericListViewTest(AnnalistTestCase):
         self.assertEqual(r.status_code,   302)
         self.assertEqual(r.reason_phrase, "FOUND")
         self.assertEqual(r.content,       "")
-        v = TestHostUri + site_view_url()
+        v = TestHostUri + collection_view_url(coll_id="testcoll")
         self.assertEqual(v, r['location'])
         return
 
