@@ -34,12 +34,13 @@ from annalist.models.recordfield    import RecordField
 from annalist.models.recordtypedata import RecordTypeData
 from annalist.models.entitydata     import EntityData
 
-from annalist.views.entityedit      import GenericEntityEditView
+from annalist.views.entityedit              import GenericEntityEditView
+from annalist.views.form_utils.fieldchoice  import FieldChoice
 
-from tests                          import TestHost, TestHostUri, TestBasePath, TestBaseUri, TestBaseDir
-from tests                          import init_annalist_test_site
-from AnnalistTestCase               import AnnalistTestCase
-from entity_testutils               import (
+from AnnalistTestCase       import AnnalistTestCase
+from tests                  import TestHost, TestHostUri, TestBasePath, TestBaseUri, TestBaseDir
+from init_tests             import init_annalist_test_site, init_annalist_test_coll, resetSitedata
+from entity_testutils       import (
     collection_create_values,
     site_dir, collection_dir, 
     continuation_url_param,
@@ -50,12 +51,12 @@ from entity_testutils               import (
     render_choice_options,
     create_test_user
     )
-from entity_testtypedata            import (
+from entity_testtypedata    import (
     recordtype_dir, 
     recordtype_edit_url,
     recordtype_create_values, 
     )
-from entity_testentitydata          import (
+from entity_testentitydata  import (
     recorddata_dir,  entitydata_dir,
     entity_url, entitydata_edit_url, 
     entitydata_list_type_url,
@@ -66,15 +67,17 @@ from entity_testentitydata          import (
     default_fields, default_label, default_comment, error_label,
     layout_classes
     )
-from entity_testsitedata            import (
-    get_site_types, get_site_types_sorted,
-    get_site_lists, get_site_lists_sorted,
+from entity_testsitedata    import (
+    get_site_types, get_site_types_sorted, get_site_types_linked,
+    get_site_lists, get_site_lists_sorted, get_site_lists_linked,
+    get_site_views, get_site_views_sorted, get_site_views_linked,
     get_site_list_types, get_site_list_types_sorted,
-    get_site_views, get_site_views_sorted,
     get_site_field_groups, get_site_field_groups_sorted, 
     get_site_fields, get_site_fields_sorted, 
     get_site_field_types, get_site_field_types_sorted, 
     )
+from entity_testviewdata    import recordview_url
+from entity_testlistdata    import recordlist_url
 
 #   -----------------------------------------------------------------------------
 #
@@ -86,20 +89,16 @@ class GenericEntityViewViewTest(AnnalistTestCase):
 
     def setUp(self):
         init_annalist_test_site()
+        init_annalist_test_coll()
         self.testsite = Site(TestBaseUri, TestBaseDir)
         self.testcoll = Collection.create(self.testsite, "testcoll", collection_create_values("testcoll"))
         self.testtype = RecordType.create(self.testcoll, "testtype", recordtype_create_values("testtype"))
         self.testdata = RecordTypeData.create(self.testcoll, "testtype", {})
-        self.type_ids   = ['testtype', 'Default_type']
-        self.no_options = ['(no options)']
-        self.view_options    = sorted(
-            [ vid for vid in self.testcoll.child_entity_ids(RecordView, self.testsite) 
-                  if vid != "_initial_values"
-            ])
-        self.list_options    = sorted(
-            [ lid for lid in self.testcoll.child_entity_ids(RecordList, self.testsite) 
-                  if lid != "_initial_values"
-            ])
+        self.no_options = [ FieldChoice('', label="(no options)") ]
+        self.no_view_id = [ FieldChoice('', label="(view id)") ]
+        self.no_list_id = [ FieldChoice('', label="(list id)") ]
+        self.view_options = get_site_views_linked("testcoll")
+        self.list_options = get_site_lists_linked("testcoll")
         # Login and permissions
         create_test_user(self.testcoll, "testuser", "testpassword")
         self.client = Client(HTTP_HOST=TestHost)
@@ -108,6 +107,12 @@ class GenericEntityViewViewTest(AnnalistTestCase):
         return
 
     def tearDown(self):
+        # resetSitedata()
+        return
+
+    @classmethod
+    def tearDownClass(cls):
+        resetSitedata()
         return
 
     #   -----------------------------------------------------------------------------
@@ -155,7 +160,7 @@ class GenericEntityViewViewTest(AnnalistTestCase):
         self.client.logout()
         f = entitydata_default_view_form_data(
                 entity_id="entityuseview", action="view",
-                use_view="Type_view", 
+                use_view="_view/Type_view", 
                 )
         u = entity_url("testcoll", "testtype", "entity1")
         r = self.client.post(u, f)
@@ -177,23 +182,13 @@ class GenericEntityViewViewTest(AnnalistTestCase):
         self.assertEqual(r.status_code,   200)
         self.assertEqual(r.reason_phrase, "OK")
         self.assertContains(r, "<title>Collection testcoll</title>")
+        cont_uri = "?continuation_url=%s"%u + "%%3Fcontinuation_url=/xyzzy/"
+        cont_uri = ""
         field_vals = default_fields(
             coll_id="testcoll", type_id="testtype", entity_id="00000001",
-            entity_url = (
-                "/testsite/c/testcoll/d/testtype/entity1/" +
-                "?continuation_url=%s"%u +
-                "%3Fcontinuation_url=/xyzzy/"
-                ),
-            default_view_url = (
-                "/testsite/c/testcoll/d/_view/Default_view/" +
-                "?continuation_url=%s"%u +
-                "%3Fcontinuation_url=/xyzzy/"
-                ),
-            default_list_url = (
-                "/testsite/c/testcoll/d/_list/Default_list/" +
-                "?continuation_url=%s"%u +
-                "%3Fcontinuation_url=/xyzzy/"
-                )
+            entity_url       = "/testsite/c/testcoll/d/testtype/entity1/" + cont_uri,
+            default_view_url = "/testsite/c/testcoll/d/_view/Default_view/" + cont_uri,
+            default_list_url = "/testsite/c/testcoll/d/_list/Default_list/" + cont_uri
             )
         formrow1 = """
             <div class="small-12 medium-6 columns">
@@ -226,7 +221,9 @@ class GenericEntityViewViewTest(AnnalistTestCase):
                   <span>Comment</span>
                 </div>
                 <div class="%(input_classes)s">
-                  <span class="textarea">Entity coll testcoll, type testtype, entity entity1</span>
+                  <span class="markdown">
+                    <p>Entity coll testcoll, type testtype, entity entity1</p>
+                  </span>
                 </div>
               </div>
             </div>
@@ -250,7 +247,7 @@ class GenericEntityViewViewTest(AnnalistTestCase):
                   <span>Default view</span>
                 </div>
                 <div class="%(input_classes)s">
-                  <a href="%(default_view_url)s">Default_view</a>
+                  <a href="%(default_view_url)s">Default record view</a>
                 </div>
               </div>
             </div>
@@ -262,7 +259,7 @@ class GenericEntityViewViewTest(AnnalistTestCase):
                   <span>Default list</span>
                 </div>
                 <div class="%(input_classes)s">
-                  <a href="%(default_list_url)s">Default_list</a>
+                  <a href="%(default_list_url)s">List entities</a>
                 </div>
               </div>
             </div>
@@ -280,13 +277,13 @@ class GenericEntityViewViewTest(AnnalistTestCase):
             <div class="%(button_wide_classes)s">
               <div class="row">
                 <div class="%(button_left_classes)s">
-                  <input type="submit" name="close" value="Close" />
-                  <input type="submit" name="copy"  value="Copy" />
                   <input type="submit" name="edit"  value="Edit" />
+                  <input type="submit" name="copy"  value="Copy" />
+                  <input type="submit" name="close" value="Close" />
                 </div>
               </div>
             </div>
-            """%field_vals(width=6)
+            """%field_vals(width=4)
         formrow7c = """
             <div class="%(button_wide_classes)s">
               <div class="row">
@@ -307,8 +304,8 @@ class GenericEntityViewViewTest(AnnalistTestCase):
                   """+
                     render_choice_options(
                       "view_choice",
-                      sorted(get_site_views()),
-                      "Type_view")+
+                      self.view_options,
+                      "_view/Type_view")+
                   """
                   </div>
                   <div class="small-3 columns">
@@ -358,72 +355,92 @@ class GenericEntityViewViewTest(AnnalistTestCase):
         self.assertEqual(r.context['type_id'],          "testtype")
         self.assertEqual(r.context['entity_id'],        "entity1")
         self.assertEqual(r.context['orig_id'],          "entity1")
-        self.assertEqual(r.context['entity_url'],       view_url)
         self.assertEqual(r.context['action'],           "edit")
         # Fields
-        self.assertEqual(len(r.context['fields']), 7)
+        self.assertEqual(len(r.context['fields']), 8)
         # 1st field - Id
-        self.assertEqual(r.context['fields'][0]['field_id'], 'Type_id')
-        self.assertEqual(r.context['fields'][0]['field_name'], 'entity_id')
-        self.assertEqual(r.context['fields'][0]['field_label'], 'Id')
-        self.assertEqual(r.context['fields'][0]['field_placeholder'], "(type id)")
+        self.assertEqual(r.context['fields'][0]['field_id'],           'Type_id')
+        self.assertEqual(r.context['fields'][0]['field_name'],         'entity_id')
+        self.assertEqual(r.context['fields'][0]['field_label'],        'Id')
+        self.assertEqual(r.context['fields'][0]['field_placeholder'],  "(type id)")
         self.assertEqual(r.context['fields'][0]['field_property_uri'], "annal:id")
+        self.assertEqual(r.context['fields'][0]['field_value_mode'],   "Value_direct")
+        self.assertEqual(r.context['fields'][0]['field_target_type'],  "annal:Slug")
         self.assertEqual(r.context['fields'][0]['field_placement'].field, "small-12 medium-6 columns")
-        self.assertEqual(r.context['fields'][0]['field_value_type'], "annal:Slug")
-        self.assertEqual(r.context['fields'][0]['field_value'], "entity1")
-        self.assertEqual(r.context['fields'][0]['options'], self.no_options)
+        self.assertEqual(r.context['fields'][0]['field_value'],        "entity1")
+        self.assertEqual(r.context['fields'][0]['options'],            self.no_options)
         # 2nd field - Label
         type_label_value = (
             "Entity testcoll/testtype/entity1"
             )
-        self.assertEqual(r.context['fields'][1]['field_id'], 'Type_label')
-        self.assertEqual(r.context['fields'][1]['field_name'], 'Type_label')
-        self.assertEqual(r.context['fields'][1]['field_label'], 'Label')
+        self.assertEqual(r.context['fields'][1]['field_id'],           'Type_label')
+        self.assertEqual(r.context['fields'][1]['field_name'],         'Type_label')
+        self.assertEqual(r.context['fields'][1]['field_label'],        'Label')
         self.assertEqual(r.context['fields'][1]['field_property_uri'], "rdfs:label")
+        self.assertEqual(r.context['fields'][1]['field_value_mode'],   "Value_direct")
+        self.assertEqual(r.context['fields'][1]['field_target_type'],  "annal:Text")
         self.assertEqual(r.context['fields'][1]['field_placement'].field, "small-12 columns")
-        self.assertEqual(r.context['fields'][1]['field_value_type'], "annal:Text")
-        self.assertEqual(r.context['fields'][1]['field_value'], type_label_value)
-        self.assertEqual(r.context['fields'][1]['options'], self.no_options)
+        self.assertEqual(r.context['fields'][1]['field_value'],        type_label_value)
+        self.assertEqual(r.context['fields'][1]['options'],            self.no_options)
         # 3rd field - comment
         type_comment_value = (
             "Entity coll testcoll, type testtype, entity entity1"
             )
-        self.assertEqual(r.context['fields'][2]['field_id'], 'Type_comment')
-        self.assertEqual(r.context['fields'][2]['field_name'], 'Type_comment')
-        self.assertEqual(r.context['fields'][2]['field_label'], 'Comment')
+        self.assertEqual(r.context['fields'][2]['field_id'],           'Type_comment')
+        self.assertEqual(r.context['fields'][2]['field_name'],         'Type_comment')
+        self.assertEqual(r.context['fields'][2]['field_label'],        'Comment')
         self.assertEqual(r.context['fields'][2]['field_property_uri'], "rdfs:comment")
+        self.assertEqual(r.context['fields'][2]['field_value_mode'],   "Value_direct")
+        self.assertEqual(r.context['fields'][2]['field_target_type'],  "annal:Richtext")
         self.assertEqual(r.context['fields'][2]['field_placement'].field, "small-12 columns")
-        self.assertEqual(r.context['fields'][2]['field_value_type'], "annal:Longtext")
-        self.assertEqual(r.context['fields'][2]['field_value'], type_comment_value)
-        self.assertEqual(r.context['fields'][2]['options'], self.no_options)
+        self.assertEqual(r.context['fields'][2]['field_value'],        type_comment_value)
+        self.assertEqual(r.context['fields'][2]['options'],            self.no_options)
         # 4th field - URI
         # (NOTE: blank unless explcicit value specified)
-        self.assertEqual(r.context['fields'][3]['field_id'], 'Type_uri')
-        self.assertEqual(r.context['fields'][3]['field_name'], 'Type_uri')
-        self.assertEqual(r.context['fields'][3]['field_label'], 'URI')
+        self.assertEqual(r.context['fields'][3]['field_id'],           'Type_uri')
+        self.assertEqual(r.context['fields'][3]['field_name'],         'Type_uri')
+        self.assertEqual(r.context['fields'][3]['field_label'],        'URI')
         self.assertEqual(r.context['fields'][3]['field_property_uri'], "annal:uri")
+        self.assertEqual(r.context['fields'][3]['field_value_mode'],   "Value_direct")
+        self.assertEqual(r.context['fields'][3]['field_target_type'],  "annal:Identifier")
         self.assertEqual(r.context['fields'][3]['field_placement'].field, "small-12 columns")
-        self.assertEqual(r.context['fields'][3]['field_value_type'], "annal:Identifier")
-        self.assertEqual(r.context['fields'][3]['field_value'], "")
-        self.assertEqual(r.context['fields'][3]['options'], self.no_options)
-        # 5th field - view id
-        self.assertEqual(r.context['fields'][4]['field_id'], 'Type_view')
-        self.assertEqual(r.context['fields'][4]['field_name'], 'Type_view')
-        self.assertEqual(r.context['fields'][4]['field_label'], 'Default view')
-        self.assertEqual(r.context['fields'][4]['field_property_uri'], "annal:type_view")
-        self.assertEqual(r.context['fields'][4]['field_placement'].field, "small-12 medium-6 columns")
-        self.assertEqual(r.context['fields'][4]['field_value_type'], "annal:View")
-        self.assertEqual(r.context['fields'][4]['field_value'], "Default_view")
-        self.assertEqual(r.context['fields'][4]['options'], self.view_options)
-        # 6th field - list id
-        self.assertEqual(r.context['fields'][5]['field_id'], 'Type_list')
-        self.assertEqual(r.context['fields'][5]['field_name'], 'Type_list')
-        self.assertEqual(r.context['fields'][5]['field_label'], 'Default list')
-        self.assertEqual(r.context['fields'][5]['field_property_uri'], "annal:type_list")
+        self.assertEqual(r.context['fields'][3]['field_value'],        "")
+        self.assertEqual(r.context['fields'][3]['options'],            self.no_options)
+        # 5th field - Supertype URIs
+        type_supertype_uris_help = (
+            "References to URIs/CURIEs of supertypes."
+            )
+        self.assertEqual(r.context['fields'][4]['field_id'],          'Type_supertype_uris')
+        self.assertEqual(r.context['fields'][4]['field_name'],        'Type_supertype_uris')
+        self.assertEqual(r.context['fields'][4]['field_label'],       'Supertype URIs')
+        self.assertEqual(r.context['fields'][4]['field_help'],        type_supertype_uris_help)
+        self.assertEqual(r.context['fields'][4]['field_placeholder'], "(Supertype URIs or CURIEs)")
+        self.assertEqual(r.context['fields'][4]['field_property_uri'], "annal:supertype_uris")
+        self.assertEqual(r.context['fields'][4]['field_value_mode'],   "Value_direct")
+        self.assertEqual(r.context['fields'][4]['field_target_type'],  "annal:Type_supertype_uri")
+        self.assertEqual(r.context['fields'][4]['field_placement'].field, "small-12 columns")
+        self.assertEqual(r.context['fields'][4]['field_value'],        "") #@@
+        self.assertEqual(r.context['fields'][4]['options'],            self.no_options)
+        # 6th field - view id
+        self.assertEqual(r.context['fields'][5]['field_id'],           'Type_view')
+        self.assertEqual(r.context['fields'][5]['field_name'],         'Type_view')
+        self.assertEqual(r.context['fields'][5]['field_label'],        'Default view')
+        self.assertEqual(r.context['fields'][5]['field_property_uri'], "annal:type_view")
+        self.assertEqual(r.context['fields'][5]['field_value_mode'],   "Value_direct")
+        self.assertEqual(r.context['fields'][5]['field_target_type'],  "annal:View")
         self.assertEqual(r.context['fields'][5]['field_placement'].field, "small-12 medium-6 columns")
-        self.assertEqual(r.context['fields'][5]['field_value_type'], "annal:List")
-        self.assertEqual(r.context['fields'][5]['field_value'], "Default_list")
-        self.assertEqual(r.context['fields'][5]['options'], self.list_options)
+        self.assertEqual(r.context['fields'][5]['field_value'],        "Default_view")
+        self.assertEqual(r.context['fields'][5]['options'],            self.no_view_id + self.view_options)
+        # 7th field - list id
+        self.assertEqual(r.context['fields'][6]['field_id'],           'Type_list')
+        self.assertEqual(r.context['fields'][6]['field_name'],         'Type_list')
+        self.assertEqual(r.context['fields'][6]['field_label'],        'Default list')
+        self.assertEqual(r.context['fields'][6]['field_property_uri'], "annal:type_list")
+        self.assertEqual(r.context['fields'][6]['field_value_mode'],   "Value_direct")
+        self.assertEqual(r.context['fields'][6]['field_target_type'],  "annal:List")
+        self.assertEqual(r.context['fields'][6]['field_placement'].field, "small-12 medium-6 columns")
+        self.assertEqual(r.context['fields'][6]['field_value'],        "Default_list")
+        self.assertEqual(r.context['fields'][6]['options'],            self.no_list_id + self.list_options)
         return
 
     def test_get_view_no_collection(self):
@@ -461,7 +478,7 @@ class GenericEntityViewViewTest(AnnalistTestCase):
         self.assertContains(r, "<h3>404: Not found</h3>", status_code=404)
         # log.debug(r.content)
         err_label = error_label("testcoll", "testtype", "entitynone")
-        self.assertContains(r, "<p>%s does not exist</p>"%err_label, status_code=404)
+        self.assertContains(r, "<p>Entity %s does not exist</p>"%err_label, status_code=404)
         return
 
     #   -----------------------------------------------------------------------------
