@@ -31,12 +31,13 @@ from annalist.models.sitedata           import SiteData
 from annalist.models.collection         import Collection
 from annalist.models.recordlist         import RecordList
 
-from annalist.views.recordlistdelete    import RecordListDeleteConfirmedView
+from annalist.views.recordlistdelete        import RecordListDeleteConfirmedView
+from annalist.views.form_utils.fieldchoice  import FieldChoice
 
-from tests                              import TestHost, TestHostUri, TestBasePath, TestBaseUri, TestBaseDir
-from tests                              import init_annalist_test_site
-from AnnalistTestCase                   import AnnalistTestCase
-from entity_testutils                   import (
+from AnnalistTestCase       import AnnalistTestCase
+from tests                  import TestHost, TestHostUri, TestBasePath, TestBaseUri, TestBaseDir
+from init_tests             import init_annalist_test_site, init_annalist_test_coll, resetSitedata
+from entity_testutils       import (
     site_dir, collection_dir,
     site_view_url, collection_edit_url, 
     collection_entity_view_url,
@@ -45,7 +46,7 @@ from entity_testutils                   import (
     render_choice_options,
     create_test_user
     )
-from entity_testlistdata                import (
+from entity_testlistdata    import (
     recordlist_dir,
     recordlist_coll_url, recordlist_site_url, recordlist_url, recordlist_edit_url,
     recordlist_value_keys, recordlist_load_keys, 
@@ -54,16 +55,17 @@ from entity_testlistdata                import (
     recordlist_view_context_data, recordlist_view_form_data, 
     recordlist_delete_confirm_form_data
     )
-from entity_testentitydata              import (
+from entity_testentitydata  import (
     entity_url, entitydata_edit_url, entitydata_list_type_url,
     default_fields, default_label, default_comment, error_label,
     layout_classes
     )
-from entity_testsitedata            import (
-    get_site_types, get_site_types_sorted,
-    get_site_lists, get_site_lists_sorted,
+from entity_testsitedata    import (
+    make_field_choices, no_selection,
+    get_site_types, get_site_types_sorted, get_site_types_linked,
+    get_site_lists, get_site_lists_sorted, get_site_lists_linked,
+    get_site_views, get_site_views_sorted, get_site_views_linked,
     get_site_list_types, get_site_list_types_sorted,
-    get_site_views, get_site_views_sorted,
     get_site_field_groups, get_site_field_groups_sorted, 
     get_site_fields, get_site_fields_sorted, 
     get_site_field_types, get_site_field_types_sorted, 
@@ -82,6 +84,7 @@ class RecordListTest(AnnalistTestCase):
 
     def setUp(self):
         init_annalist_test_site()
+        init_annalist_test_coll()
         self.testsite = Site(TestBaseUri, TestBaseDir)
         self.sitedata = SiteData(self.testsite)
         self.testcoll = Collection(self.testsite, "testcoll")
@@ -90,12 +93,17 @@ class RecordListTest(AnnalistTestCase):
     def tearDown(self):
         return
 
+    @classmethod
+    def tearDownClass(cls):
+        resetSitedata()
+        return
+
     def test_RecordListTest(self):
         self.assertEqual(Collection.__name__, "Collection", "Check Collection class name")
         return
 
     def test_recordlist_init(self):
-        t = RecordList(self.testcoll, "testlist", self.testsite)
+        t = RecordList(self.testcoll, "testlist")
         u = recordlist_coll_url(self.testsite, coll_id="testcoll", list_id="testlist")
         self.assertEqual(t._entitytype,     ANNAL.CURIE.List)
         self.assertEqual(t._entityfile,     layout.LIST_META_FILE)
@@ -107,7 +115,7 @@ class RecordListTest(AnnalistTestCase):
         return
 
     def test_recordlist1_data(self):
-        t = RecordList(self.testcoll, "list1", self.testsite)
+        t = RecordList(self.testcoll, "list1")
         self.assertEqual(t.get_id(), "list1")
         self.assertEqual(t.get_type_id(), "_list")
         self.assertIn("/c/testcoll/_annalist_collection/lists/list1/", t.get_url())
@@ -120,7 +128,7 @@ class RecordListTest(AnnalistTestCase):
         return
 
     def test_recordlist2_data(self):
-        t = RecordList(self.testcoll, "list2", self.testsite)
+        t = RecordList(self.testcoll, "list2")
         self.assertEqual(t.get_id(), "list2")
         self.assertEqual(t.get_type_id(), "_list")
         self.assertIn("/c/testcoll/_annalist_collection/lists/list2/", t.get_url())
@@ -141,9 +149,10 @@ class RecordListTest(AnnalistTestCase):
         return
 
     def test_recordlist_default_data(self):
-        t = RecordList.load(self.testcoll, "Default_list", altparent=self.testsite)
+        t = RecordList.load(self.testcoll, "Default_list", altscope="all")
         self.assertEqual(t.get_id(), "Default_list")
-        self.assertIn("/c/testcoll/_annalist_collection/lists/Default_list", t.get_url())
+        self.assertIn("/c/_annalist_site/_annalist_collection/lists/Default_list", t.get_url())
+        self.assertIn("/c/testcoll/d/_list/Default_list", t.get_view_url())
         self.assertEqual(t.get_type_id(), "_list")
         td = t.get_values()
         self.assertEqual(set(td.keys()), set(recordlist_load_keys(list_uri=True)))
@@ -170,10 +179,20 @@ class RecordListEditViewTest(AnnalistTestCase):
 
     def setUp(self):
         init_annalist_test_site()
-        self.testsite = Site(TestBaseUri, TestBaseDir)
-        self.testcoll = Collection.create(self.testsite, "testcoll", collection_create_values("testcoll"))
-        self.no_options = ['(no options)']
+        init_annalist_test_coll()
+        self.testsite   = Site(TestBaseUri, TestBaseDir)
+        self.testcoll   = Collection.create(self.testsite, "testcoll", collection_create_values("testcoll"))
         self.continuation_url = TestHostUri + entitydata_list_type_url(coll_id="testcoll", type_id="_list")
+        self.no_options = [ FieldChoice('', label="(no options)") ]
+        self.type_options   = get_site_types_linked("testcoll")
+        self.type_options.append(
+            FieldChoice("_type/testtype", 
+                label="RecordType testcoll/testtype", 
+                link=entity_url("testcoll", "_type", "testtype")
+            ))
+        self.view_options   = get_site_views_linked("testcoll")
+        self.list_options   = get_site_lists_linked("testcoll")
+        self.list_type_opts = get_site_list_types_sorted()
         # Login and permissions
         create_test_user(self.testcoll, "testuser", "testpassword")
         self.client = Client(HTTP_HOST=TestHost)
@@ -182,6 +201,12 @@ class RecordListEditViewTest(AnnalistTestCase):
         return
 
     def tearDown(self):
+        resetSitedata(scope="collections")
+        return
+
+    @classmethod
+    def tearDownClass(cls):
+        resetSitedata()
         return
 
     #   -----------------------------------------------------------------------------
@@ -215,9 +240,9 @@ class RecordListEditViewTest(AnnalistTestCase):
             list_help="(?list_help)",
             list_url="(?list_url)",
             list_uri="(?list_uri)",
-            list_type="List",
-            list_default_type="Default_type",
-            list_default_view="Default_view",
+            list_type="Enum_list_type/List",
+            list_default_type="_type/Default_type",
+            list_default_view="_view/Default_view",
             list_selector="ALL",
             list_target_type=""
             ):
@@ -229,7 +254,6 @@ class RecordListEditViewTest(AnnalistTestCase):
         self.assertEqual(r.context['type_id'],          '_list')
         self.assertEqual(r.context['orig_type'],        '_list')
         self.assertEqual(r.context['coll_id'],          'testcoll')
-        self.assertEqual(r.context['entity_url'],       list_url)
         self.assertEqual(r.context['entity_uri'],       list_uri)
         self.assertEqual(r.context['action'],           action)
         self.assertEqual(r.context['view_id'],          'List_view')
@@ -276,22 +300,41 @@ class RecordListEditViewTest(AnnalistTestCase):
         self.assertEqual(r.context['fields'][7]['field_label'], 'Record type URI')
         self.assertEqual(r.context['fields'][7]['field_value'], list_target_type)
         # 9th field - list of fields from target entity for each list entry
-        expect_field_data = (
-            [ { 'annal:field_id':             "Entity_id"
-              , 'annal:field_placement':      "small:0,3"
-              }
-            , { 'annal:field_id':             "Entity_label"
-              , 'annal:field_placement':      "small:3,9"
-              }
-            ])
-        self.assertEqual(r.context['fields'][8]['field_id'], 'List_repeat_fields')
-        self.assertEqual(r.context['fields'][8]['field_name'], 'List_repeat_fields')
-        self.assertEqual(r.context['fields'][8]['field_label'], 'Fields')
+        if num_fields == 2:
+            expect_field_data = (
+                [ { "annal:field_id":         "_field/Entity_id" 
+                  , "annal:field_placement":  "small:0,3"
+                  } 
+                , { "annal:field_id":         "_field/Entity_label" 
+                  , "annal:field_placement":  "small:3,9"
+                  } 
+                ])
+        elif num_fields == 3:
+            expect_field_data = (
+                [ { "annal:field_id":         "_field/Entity_id" 
+                  , "annal:field_placement":  "small:0,3"
+                  } 
+                , { "annal:field_id":         "_field/Entity_type" 
+                  , "annal:field_placement":  "small:3,3"
+                  } 
+                , { "annal:field_id":         "_field/Entity_label" 
+                  , "annal:field_placement":  "small:6,6"
+                  } 
+                ])
+        else:
+            self.fail(
+                "_check_list_view_context_fields expect num_fields 2 or 3, got %d"
+                %(num_fields,)
+                )
+        self.assertEqual(r.context['fields'][8]['field_id'],           'List_fields')
+        self.assertEqual(r.context['fields'][8]['field_name'],         'List_fields')
+        self.assertEqual(r.context['fields'][8]['field_label'],        'Fields')
         self.assertEqual(r.context['fields'][8]['field_property_uri'], "annal:list_fields")
-        self.assertEqual(r.context['fields'][8]['field_value_type'], "annal:Field_group")
-        self.assertEqual(len(r.context['fields'][8]['field_value']), 2)
-        self.assertEqual(r.context['fields'][8]['field_value'], expect_field_data)
-        self.assertEqual(r.context['fields'][8]['options'], self.no_options)
+        self.assertEqual(r.context['fields'][8]['field_value_mode'],   "Value_direct")
+        self.assertEqual(r.context['fields'][8]['field_target_type'],  "annal:Field_group")
+        self.assertEqual(len(r.context['fields'][8]['field_value']),   num_fields)
+        self.assertEqual(r.context['fields'][8]['field_value'],        expect_field_data)
+        self.assertEqual(r.context['fields'][8]['options'],            self.no_options)
         return
 
     #   -----------------------------------------------------------------------------
@@ -329,8 +372,8 @@ class RecordListEditViewTest(AnnalistTestCase):
                 """+
                   render_choice_options(
                     "List_type",
-                    get_site_list_types_sorted(),
-                    "List")+
+                    self.list_type_opts,
+                    "Enum_list_type/List")+
                 """
                 </div>
               </div>
@@ -367,7 +410,7 @@ class RecordListEditViewTest(AnnalistTestCase):
             </div>
             """%field_vals(width=12)
         formrow4 = ("""
-            <div class="small-6 columns">
+            <div class="small-12 medium-6 columns">
               <div class="row view-value-row">
                 <div class="%(label_classes)s">
                   <span>Record type</span>
@@ -376,15 +419,17 @@ class RecordListEditViewTest(AnnalistTestCase):
                 """+
                   render_select_options(
                     "List_default_type", "Record type",
-                    get_site_types_sorted()+["testtype"],
-                    "Default_type")+
+                    no_selection("(default record type)") + self.type_options,
+                    "_type/Default_type",
+                    placeholder="(default record type)"
+                    )+
                 """
                 </div>
               </div>
             </div>
             """)%field_vals(width=6)
         formrow5 = ("""
-            <div class="small-6 columns">
+            <div class="small-12 medium-6 columns">
               <div class="row view-value-row">
                 <div class="%(label_classes)s">
                   <span>View</span>
@@ -393,8 +438,10 @@ class RecordListEditViewTest(AnnalistTestCase):
                 """+
                   render_select_options(
                     "List_default_view", "View",
-                    sorted(get_site_views()),
-                    "Default_view")+
+                    no_selection("(view id)") + self.view_options,
+                    "_view/Default_view",
+                    placeholder="(view id)"
+                    )+
                 """
                 </div>
               </div>
@@ -442,22 +489,19 @@ class RecordListEditViewTest(AnnalistTestCase):
         self.assertEqual(r.context['type_id'],          "_list")
         self.assertEqual(r.context['entity_id'],        "00000001")
         self.assertEqual(r.context['orig_id'],          "00000001")
-        self.assertEqual(r.context['entity_url'],       list_url)
         self.assertEqual(r.context['entity_uri'],       None)
         self.assertEqual(r.context['action'],           "new")
         self.assertEqual(r.context['continuation_url'], "/xyzzy/")
         # Fields
         self._check_list_view_context_fields(r, 
             action="new",
-            num_fields=2,
+            num_fields=3,
             list_id="00000001",
             list_label=default_label("testcoll", "_list", "00000001"),
             list_help=default_comment("testcoll", "_list", "00000001"),
             list_url=list_url,
             list_uri=None,
-            list_type="List",
-            list_default_type="Default_type",
-            list_default_view="Default_view",
+            list_type="Enum_list_type/List",
             list_selector="ALL"
             )
         return
@@ -473,7 +517,6 @@ class RecordListEditViewTest(AnnalistTestCase):
         self.assertEqual(r.context['type_id'],          "_list")
         self.assertEqual(r.context['entity_id'],        "Default_list")
         self.assertEqual(r.context['orig_id'],          "Default_list")
-        self.assertEqual(r.context['entity_url'],       list_url)
         self.assertEqual(r.context['entity_uri'],       None)
         self.assertEqual(r.context['action'],           "copy")
         self.assertEqual(r.context['continuation_url'], "")
@@ -498,7 +541,7 @@ class RecordListEditViewTest(AnnalistTestCase):
         self.assertContains(r, "<title>Annalist error</title>", status_code=404)
         self.assertContains(r, "<h3>404: Not found</h3>", status_code=404)
         err_label = error_label("testcoll", "_list", "nolist")
-        self.assertContains(r, "<p>%s does not exist</p>"%(err_label), status_code=404)
+        self.assertContains(r, "<p>Entity %s does not exist</p>"%(err_label), status_code=404)
         return
 
     def test_get_edit(self):
@@ -515,7 +558,6 @@ class RecordListEditViewTest(AnnalistTestCase):
         self.assertEqual(r.context['type_id'],          "_list")
         self.assertEqual(r.context['entity_id'],        "Default_list")
         self.assertEqual(r.context['orig_id'],          "Default_list")
-        self.assertEqual(r.context['entity_url'],       list_url)
         self.assertEqual(r.context['entity_uri'],       "annal:display/Default_list")
         self.assertEqual(r.context['action'],           "edit")
         self.assertEqual(r.context['continuation_url'], "")
@@ -543,7 +585,7 @@ class RecordListEditViewTest(AnnalistTestCase):
         self.assertContains(r, "<title>Annalist error</title>", status_code=404)
         self.assertContains(r, "<h3>404: Not found</h3>", status_code=404)
         err_label = error_label("testcoll", "_list", "nolist")
-        self.assertContains(r, "<p>%s does not exist</p>"%(err_label), status_code=404)
+        self.assertContains(r, "<p>Entity %s does not exist</p>"%(err_label), status_code=404)
         return
 
     # Test rendering of view with repeated field structure - in this case, List_view
@@ -562,7 +604,6 @@ class RecordListEditViewTest(AnnalistTestCase):
         self.assertEqual(r.context['type_id'],          "_list")
         self.assertEqual(r.context['entity_id'],        "Default_list")
         self.assertEqual(r.context['orig_id'],          "Default_list")
-        self.assertEqual(r.context['entity_url'],       list_url)
         self.assertEqual(r.context['entity_uri'],       "annal:display/Default_list")
         self.assertEqual(r.context['action'],           "edit")
         self.assertEqual(r.context['continuation_url'], "")
