@@ -17,7 +17,9 @@ from django.core.urlresolvers           import resolve, reverse
 from annalist                           import message
 from annalist.exceptions                import Annalist_Error
 from annalist.identifiers               import RDFS, ANNAL
+from annalist.util                      import split_type_entity_id, extract_entity_id
 
+import annalist.models.entitytypeinfo as entitytypeinfo
 from annalist.models.collection         import Collection
 from annalist.models.recordtype         import RecordType
 from annalist.models.recordtypedata     import RecordTypeData
@@ -79,16 +81,19 @@ class EntityGenericListView(AnnalistGenericView):
 
     def __init__(self):
         super(EntityGenericListView, self).__init__()
-        self.help = "entity-list-help"
+        self.help          = "entity-list-help"
         return
 
     # Helper functions
 
-    def list_setup(self, coll_id, type_id, list_id):
+    def list_setup(self, coll_id, type_id, list_id, request_dict):
         """
-        Assemble display information for list view request handler
+        Assemble display information for list view request handlers
         """
-        listinfo = DisplayInfo(self, "list")
+        # print "@@ list_setup coll_id %s, type_id %s, list_id %s"%(coll_id, type_id, list_id)
+        #@@ self.site_view_url       = self.view_uri("AnnalistSiteView")
+        self.collection_view_url = self.view_uri("AnnalistCollectionView", coll_id=coll_id)
+        listinfo = DisplayInfo(self, "list", request_dict, self.collection_view_url)
         listinfo.get_site_info(self.get_request_host())
         listinfo.get_coll_info(coll_id)
         listinfo.get_type_info(type_id)
@@ -120,18 +125,18 @@ class EntityGenericListView(AnnalistGenericView):
         entitymap.add_map_entry(fieldlistmap)  # For access to field headings
 
         repeatrows_field_descr = (
-            { "annal:id":                   "List_rows"
-            , "rdfs:label":                 "Fields"
-            , "rdfs:comment":               "This resource describes the repeated field description used when displaying and/or editing a record view description"
-            , "annal:field_name":           "List_rows"
-            , "annal:field_render_type":    "RepeatListRow"
-            , "annal:property_uri":         "_list_entities_"
-            , "annal:group_ref":            "List_fields"
+            { ANNAL.CURIE.id:                   "List_rows"
+            , RDFS.CURIE.label:                 "Fields"
+            , RDFS.CURIE.comment:               "This resource describes the repeated field description used when displaying and/or editing a record view description"
+            , ANNAL.CURIE.field_name:           "List_rows"
+            , ANNAL.CURIE.field_render_type:    "RepeatListRow"
+            , ANNAL.CURIE.property_uri:         "_list_entities_"
+            , ANNAL.CURIE.group_ref:            "_group/List_fields"
             })
         repeatrows_group_descr = (
-            { "annal:id":           "List_fields"
-            , "rdfs:label":         "List fields description"
-            , "annal:group_fields": listinfo.recordlist[ANNAL.CURIE.list_fields]
+            { ANNAL.CURIE.id:           "List_fields"
+            , RDFS.CURIE.label:         "List fields description"
+            , ANNAL.CURIE.group_fields: listinfo.recordlist[ANNAL.CURIE.list_fields]
             })
         repeatrows_descr = FieldDescription(
             listinfo.collection, 
@@ -152,41 +157,50 @@ class EntityGenericListView(AnnalistGenericView):
             "views.entitylist.get:  coll_id %s, type_id %s, list_id %s, scope %s"%
             (coll_id, type_id, list_id, scope)
             )
-        listinfo = self.list_setup(coll_id, type_id, list_id)
+        listinfo    = self.list_setup(coll_id, type_id, list_id, request.GET.dict())
         if listinfo.http_response:
             return listinfo.http_response
-        log.debug("list_id %s"%listinfo.list_id)
+        log.debug("listinfo.list_id %s"%listinfo.list_id)
         # Prepare list and entity IDs for rendering form
-        selector    = listinfo.recordlist.get_values().get(ANNAL.CURIE.list_entity_selector, "")
-        search_for  = request.GET.get('search', "")
-        user_perms  = self.get_permissions(listinfo.collection)
-        entity_list = (
-            EntityFinder(listinfo.collection, selector=selector)
-                .get_entities_sorted(
-                    user_perms, type_id=type_id, scope=scope,
-                    context=listinfo.recordlist, search=search_for
+        try:
+            selector    = listinfo.recordlist.get_values().get(ANNAL.CURIE.list_entity_selector, "")
+            search_for  = request.GET.get('search', "")
+            user_perms  = self.get_permissions(listinfo.collection)
+            entity_list = (
+                EntityFinder(listinfo.collection, selector=selector)
+                    .get_entities_sorted(
+                        user_perms, type_id=type_id, scope=scope,
+                        context=listinfo.recordlist, search=search_for
+                        )
+                )
+            typeinfo      = listinfo.entitytypeinfo
+            entityvallist = { '_list_entities_': [ get_entity_values(typeinfo, e) for e in entity_list ] }
+            # Set up initial view context
+            context_extra_values = (
+                { 'continuation_url':       listinfo.get_continuation_url() or ""
+                , 'request_url':            self.get_request_path()
+                , 'coll_id':                coll_id
+                , 'type_id':                type_id
+                , 'list_id':                listinfo.list_id
+                , 'search_for':             search_for
+                , 'list_choices':           self.get_list_choices_field(listinfo)
+                , 'collection_view':        self.collection_view_url
+                , 'default_view_id':        listinfo.recordlist[ANNAL.CURIE.default_view]
+                , 'default_view_enable':    ("" if list_id else 'disabled="disabled"')
+                })
+            entityvaluemap = self.get_list_entityvaluemap(listinfo, context_extra_values)
+            listcontext = entityvaluemap.map_value_to_context(
+                entityvallist,
+                **context_extra_values
+                )
+            listcontext.update(listinfo.context_data())
+        except Exception as e:
+            log.exception(str(e))
+            return self.error(
+                dict(self.error500values(),
+                    message=str(e)+" - see server log for details"
                     )
-            )
-        entityvallist = { '_list_entities_': [ get_entity_values(listinfo, e) for e in entity_list ] }
-        # Set up initial view context
-        context_extra_values = (
-            { 'continuation_url':       request.GET.get('continuation_url', "")
-            , 'request_url':            self.get_request_path()
-            , 'coll_id':                coll_id
-            , 'type_id':                type_id
-            , 'list_id':                listinfo.list_id
-            , 'search_for':             search_for
-            , 'list_choices':           self.get_list_choices_field(listinfo)
-            , 'collection_view':        self.view_uri("AnnalistCollectionView", coll_id=coll_id)
-            , 'default_view_id':        listinfo.recordlist[ANNAL.CURIE.default_view]
-            , 'default_view_enable':    ("" if list_id else 'disabled="disabled"')
-            })
-        entityvaluemap = self.get_list_entityvaluemap(listinfo, context_extra_values)
-        listcontext = entityvaluemap.map_value_to_context(
-            entityvallist,
-            **context_extra_values
-            )
-        listcontext.update(listinfo.context_data())
+                )
         # log.debug("EntityGenericListView.get listcontext %r"%(listcontext))
         # Generate and return form data
         return (
@@ -203,21 +217,11 @@ class EntityGenericListView(AnnalistGenericView):
         log.info("views.entitylist.post: coll_id %s, type_id %s, list_id %s"%(coll_id, type_id, list_id))
         # log.info("  %s"%(self.get_request_path()))
         # log.info("  form data %r"%(request.POST))
-        continuation_next, continuation_here = self.continuation_urls(
-            request.POST,
-            None
-            # self.view_uri("AnnalistSiteView")
-            # self.view_uri("AnnalistCollectionEditView", coll_id=coll_id)
-            )
-        if 'close' in request.POST:
-            return HttpResponseRedirect(
-                continuation_next.get('continuation_url', self.view_uri("AnnalistSiteView"))
-                )
-
-        # Not "Close": set up list parameters
-        listinfo = self.list_setup(coll_id, type_id, list_id)
+        listinfo = self.list_setup(coll_id, type_id, list_id, request.POST.dict())
         if listinfo.http_response:
             return listinfo.http_response
+        if 'close' in request.POST:
+            return HttpResponseRedirect(listinfo.get_continuation_url() or self.collection_view_url)
 
         # Process requested action
         redirect_uri = None
@@ -225,31 +229,34 @@ class EntityGenericListView(AnnalistGenericView):
         log.debug("entity_ids %r"%(entity_ids))
         if len(entity_ids) > 1:
             action = ""
-            redirect_uri = self.check_value_supplied(None, message.TOO_MANY_ENTITIES_SEL)
-        else:
-            (entity_type, entity_id) = (
-                entity_ids[0].split("/") if len(entity_ids) == 1 else (None, None)
+            redirect_uri = self.check_value_supplied(
+                None, message.TOO_MANY_ENTITIES_SEL,
+                continuation_url=listinfo.get_continuation_url()
                 )
-            entity_type = entity_type or type_id or listinfo.get_list_type_id()
+        else:
+            entity_type = type_id or listinfo.get_list_type_id()
+            entity_id   = None
+            if len(entity_ids) == 1:
+                (entity_type, entity_id) = split_type_entity_id(entity_ids[0], entity_type)
             if "new" in request.POST:
                 action = "new"
                 redirect_uri = uri_with_params(
                     listinfo.get_new_view_uri(coll_id, entity_type), 
-                    continuation_here
+                    {'continuation_url': listinfo.get_continuation_here()}
                     )
             if "copy" in request.POST:
                 action = "copy"
                 redirect_uri = (
                     self.check_value_supplied(entity_id, 
                         message.NO_ENTITY_FOR_COPY, 
-                        continuation_url=continuation_next
+                        continuation_url=listinfo.get_continuation_url()
                         )
                     or
                     uri_with_params(
                         listinfo.get_edit_view_uri(
                             coll_id, entity_type, entity_id, action
                             ),
-                        continuation_here
+                        {'continuation_url': listinfo.get_continuation_here()}
                         )
                     )
             if "edit" in request.POST:
@@ -257,14 +264,14 @@ class EntityGenericListView(AnnalistGenericView):
                 redirect_uri = (
                     self.check_value_supplied(entity_id, 
                         message.NO_ENTITY_FOR_EDIT,
-                        continuation_url=continuation_next
+                        continuation_url=listinfo.get_continuation_url()
                         )
                     or
                     uri_with_params(
                         listinfo.get_edit_view_uri(
                             coll_id, entity_type, entity_id, action
                             ),
-                        continuation_here
+                        {'continuation_url': listinfo.get_continuation_here()}
                         )
                     )
             if "delete" in request.POST:
@@ -272,18 +279,16 @@ class EntityGenericListView(AnnalistGenericView):
                 redirect_uri = (
                     self.check_value_supplied(entity_id, 
                         message.NO_ENTITY_FOR_DELETE,
-                        continuation_url=continuation_next
+                        continuation_url=listinfo.get_continuation_url()
                         )
                     or
                     listinfo.check_collection_entity(entity_id, entity_type,
-                        message.SITE_ENTITY_FOR_DELETE%{'id': entity_id},
-                        continuation_url=continuation_next
+                        message.SITE_ENTITY_FOR_DELETE%{'id': entity_id}
                         )
                     or
                     self.check_delete_type_values(listinfo,
                         entity_id, entity_type,
-                        message.TYPE_VALUES_FOR_DELETE%{'type_id': entity_id},
-                        continuation_url=continuation_next
+                        message.TYPE_VALUES_FOR_DELETE%{'type_id': entity_id}
                         )
                     )
                 if not redirect_uri:
@@ -296,10 +301,12 @@ class EntityGenericListView(AnnalistGenericView):
                     delete_params = dict_querydict(
                         { "entity_delete":      ["Delete"]
                         , "entity_id":          [entity_id]
-                        , "completion_url":     [continuation_here['continuation_url']]
-                        , "continuation_url":   [continuation_next.get('continuation_url')]
+                        , "completion_url":     [listinfo.get_continuation_here()]
                         , "search_for":         [request.POST['search_for']]
                         })
+                    curi = listinfo.get_continuation_url()
+                    if curi:
+                        dict_querydict["continuation_url"] = [curi]
                     message_vals = {'id': entity_id, 'type_id': entity_type, 'coll_id': coll_id}
                     typeinfo = listinfo.entitytypeinfo
                     if typeinfo is None:
@@ -312,7 +319,7 @@ class EntityGenericListView(AnnalistGenericView):
                             action_description=     message.REMOVE_ENTITY_DATA%message_vals,
                             confirmed_action_uri=   confirmed_action_uri,
                             action_params=          delete_params,
-                            cancel_action_uri=      self.get_request_path(),
+                            cancel_action_uri=      listinfo.get_continuation_here(),
                             title=                  self.site_data()["title"]
                             )
                         )
@@ -331,25 +338,22 @@ class EntityGenericListView(AnnalistGenericView):
                     uri_with_params(
                         self.get_request_path(), 
                         self.info_params(msg),
-                        continuation_next
+                        listinfo.get_continuation_url_dict()
                         )
                     )
             if ("view" in request.POST) or ("view_all" in request.POST):
                 action = "list"         
                 search = request.POST['search_for']
-                params = continuation_next
+                params = listinfo.get_continuation_url_dict()
                 if search:
                     params = dict(params, search=search)
+                post_list_id = extract_entity_id(request.POST['list_choice'])
                 list_uri_params = (
                     { 'coll_id': coll_id
-                    , 'list_id': request.POST['list_choice']
+                    , 'list_id': post_list_id
                     })
                 if "view_all" in request.POST:
                     list_uri_params['scope']  = "all"
-                #@@
-                # if type_id:
-                #     list_uri_params.update({'type_id': type_id})
-                #@@
                 redirect_uri = (
                     uri_with_params(
                         self.view_uri("AnnalistEntityGenericList", **list_uri_params),
@@ -364,7 +368,7 @@ class EntityGenericListView(AnnalistGenericView):
                             "AnnalistCollectionEditView", 
                             coll_id=coll_id
                             ),
-                        continuation_here
+                        {'continuation_url': listinfo.get_continuation_here()}
                         )
                     )
         if redirect_uri:
@@ -380,7 +384,7 @@ class EntityGenericListView(AnnalistGenericView):
             message.UNEXPECTED_FORM_DATA%(request.POST), 
             message.SYSTEM_ERROR
             )
-        redirect_uri = uri_with_params(continuation_next['continuation_url'], err_values)
+        redirect_uri = uri_with_params(listinfo.get_continuation_next(), err_values)
         return HttpResponseRedirect(redirect_uri)
 
     def get_list_choices_field(self, listinfo):
@@ -395,12 +399,10 @@ class EntityGenericListView(AnnalistGenericView):
             , ANNAL.CURIE.field_placement: "small:0,12;medium:5,5" },
             {}
             )
-        entityvals        = { field_description['field_property_uri']: listinfo.list_id }
+        entityvals = { field_description['field_property_uri']: listinfo.list_id }
         return bound_field(field_description, entityvals)
 
-    def check_delete_type_values(
-            self, listinfo, entity_id, entity_type, msg, continuation_url=None
-            ):
+    def check_delete_type_values(self, listinfo, entity_id, entity_type, msg):
         """
         Checks for attempt to delete type with existing values
 
@@ -410,13 +412,13 @@ class EntityGenericListView(AnnalistGenericView):
             typeinfo = EntityTypeInfo(
                 listinfo.site, listinfo.collection, entity_id
                 )
-            if next(typeinfo.enum_entities(), None) is not None:
+            if next(typeinfo.enum_entity_ids(), None) is not None:
                 return (
-                    # Type has valu7es: redisplay form with error message
+                    # Type has values: redisplay form with error message
                     uri_with_params(
                         listinfo.view.get_request_path(),
                         listinfo.view.error_params(msg),
-                        continuation_url
+                        listinfo.get_continuation_url_dict()
                         )
                     )
         return None
