@@ -6,30 +6,39 @@ __author__      = "Graham Klyne (GK@ACM.ORG)"
 __copyright__   = "Copyright 2014, G. Klyne"
 __license__     = "MIT (http://opensource.org/licenses/MIT)"
 
+import sys
+import traceback
 import collections
 
 import logging
 log = logging.getLogger(__name__)
 
-from annalist.identifiers               import RDFS, ANNAL
-from annalist.exceptions                import EntityNotFound_Error
+from annalist.identifiers   import RDFS, ANNAL
+from annalist.exceptions    import Annalist_Error, EntityNotFound_Error
+from annalist.util          import extract_entity_id
 
-from annalist.models.recordgroup        import RecordGroup
-from annalist.models.recordfield        import RecordField
-from annalist.models.entitytypeinfo     import EntityTypeInfo
-from annalist.models.entityfinder       import EntityFinder
+from annalist.models.recordgroup            import RecordGroup
+from annalist.models.recordfield            import RecordField
+from annalist.models.entitytypeinfo         import EntityTypeInfo
+from annalist.models.entityfinder           import EntityFinder
 
-from annalist.views.fields.render_utils import (
+from annalist.views.fields.render_utils     import (
     get_view_renderer,
     get_edit_renderer, 
-    get_colhead_renderer, 
-    get_colview_renderer,
-    get_coledit_renderer,
+    get_label_view_renderer,
+    get_label_edit_renderer, 
+    get_col_head_renderer, 
+    get_col_head_view_renderer, 
+    get_col_head_edit_renderer, 
+    get_col_view_renderer,
+    get_col_edit_renderer,
+    get_mode_renderer,
     get_value_mapper
     )
 from annalist.views.fields.render_placement import (
     get_placement_classes
     )
+from annalist.views.form_utils.fieldchoice  import FieldChoice
 
 class FieldDescription(object):
     """
@@ -39,7 +48,7 @@ class FieldDescription(object):
     def __init__(self, 
             collection, recordfield, view_context=None, 
             field_property=None, field_placement=None, 
-            group_view=None
+            group_view=None, group_ids_seen=[]
             ):
         """
         Creates a field description value to use in a context value when
@@ -61,71 +70,109 @@ class FieldDescription(object):
         group_view      if the field itself references a list of fields, this is a
                         RecordGroup value or dictionary containing the referenced list 
                         of fields.
+        group_ids_seen  group ids expanded so far, to check for recursive reference.
         """
+        self._collection = collection
         # log.debug("FieldDescription recordfield: %r"%(recordfield,))
         field_id            = recordfield.get(ANNAL.CURIE.id,         "_missing_id_")
         field_name          = recordfield.get(ANNAL.CURIE.field_name, field_id)  # Field name in form
         field_label         = recordfield.get(RDFS.CURIE.label, "")
         field_property      = field_property or recordfield.get(ANNAL.CURIE.property_uri, "")
         field_placement     = field_placement or recordfield.get(ANNAL.CURIE.field_placement, "")
-        field_render_type   = recordfield.get(ANNAL.CURIE.field_render_type, "")
+        field_placeholder   = recordfield.get(ANNAL.CURIE.placeholder, "")
+        field_render_type   = extract_entity_id(recordfield.get(ANNAL.CURIE.field_render_type, ""))
+        field_value_mode    = extract_entity_id(recordfield.get(ANNAL.CURIE.field_value_mode, "@@FieldDescription:value_mode@@"))
+        field_ref_type      = extract_entity_id(recordfield.get(ANNAL.CURIE.field_ref_type, None))
+        field_val_type      = recordfield.get(ANNAL.CURIE.field_value_type, "")
+        field_entity_type   = recordfield.get(ANNAL.CURIE.field_entity_type, None)
+        field_group_ref     = extract_entity_id(recordfield.get(ANNAL.CURIE.group_ref, None))
         self._field_desc    = (
             { 'field_id':                   field_id
             , 'field_name':                 field_name
+            , 'field_instance_name':        field_name
+            , 'field_render_type':          field_render_type
+            , 'field_value_mode':           field_value_mode
             , 'field_label':                field_label
             , 'field_help':                 recordfield.get(RDFS.CURIE.comment, "")
             , 'field_property_uri':         field_property
             , 'field_placement':            get_placement_classes(field_placement)
-            , 'field_render_type':          field_render_type
-            , 'field_render_view':          get_view_renderer(field_render_type)
-            , 'field_render_edit':          get_edit_renderer(field_render_type)
-            , 'field_render_colhead':       get_colhead_renderer(field_render_type)
-            , 'field_render_colview':       get_colview_renderer(field_render_type)
-            , 'field_render_coledit':       get_coledit_renderer(field_render_type)
-            , 'field_value_mapper':         get_value_mapper(field_render_type)
-            , 'field_value_type':           recordfield.get(ANNAL.CURIE.field_value_type, "")
-            , 'field_placeholder':          recordfield.get(ANNAL.CURIE.placeholder, "")
+            #@@ , 'field_value_type':           field_val_type
+            #@@TODO: LATER: rename 'field_target_type' to 'field_value_type' when old references are flushed out
+            #@@      See also references to 'field_target_type' in entityedit.py
+            , 'field_target_type':          recordfield.get(ANNAL.CURIE.field_target_type, field_val_type)
+            , 'field_placeholder':          field_placeholder
             , 'field_default_value':        recordfield.get(ANNAL.CURIE.default_value, None)
-            , 'field_options_typeref':      recordfield.get(ANNAL.CURIE.options_typeref, None)
-            , 'field_restrict_values':      recordfield.get(ANNAL.CURIE.restrict_values, "ALL")
-            , 'field_choice_labels':        None
-            , 'field_choice_links':         None
-            , 'field_group_ref':            recordfield.get(ANNAL.CURIE.group_ref, None)
+            , 'field_ref_type':             field_ref_type
+            , 'field_ref_field':            recordfield.get(ANNAL.CURIE.field_ref_field, None)
+            , 'field_ref_restriction':      recordfield.get(ANNAL.CURIE.field_ref_restriction, "ALL")
+            , 'field_entity_type':          field_entity_type
+            , 'field_choices':              None
+            # , 'field_choice_labels':        None
+            # , 'field_choice_links':         None
+            , 'field_group_ref':            field_group_ref
             , 'group_label':                None
             , 'group_add_label':            None
             , 'group_delete_label':         None
             , 'group_view':                 None
             , 'group_field_descs':          None
+            , 'field_render_view':          get_view_renderer(         field_render_type, field_value_mode)
+            , 'field_render_edit':          get_edit_renderer(         field_render_type, field_value_mode)
+            , 'field_render_label_view':    get_label_view_renderer(   field_render_type, field_value_mode)
+            , 'field_render_label_edit':    get_label_edit_renderer(   field_render_type, field_value_mode)
+            , 'field_render_colhead':       get_col_head_renderer(     field_render_type, field_value_mode)
+            , 'field_render_colhead_view':  get_col_head_view_renderer(field_render_type, field_value_mode)
+            , 'field_render_colhead_edit':  get_col_head_edit_renderer(field_render_type, field_value_mode)
+            , 'field_render_colview':       get_col_view_renderer(     field_render_type, field_value_mode)
+            , 'field_render_coledit':       get_col_edit_renderer(     field_render_type, field_value_mode)
+            , 'field_render_mode':          get_mode_renderer(         field_render_type, field_value_mode)
+            , 'field_value_mapper':         get_value_mapper(field_render_type)
             })
-        self._field_suffix_index = 0    # No dup
-        self._field_suffix       = ""
+        self._field_suffix_index  = 0    # No dup
+        self._field_suffix        = ""
         # If field references type, pull in copy of type id and link values
-        type_ref = self._field_desc['field_options_typeref']
+        type_ref = self._field_desc['field_ref_type']
         if type_ref:
-            restrict_values = self._field_desc['field_restrict_values']
+            restrict_values = self._field_desc['field_ref_restriction']
             entity_finder   = EntityFinder(collection, selector=restrict_values)
+            # Determine subtypes of field entity type, if specified
+            # @@TODO: subtype logic here is just ugly...
+            #         need context to convert info that can be used to calculate supertypes
+            #         on-the-fly as needed by the field restriction expression.  E.g. include
+            #         collection object in context.
+            if field_entity_type and restrict_values:
+                field_entity_subtypes = (
+                    [ t.get_type_uri()
+                      for t in entity_finder.get_collection_uri_subtypes(field_entity_type)
+                    ])
+                self._field_desc['field_entity_subtypes'] = field_entity_subtypes
+                field_view_context = dict(view_context or {}, field={'subtypes': field_entity_subtypes})
+            else:
+                field_view_context = view_context
             entities        = entity_finder.get_entities_sorted(
-                type_id=type_ref, context=view_context, scope="all"
+                type_id=type_ref, context=field_view_context, scope="select"
                 )
             # Note: the options list may be used more than once, so the id generator
             # returned must be materialized as a list
             # Uses collections.OrderedfDict to preserve entity ordering
             # 'Enum_optional' adds a blank entry at the start of the list
-            self._field_desc['field_choice_labels'] = collections.OrderedDict()
-            self._field_desc['field_choice_links']  = collections.OrderedDict()
+            self._field_desc['field_choices'] = collections.OrderedDict()
             if field_render_type == "Enum_optional":
-                self._field_desc['field_choice_labels'][''] = ""
-                self._field_desc['field_choice_links']['']  = None
+                self._field_desc['field_choices'][''] = FieldChoice('', label=field_placeholder)
             for e in entities:
                 eid = e.get_id()
+                val = e.get_type_entity_id()
                 if eid != "_initial_values":
-                    self._field_desc['field_choice_labels'][eid] = eid   # @@TODO: be smarter about label?
-                    self._field_desc['field_choice_links'][eid]  = e.get_view_url_path()
-            # log.info("typeref %s: %r"%
-            #     (self._field_desc['field_options_typeref'], list(self._field_desc['field_choices']))
-            #     )
+                    self._field_desc['field_choices'][val] = FieldChoice(
+                        val, label=e.get_label(), link=e.get_view_url_path()
+                        )
+            log.debug("FieldDescription: typeref %s: %r"%
+                (self._field_desc['field_ref_type'], list(self._field_desc['field_choices'].items()))
+                )
         # If field references group, pull in field details
         if group_view:
+            if field_id in group_ids_seen:
+                raise Annalist_Error(field_id, "Recursive field reference in field group")
+            group_ids_seen = group_ids_seen + [field_id]
             group_label = (field_label or 
                 group_view.get(RDFS.CURIE.label, self._field_desc['field_group_ref'])
                 )
@@ -133,7 +180,7 @@ class FieldDescription(object):
             remove_label = recordfield.get(ANNAL.CURIE.repeat_label_delete, None) or "Remove "+field_id
             group_field_descs = []
             for subfield in group_view[ANNAL.CURIE.group_fields]:
-                f = field_description_from_view_field(collection, subfield, view_context)
+                f = field_description_from_view_field(collection, subfield, view_context, group_ids_seen)
                 group_field_descs.append(f)
             self._field_desc.update(
                 { 'group_id':           field_id
@@ -185,6 +232,22 @@ class FieldDescription(object):
         """
         return self._field_desc['field_name']
 
+    def set_field_instance_name(self, instance_name):
+        """
+        Set instance name for field (used when scanning field groups)
+
+        This is the full field instance name including group ids and 
+        indexes.
+        """
+        self._field_desc['field_instance_name'] = instance_name
+        return
+
+    def get_field_instance_name(self):
+        """
+        Get instance name for field (used when scanning field groups)
+        """
+        return self._field_desc['field_instance_name']
+
     def get_field_property_uri(self):
         """
         Returns form field property URI to be used for the described field
@@ -228,7 +291,7 @@ class FieldDescription(object):
     def is_enum_field(self):
         """
         Returns true if this is an enumerated-value field, in which case the
-        'field_options_typeref' field is assumed to the type_id of the value 
+        'field_ref_type' field is assumed to the type_id of the value 
         type to be enumerated.
         """
         enum_render_types = (
@@ -240,22 +303,35 @@ class FieldDescription(object):
 
     def has_new_button(self):
         """
-        Returns true if this field has a control (a 'new' or +' button)')
+        Returns true if this field has a control (a 'new' or '+' button)
         that invokes a new form to create a new entity.
         """
-        new_render_types = (
-            [ "EntityTypeId"
-            , "Type", "View", "List", "Field"
-            , "Enum", "Enum_optional"
-            ])
-        return self._field_desc['field_render_type'] in new_render_types
+        # Strictly, this test includes 'Enum_choice' which does not have a '+' button,
+        # but since the absent button cannot be clicked, the infidelity here is benign
+        return self._field_desc['field_ref_type'] is not None
+
+    def is_import_field(self):
+        """
+        Returns 'true' if this field has a control (an 'import' button) that is used 
+        to request additional external data is added to an entity
+        """
+        return self._field_desc['field_value_mode'] == "Value_import"
+
+    def is_upload_field(self):
+        """
+        Returns 'true' if this field is a file-upload field (selected file contents are 
+        returned with the form response.)
+        """
+        return self._field_desc['field_value_mode'] == "Value_upload"
 
     def has_field_group_ref(self):
         """
         Returns true if this field contains a reference to a field group,
         which in turn references further field descriptions.
 
-        (Currently, this function duplicates `is_repeat_group`.)
+        @@ (Currently, this function duplicates `is_repeat_group`.)
+
+        @@ test for:  group_ref, group_field_descs, and group_id
         """
         field_group_types = ["RepeatGroup", "RepeatGroupRow", "RepeatListRow"]
         return self._field_desc['field_render_type'] in field_group_types
@@ -267,7 +343,7 @@ class FieldDescription(object):
             "  , 'field_name': %r\n"%(self.get_field_name())+
             "  , 'field_render_type': %r\n"%(self._field_desc["field_render_type"])+
             "  , 'field_property_uri': %r\n"%(self.get_field_property_uri())+
-            "  , 'type_ref': %r"%(self._field_desc["field_options_typeref"])+
+            "  , 'type_ref': %r"%(self._field_desc["field_ref_type"])+
             "  , 'group_ref': %r"%(self._field_desc["field_group_ref"])+
             "  })"
             )
@@ -314,13 +390,13 @@ class FieldDescription(object):
             yield k
         return
 
-def field_description_from_view_field(collection, field, view_context=None):
+def field_description_from_view_field(collection, field, view_context=None, group_ids_seen=[]):
     """
     Returns a field description value created using information from
     a field reference in a view description record (i.e. a dictionary
     containing a field id value and optional field property URI and
     placement values.  (The optional values, if not provided, are 
-    obtained from the referenced field descriptionb)
+    obtained from the referenced field description)
 
     collection      is a collection from which data is being rendered.
     field           is a dictionary with the field description from a view or list 
@@ -329,15 +405,16 @@ def field_description_from_view_field(collection, field, view_context=None):
                     values to be used when rendering the field.  In particular, a copy 
                     of the view description record provides context for some enumeration 
                     type selections.
+    group_ids_seen  group ids expanded so far, to check for recursive reference.
     """
     #@@TODO: for resilience, revert this when all tests pass?
     # field_id    = field.get(ANNAL.CURIE.field_id, "Field_id_missing")  # Field ID slug in URI
     #@@
-    field_id    = field[ANNAL.CURIE.field_id]
-    recordfield = RecordField.load(collection, field_id, collection._parentsite)
+    field_id    = extract_entity_id(field[ANNAL.CURIE.field_id])
+    recordfield = RecordField.load(collection, field_id, altscope="all")
     if recordfield is None:
         log.warning("Can't retrieve definition for field %s"%(field_id))
-        recordfield = RecordField.load(collection, "Field_missing", collection._parentsite)
+        recordfield = RecordField.load(collection, "Field_missing", altscope="all")
     field_property  = (
         field.get(ANNAL.CURIE.property_uri, None) or 
         recordfield.get(ANNAL.CURIE.property_uri, "")
@@ -347,18 +424,22 @@ def field_description_from_view_field(collection, field, view_context=None):
         recordfield.get(ANNAL.CURIE.field_placement, "")
         )
     # If field references group, pull in field details
-    group_ref = recordfield.get(ANNAL.CURIE.group_ref, None)
+    group_ref = extract_entity_id(recordfield.get(ANNAL.CURIE.group_ref, None))
     if group_ref:
-        group_view = RecordGroup.load(collection, group_ref, collection._parentsite)
+        group_view = RecordGroup.load(collection, group_ref, altscope="all")
         if not group_view:
-            raise EntityNotFound_Error("Group %s used in field %s"%(group_ref, field_id))
+            log.error("Group %s used in field %s"%(group_ref, field_id))
+            ex_type, ex, tb = sys.exc_info()
+            traceback.print_tb(tb)
+            # raise EntityNotFound_Error("Group %s used in field %s"%(group_ref, field_id))
     else:
         group_view = None
     return FieldDescription(
         collection, recordfield, view_context=view_context, 
         field_property=field.get(ANNAL.CURIE.property_uri, None),
         field_placement=field.get(ANNAL.CURIE.field_placement, None), 
-        group_view=group_view
+        group_view=group_view,
+        group_ids_seen=group_ids_seen
         )
 
 # End.
