@@ -8,13 +8,7 @@ __license__     = "MIT (http://opensource.org/licenses/MIT)"
 
 import os
 import os.path
-import json
-import random
-import uuid
-import copy
-
-import httplib2
-
+import traceback
 import logging
 log = logging.getLogger(__name__)
 
@@ -66,8 +60,7 @@ class AnnalistGenericView(ContentNegotiationView):
         self._sitebasedir = os.path.join(settings.BASE_DATA_DIR, layout.SITE_DIR)
         self._site        = None
         self._site_data   = None
-        self._user_perms  = None
-        ## self.credential = None
+        self._user_perms  = {}
         return
 
     # @@TODO: make host parameter required in the following?
@@ -102,27 +95,23 @@ class AnnalistGenericView(ContentNegotiationView):
         """
         return reverse(viewname, kwargs=kwargs)
 
-    def continuation_urls(self, request_dict={}, default_cont="", base_here=None):
+    def continuation_next(self, request_dict={}, default_cont=None):
         """
-        Returns a tuple of two continuation URI dictionary values:
+        Returns a continuation URL to be used when returning from the current view,
+        or the supplied default if no continuation is specified for the current view.
+        """
+        return request_dict.get("continuation_url") or default_cont or None
 
-        [0] { 'continuation_url': continuation_next }
-        [1] { 'continuation_url': continuation_here }
-
-        where:
-
-        `continuation_next` is the URI to use after the current page has completed
-        processing, which is either supplied as a parameter to the current page or 
-        set to an indicated default.
-
-        `continuation_here` is a URI that returns control to the current page, to be passed
-        as a contionuation_uri parameter to any subsidiary pages invoked.  Such continuation 
-        URIs are cascaded, so that the return URI includes a the `continuation_url` for the 
-        current page.
+    def continuation_here(self, request_dict={}, default_cont="", base_here=None):
+        """
+        Returns a URL that returns control to the current page, to be passed as a
+        contionuation_uri parameter to any subsidiary pages invoked.  Such continuation 
+        URIs are cascaded, so that the return URI includes a the `continuation_url` 
+        for the current page.
 
         request_dict    is a request dictionary that is expected to contain a 
-                        to continuation_url value to use, other parameters to be 
-                        included an any continuation back to the current page.
+                        to continuation_url value to use, and other parameters to 
+                        be included an any continuation back to the current page.
         default_cont    is a default continuation URI to be used for returning from 
                         the current page if the current POST request does not specify
                         a continuation_url query parameter.
@@ -132,17 +121,12 @@ class AnnalistGenericView(ContentNegotiationView):
                         entity).
         """
         # Note: use default if request/form parameter is present but blank:
-        continuation_url    = request_dict.get("continuation_url") or default_cont or None
         if not base_here:
             base_here = self.get_request_path()
-        if continuation_url:
-            continuation_next = { "continuation_url": continuation_url }
-        else:
-            continuation_next = {}
-        continuation_here = uri_with_params(base_here, 
-            continuation_params(continuation_next, request_dict.dict())
+        continuation_next = self.continuation_next(request_dict, default_cont)
+        return uri_with_params(base_here, 
+            continuation_params({"continuation_url": continuation_next}, request_dict)
             )
-        return (continuation_next, {"continuation_url": continuation_here})
 
     def info_params(self, info_message, info_head=message.ACTION_COMPLETED):
         """
@@ -178,7 +162,7 @@ class AnnalistGenericView(ContentNegotiationView):
 
     def check_site_data(self):
         """
-        Check thaty site data is present and accessible.  If not, return an HTTP error
+        Check that site data is present and accessible.  If not, return an HTTP error
         response, otherwise None.
         """
         site_data = self.site_data()
@@ -190,29 +174,33 @@ class AnnalistGenericView(ContentNegotiationView):
                 )
         return None
 
-    def check_value_supplied(self, val, msg, continuation_url={}, testfn=(lambda v: v)):
+    def check_value_supplied(self, val, msg, continuation_url=None, testfn=(lambda v: v)):
         """
-        Test a supplied value is specified (not None) and passes a supplied test,
+        Test if a supplied value is specified (not None) and passes a supplied test,
         returning a URI to display a supplied error message if the test fails.
 
         NOTE: this function works with the generic base template base_generic.html, which
         is assumed to provide an underlay for the currently viewed page.
 
-        val         value that is required to be not None and not empty or False
-        msg         message to display if the value evaluated to False
-        testfn      is a function to test the value (if not None).  If not specified, 
-                    the default test checks that the value does not evaluate as false
-                    (e.g. is a non-empty string, list or collection).
+        val                 value that is required to be not None and not empty or False
+        msg                 message to display if the value evaluated to False
+        continuation_url    a continuation URL for the resdiplayed page.
+        testfn              is a function to test the value (if not None).  If not specified, 
+                            the default test checks that the value does not evaluate as false
+                            (e.g. is a non-empty string, list or collection).
 
         returns a URI string for use with HttpResponseRedirect to redisplay the 
         current page with the supplied message, or None if the value passes the test.
         """
         redirect_uri = None
+        continuation_url_dict = {}
+        if continuation_url:
+            continuation_url_dict = {'continuation_url': continuation_url}
         if (val is None) or not testfn(val):
             redirect_uri = uri_with_params(
                 self.get_request_path(), 
                 self.error_params(msg),
-                continuation_url
+                continuation_url_dict
                 )
         return redirect_uri
 
@@ -272,7 +260,8 @@ class AnnalistGenericView(ContentNegotiationView):
 
         returns an AnnalistUser object containing permissions for the identified user.
         """
-        if not self._user_perms:
+        coll_id = collection.get_id() if collection else layout.SITEDATA_ID
+        if coll_id not in self._user_perms:
             # if collection:
             #     log.info("user_id %s (%s), coll_id %s, %r"%
             #         (user_id, user_uri, collection.get_id(), collection.get_user_permissions(user_id, user_uri))
@@ -282,9 +271,9 @@ class AnnalistGenericView(ContentNegotiationView):
                 self.site().get_user_permissions(user_id, user_uri) or
                 self.site().get_user_permissions("_default_user_perms", "annal:User/_default_user_perms")
                 )
-            self._user_perms = user_perms
-            log.debug("get_user_permissions %r"%(self._user_perms,))
-        return self._user_perms
+            self._user_perms[coll_id] = user_perms
+            # log.debug("get_user_permissions %r"%(self._user_perms,))
+        return self._user_perms[coll_id]
 
     def get_permissions(self, collection):
         """
@@ -313,6 +302,7 @@ class AnnalistGenericView(ContentNegotiationView):
         user_perms = self.get_user_permissions(collection, user_id, user_uri)
         if not user_perms:
             log.warning("No user permissions found for user_id %s, URI %s"%(user_id, user_uri))
+            log.warning("".join(traceback.format_stack()))
             return self.error(self.error403values(scope=scope))
         # log.info("Authorize %s in %s, %s, %r"%(user_id, coll_id, scope, user_perms[ANNAL.CURIE.user_permissions]))
         # user_perms is an AnnalistrUser object
@@ -342,27 +332,65 @@ class AnnalistGenericView(ContentNegotiationView):
         an HTTP response value to return an error condition.
         """
         # @@TODO: in due course, eliminate action_scope.
-        action_scope = (
-            { "view":   "VIEW"      # View data record
-            , "list":   "VIEW"      # ..
-            , "search": "VIEW"      # ..
-            , "new":    "CREATE"    # Create data record
-            , "copy":   "CREATE"    # ..
-            , "edit":   "UPDATE"    # Update data record
-            , "delete": "DELETE"    # Delete datra record
-            , "config": "CONFIG"    # Change collection configuration
-            , "admin":  "ADMIN"     # Change users or permissions
-            })
+        # action_scope = (
+        #     { "view":   "VIEW"      # View data record
+        #     , "list":   "VIEW"      # ..
+        #     , "search": "VIEW"      # ..
+        #     , "new":    "CREATE"    # Create data record
+        #     , "copy":   "CREATE"    # ..
+        #     , "edit":   "UPDATE"    # Update data record
+        #     , "delete": "DELETE"    # Delete datra record
+        #     , "config": "CONFIG"    # Change collection configuration
+        #     , "admin":  "ADMIN"     # Change users or permissions
+        #     })
         if action in perm_required:
             auth_scope = perm_required[action]
-        elif action in action_scope:
-            auth_scope = action_scope[action]
+        # elif action in action_scope:
+        #     auth_scope = action_scope[action]
         else:
             log.warning("form_action_auth: unknown action: %s"%(action))
+            log.warning("perm_required: %r"%(perm_required,))
             auth_scope = "UNKNOWN"
-        # return self.authorize(auth_scope, auth_resource)
         return self.authorize(auth_scope, auth_collection)
 
+    # Entity access
+
+    def get_entity(self, entity_id, typeinfo, action):
+        """
+        Create local entity object or load values from existing.
+
+        entity_id       entity id to create or load
+        typeinfo        EntityTypeInfo object for the entity
+        action          is the requested action: new, edit, copy, view
+
+        returns an object of the appropriate type.
+
+        If an existing entity is accessed, values are read from storage, 
+        otherwise a new entity object is created but not yet saved.
+        """
+        # log.info(
+        #     "get_entity id %s, parent %s, action %s, altparent %s"%
+        #     (entity_id, typeinfo.entityparent, action, typeinfo.entityaltparent)
+        #     )
+        entity = typeinfo.get_entity(entity_id, action)
+        if entity is None:
+            parent_id    = typeinfo.entityparent.get_id()
+            altparent_id = (
+                typeinfo.entityaltparent.get_id() if typeinfo.entityaltparent 
+                else "(none)"
+                )
+            log.info(
+                "Entity not found: parent %s, altparent %s, entity_id %s"%
+                (parent_id, altparent_id, entity_id)
+                )
+            log.info(
+                "get_entity id %s, parent %s, action %s, altparent %s"%
+                (entity_id, typeinfo.entityparent, action, typeinfo.entityaltparent)
+                )
+        return entity
+
+    # HTML rendering
+
     @ContentNegotiationView.accept_types(["text/html", "application/html", "*/*"])
     def render_html(self, resultdata, template_name):
         """
@@ -395,7 +423,10 @@ class AnnalistGenericView(ContentNegotiationView):
         template  = loader.get_template(template_name)
         context   = RequestContext(self.request, resultdata)
         # log.debug("render_html - data: %r"%(resultdata))
-        return HttpResponse(template.render(context))
+        response = HttpResponse(template.render(context))
+        if "entity_data_ref" in resultdata:
+            response["Link"] = "<%(entity_data_ref)s>; rel=alternate"%resultdata
+        return response
 
     # Default view methods return 405 Forbidden
 
